-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_getnotesview.sql
-- SID: 1.1.3.13
-- Date Modified: 04/11/10 13:43:11
--
-- _GetNotesView.sql
--
-- Module Description
--   Provide a search of notes instances, by user name and other filters

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _GetNotesView method
PRINT 'Creating procedure _GetNotesView'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetNotesView' AND type = 'P')
  DROP PROCEDURE _GetNotesView
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_getnotesview.sql
-- SID: 1.1.3.13
-- Date Modified: 04/11/10 13:43:11
--
-- Module Description
--   Provide a search of notes instances, by user name and other filters

CREATE PROCEDURE _GetNotesView
  @cid            ClassID,                 -- not used
  @id             ObjID,                   -- not used
  @User           NVARCHAR(4000) = NULL,   -- userid filter. Note: comparisons on userid are case
                                           -- insensitive. Caller must convert userid(s) to lower case
  @NoteStatus     NVARCHAR(255) = NULL,    -- note status filter
  @NoteType       NVARCHAR(255) = NULL,    -- note type filter
  @Found          INT = NULL OUTPUT,       -- number of notes found, no matter how many returned
  @verbose        TINYINT = 0              -- debugging aid
AS

  IF @verbose >= 1
  BEGIN
    RAISERROR('_GetNotesView @User=<<%s>> @NoteStatus=<<%s>> @NoteType=<<%s>>', 0, 1, @User, @NoteStatus, @NoteType)
  END

  -- Local table for resources whose notes match the specified criteria. Only the cid/id are saved because
  -- the temp table row size is potentially exceeded if all the notes columns are saved in the temp table.
  CREATE TABLE #resources
  (
    noteid               ObjID NOT NULL,
    cid                  ClassID NOT NULL,
    id                   ObjID NOT NULL,
    name                 ObjName NULL,
    _LinkedCID           ClassID NULL DEFAULT NULL
  )

  -- search display limitation
  DECLARE @ReturnLimit INT
  SELECT @ReturnLimit = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration
  WHERE groupname = '_GetNotesView' AND propertyname = 'RETURN_LIMIT'
  IF @ReturnLimit IS NULL OR @ReturnLimit < 0
    SET @ReturnLimit = 5000
  
  IF @verbose >= 1
  BEGIN
    RAISERROR('_GetNotesView: Using return limit %d  ', 0, 1, @ReturnLimit) WITH NOWAIT
  END

  DECLARE @conditions NVARCHAR(4000)

  SET @conditions = ''

  -- User id filter specified?
  IF @User IS NOT NULL 
    SET @User = RTRIM(LTRIM(@User))
  ELSE
    SET @User = ''

  IF @User <> ''
  BEGIN 
    DECLARE @patindex INT
    SELECT @patindex = PATINDEX ( '%||%' , @User )
    IF @patindex IN (0, 1)
    BEGIN 
      IF @patindex = 0  -- IN passed
        SELECT @conditions =  @conditions + " AND LOWER(n._UserID) " + @User
      ELSE 
      BEGIN  -- LIKE passed 
        SELECT @User = SUBSTRING(@User, @patindex+2, LEN(@User)-2)  
        SELECT @conditions =  @conditions + " AND LOWER(n._UserID) " + @User
      END
    END
    ELSE 
    BEGIN   -- IN and LIKE passed
      DECLARE @UserStr1 NVARCHAR(4000), @UserStr2 NVARCHAR(4000)
      SELECT @UserStr1 = SUBSTRING(@User, 1, @patindex-1)
      SELECT @UserStr2 = SUBSTRING(@User, @patindex+2, (LEN(@User)-LEN(@UserStr1)-2))  
      SELECT @conditions = @conditions + " AND (LOWER(n._UserID) " + @UserStr1 + " OR LOWER(n._UserID) " + @UserStr2 + ")"
    END
  END

  -- Note status filter specified?
  IF @NoteStatus IS NOT NULL 
    SET @NoteStatus = RTRIM(LTRIM(@NoteStatus))
  ELSE
    SET @NoteStatus = ''

  IF @NoteStatus <> ''
    SET @conditions = @conditions + " AND n._Status " + @NoteStatus

  -- Note type filter specified?
  IF @NoteType IS NOT NULL 
    SET @NoteType = RTRIM(LTRIM(@NoteType))
  ELSE
    SET @NoteType = ''

  IF @NoteType <> ''
    SET @conditions = @conditions + " AND n._Type " + @NoteType

  IF @verbose >= 1
  BEGIN
    RAISERROR('_GetNotesView: Search conditions = <<%s>>', 0, 1, @conditions)
  END

  -- Select the specified notes into the temporary table
  EXEC("INSERT INTO #resources(noteid, cid, id) SELECT id, _CreatorCID, _CreatorObjID " + 
          "FROM Note_V AS n WHERE deleted = 0" + @conditions)

  -- Now we have to get the name and _LinkedCID using object path cache and lob_link
  UPDATE r SET r.name = o.resource
    FROM #resources AS r  
    JOIN ObjPathCache AS o
    ON r.cid = o.cid AND r.id = o.id

  UPDATE r SET r._LinkedCID = l.phy_cid
    FROM #resources AS r  
    JOIN lob_link AS l
    ON r.cid = 'LOB' AND r.id = l.dst_id AND l.src_id = 0

  IF @verbose >= 2
  BEGIN
    SELECT * FROM #resources
  END

  -- Set output parm
  SELECT @Found = COUNT(*) FROM #resources

  SET ROWCOUNT @ReturnLimit
  -- Return the notes
  EXEC("SELECT cid = 'NOTE', n.id, _UserID, _Subject, _Status, _Body, ctime, _Type, _ProblemID, _Originator, " +
         "r.cid AS _CreatorCID, r.id AS _CreatorObjID, r.name, r._LinkedCID " +
         "FROM Note_V AS n " + 
         "JOIN #resources AS r ON n.id = r.noteid " + 
         "JOIN enum_value AS ev ON ev.name = n._Type " +
         "JOIN enumeration AS e ON e.eid = ev.eid " +
         "WHERE deleted = 0 AND e.name = 'NoteTypes' " + 
         @conditions + 
         " ORDER BY ctime DESC")

GO

PRINT 'Done.'
GO
