-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_getsummarynotes.sql
-- SID: 1.6.1.9
-- Date Modified: 04/11/10 13:43:11
--
-- _GetSummaryNotes.sql
--
-- Module Description
--   Defines the _GetSummaryNotes stored procedure. This procedure is used to provide a list of    
--   notes created for a certain resource or dependent on a certain resource.

USE Object
GO

---------------------------------------------------------------------------------------------------
-- _GetSummaryNotes


PRINT 'Creating procedure _GetSummaryNotes'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetSummaryNotes' AND type = 'P')
  DROP PROCEDURE _GetSummaryNotes
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_getsummarynotes.sql
-- SID: 1.6.1.9
-- Date Modified: 04/11/10 13:43:11
--
-- _GetSummaryNotes.sql
--
-- Module Description
--   Defines the _GetSummaryNotes stored procedure. This procedure is used to provide a list of    
--   notes created for a certain resource or dependent on a certain resource.

CREATE PROCEDURE _GetSummaryNotes
  @cid			ClassID,         -- class ID of resource for notes search
  @id			ObjID,           -- instance ID of resource for notes search
  @note_type  	        sysname = NULL,  -- filter for note type
  @note_status          sysname = NULL,  -- filter for note status
  @note_owner		NVARCHAR(4000) = NULL,  -- filter for note owner. Note: comparisons on userid are case
                                         -- insensitive. Caller must convert userid(s) to lower case
  @problem_status       sysname = NULL,  -- never used; left to match method_param definition
  @problem_assignedto   sysname = NULL,  -- never used; left to match method_param definition
  @verbose              TINYINT = 0      -- debugging aid
AS

  IF @verbose >= 1
  BEGIN
    RAISERROR('_GetSummaryNotes: cid=%s, id=%d, @note_type=%s, @note_status=%s, @note_owner=%s', 0, 1,
               @cid, @id, @note_type, @note_status, @note_owner) WITH NOWAIT
  END

  DECLARE @linked_cid ClassID, @linked_id ObjID

  -- Temparary TABLE to hold final results.
  CREATE TABLE #notesummary
  (
    note_cid		  ClassID,
    note_id		  ObjID,
    note_owner		  sysname NULL,
    note_subject          NVARCHAR(255) NULL,
    note_status		  sysname NULL,
    note_description      NVARCHAR(255) NULL,
    note_originator       NVARCHAR(255) NULL,
    note_datetime         DATETIME NULL,
    note_type		  sysname NULL,
    note_problem_ticket   NVARCHAR(255) NULL,
    _CreatorObjID         ObjID,
    _CreatorCID           ClassID, 
    name                  ObjName NULL,
    note_age		  INT NULL,
    _LinkedCID            ClassID NULL
  )

  -- For linked LOB, also need to identify physical resource or source LOB
  IF RTRIM(@cid) = 'LOB'
  BEGIN
    SELECT @linked_cid = phy_cid, @linked_id = phy_id 
      FROM lob_link WHERE src_id = 0 AND dst_id = @id AND phy_cid IS NOT NULL
    IF @verbose >= 1
    BEGIN
      IF @linked_cid IS NOT NULL
      BEGIN
        RAISERROR('_GetSummaryNotes: LOB instance is linked to cid=%s, id=%d...', 0, 1, @linked_cid, @linked_id) WITH NOWAIT
      END
    END
  END

  -- Set up filter conditions
  DECLARE @note_sql NVARCHAR(4000)
  DECLARE @conditions NVARCHAR(4000)
  DECLARE @where_clause NVARCHAR(4000)

  -- Set up the base query
  IF @linked_cid IS NULL
  BEGIN
    SET @note_sql ="INSERT #notesummary 
     SELECT 'NOTE', id, _UserID, _Subject, _Status, _Body, _Originator, ctime, _Type, _ProblemID, 
            _CreatorObjID, _CreatorCID, '', DATEDIFF(mi,ctime,GETDATE()), NULL
       FROM Note_V AS n 
       JOIN enum_value AS ev ON ev.name = n._Type       
       JOIN enumeration AS e ON e.eid = ev.eid          
       WHERE n.deleted = 0 AND e.name = 'NoteTypes' AND
       n._CreatorCID = '" + @cid + "' AND n._CreatorObjID = " + CONVERT(NVARCHAR(9),@id) + " "
  END
  ELSE
  BEGIN
    SET @note_sql ="INSERT #notesummary 
     SELECT 'NOTE', id, _UserID, _Subject, _Status, _Body, _Originator, ctime, _Type, _ProblemID, 
            _CreatorObjID, _CreatorCID, '', DATEDIFF(mi,ctime,GETDATE()), NULL
       FROM Note_V AS n 
       JOIN enum_value AS ev ON ev.name = n._Type       
       JOIN enumeration AS e ON e.eid = ev.eid          
       WHERE n.deleted = 0 AND e.name = 'NoteTypes' AND
       ((n._CreatorCID = '" + @cid + "' AND n._CreatorObjID = " + CONVERT(NVARCHAR(9),@id) + ") OR " +
       "(n._CreatorCID = '" + @linked_cid + "' AND n._CreatorObjID = " + CONVERT(NVARCHAR(9),@linked_id) + ")) "
  END

  SET @conditions = ''

  -- Note type filter specified?
  IF @note_type IS NOT NULL 
    SET @note_type = RTRIM(LTRIM(@note_type))
  ELSE
    SET @note_type = ''

  IF @note_type <> ''
    SET @conditions = CASE WHEN @conditions = '' THEN " n._Type " + @note_type 
                           ELSE @conditions + " AND n._Type " + @note_type END

  -- Note status filter specified?
  IF @note_status IS NOT NULL 
    SET @note_status = RTRIM(LTRIM(@note_status))
  ELSE
    SET @note_status = ''

  IF @note_status <> ''
    SET @conditions = CASE WHEN @conditions = '' THEN " n._Status " + @note_status 
                           ELSE @conditions + " AND n._Status " + @note_status END

  -- User id filter specified?
  IF @note_owner IS NOT NULL 
    SET @note_owner = RTRIM(LTRIM(@note_owner))
  ELSE
    SET @note_owner = ''

  IF @note_owner <> ''
  BEGIN 
    DECLARE @patindex INT
    SELECT @patindex = PATINDEX ( '%||%' , @note_owner )
    IF @patindex IN (0, 1)
    BEGIN 
      IF @patindex = 0  -- IN passed
      BEGIN 
        SELECT @conditions = CASE WHEN @conditions = '' THEN " LOWER(n._UserID) " + @note_owner 
                                  ELSE @conditions + " AND LOWER(n._UserID) " + @note_owner END
      END
      ELSE 
      BEGIN  -- LIKE passed 
        SELECT @note_owner = SUBSTRING(@note_owner, @patindex+2, LEN(@note_owner)-2)  
        SELECT @conditions = CASE WHEN @conditions = '' THEN " LOWER(n._UserID) " + @note_owner 
                                  ELSE @conditions + " AND LOWER(n._UserID) " + @note_owner END
      END
    END
    ELSE 
    BEGIN  -- IN and LIKE passed 
      DECLARE @note_ownerStr1 NVARCHAR(4000), @note_ownerStr2 NVARCHAR(4000)
      SELECT @note_ownerStr1 = SUBSTRING(@note_owner, 1, @patindex-1)
      SELECT @note_ownerStr2 = SUBSTRING(@note_owner, @patindex+2, (LEN(@note_owner)-LEN(@note_ownerStr1)-2))
      SELECT @conditions = CASE WHEN @conditions = '' THEN " (LOWER(n._UserID) " + @note_ownerStr1 + " OR LOWER(n._UserID) " + @note_ownerStr2 + ")"
                                ELSE @conditions + " AND (LOWER(n._UserID) " + @note_ownerStr1 + " OR LOWER(n._UserID) " + @note_ownerStr2 + ")" END
    END
  END

  -- final setup 
  IF @conditions <> '' 
    SET @where_clause = ' AND' + @conditions
  ELSE
    SET @where_clause = ''

  IF @verbose >= 1
  BEGIN
    RAISERROR('_GetSummaryNotes: where clause for search = <<%s>>', 0, 1, @where_clause)
  END

  -- This will get the Info and Problem notes, and Ownership notes created directly on resource
  EXEC(@note_sql + @where_clause)

  IF @verbose >= 2
  BEGIN
    SELECT * FROM #notesummary
  END

  -- Now set up the call to look at NoteDependent to pick up other notes
  SET @note_sql ="INSERT #notesummary 
     SELECT DISTINCT 'NOTE', nd.noteid, n._UserID, n._Subject, n._Status, n._Body, n._Originator, 
                     n.ctime, n._Type, n._ProblemID, n._CreatorObjID, n._CreatorCID, '', 
                     DATEDIFF(mi,n.ctime,GETDATE()), NULL
                    FROM NoteDependent AS nd
                    JOIN Note_V AS n ON nd.noteid = n.id
                    WHERE depcid = '" + @cid + "' AND depid = " + CONVERT(NVARCHAR(9), @id) + 
                    " AND nd.noteid NOT IN (SELECT note_id FROM #notesummary) "

  -- This will get the Ownership notes this resource is a dependent of                         
  EXEC(@note_sql + @where_clause)

  IF @verbose >= 2
  BEGIN
    SELECT * FROM #notesummary
  END

  -- Now we have to add the name attribute, which we'll get from the object path cache
  UPDATE ns SET ns.name = o.resource
    FROM #notesummary AS ns  
    JOIN ObjPathCache AS o
    ON ns._CreatorCID = o.cid AND ns._CreatorObjID = o.id

  -- Fill in _LinkedCID where needed
  UPDATE ns SET ns._LinkedCID = l.phy_cid
    FROM #notesummary AS ns  
    JOIN lob_link AS l
    ON ns._CreatorCID = 'LOB' AND ns._CreatorObjID = l.dst_id AND l.src_id = 0

  SELECT * 
    FROM #notesummary
    ORDER BY note_datetime DESC

GO
