-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobevaluatecriteria.sql
-- SID: 1.5.4.1
-- Date Modified: 04/11/10 08:01:49
--
-- Module Description
--   Defines the alobEvaluateCriteria stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- alobEvaluateCriteria
PRINT 'Creating procedure alobEvaluateCriteria'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobEvaluateCriteria' AND type = 'P')
  DROP PROCEDURE alobEvaluateCriteria
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobevaluatecriteria.sql
-- SID: 1.5.4.1
-- Date Modified: 04/11/10 08:01:49
CREATE PROCEDURE alobEvaluateCriteria
  
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobEvaluateCriteria
  ELSE
    SAVE TRAN alobEvaluateCriteria

  -- Standard Announcement 
  RAISERROR( 'alobEvaluateCriteria', 0, 1 )

  -- must have #resultCriteria
  -- must have alobListCriteriaAlob
  -- must have #buffer

  CREATE TABLE #batch
  ( 
    iBatch INTEGER IDENTITY,
    sData  VARCHAR(8000)     NOT NULL
  )

/*
  -- HACK 
  PRINT 'SELECT * FROM alobListCriteriaAlob'
  SELECT * FROM alobListCriteriaAlob
  PRINT 'SELECT * FROM #resultCriteria'
  SELECT * FROM #resultCriteria
*/


  -- For each iCriteria in #resultCriteria

  -- Start here
  -- A preprocessing step needs to be performed here.
  -- For each bStart = 1 set the bSeed = 1 where the following match iCriteria, iPattern, iClass, iObject
  --    and the iClassRelated and the iObjectRelated are pointing to the 'proper' iPattern

  -- This is a fix to provide the proper output if the user sneaks in a circular reference in alobListCriteriaAlob
  -- where the chaining of iPattern to iPatternRelated eventually gets one back to the initiating pattern.  This 
  -- may cause the inclusion of successful pattern hits that are not directly related to the original start values.

  -- HACK THIS EXPPLAINATION NEED ADJUSTMENT
  -- The fix is to set bSeed = 1 for only those patterns where bStart = 1 and the iCriteria, iPattern, iClass
  -- iObject, iClassRelated, and iObjectRelated make sense in relation to the pattern.
  
  -- This impacts that actual query that gets built to figure out what criteria has been satisfied.

  -- If the circular references are guaranteed not to occur this logic could be removed and the bStart flag
  -- should be used instead of the bSeed flag.

  CREATE TABLE #start
  (
    iClass      ClassID,
    iObject     ObjID 
  )

  INSERT INTO #start (    iClass, iObject )
  SELECT DISTINCT      rc.iClass, rc.iObject
  FROM  #resultCriteria AS rc
  WHERE 
           rc.bStart = 1

  CREATE INDEX temp_start_comp1 on #start(iClass,iObject)  

  UPDATE #resultCriteria
  SET
    bSeed = 1
  FROM #resultCriteria AS rc
  INNER JOIN #start    AS  s ON s.iClass  = rc.iClass 
                            AND s.iObject = rc.iObject
/*
  -- iCriteria   iPattern    iPatternRelated  rc.bStart 
  -- ----------- ----------- --------------- ---------- 
  -- 1           1           1                        1 <- This is the record type that will get updated 
  UPDATE #resultCriteria
  SET 
    bSeed = 1
  FROM #resultCriteria AS rc
  INNER JOIN alobListCriteriaAlob AS c ON c.iCriteria = rc.iCriteria
                                      AND c.iPattern  = rc.iPattern
  WHERE
           rc.bStart  = 1
       AND rc.iClass  = rc.iClassRelated
       AND rc.iObject = rc.iObjectRelated
       AND c.iPattern = c.iPatternRelated

  -- iCriteria   iPattern    iPatternRelated  rc.bStart 
  -- ----------- ----------- --------------- ---------- 
  -- 1           1           not 1                    1 <- This is the record type that will get updated 
  UPDATE #resultCriteria
  SET 
    bSeed = 1
  FROM #resultCriteria AS rc
  INNER JOIN alobListCriteriaAlob AS c ON c.iCriteria = rc.iCriteria
                                      AND c.iPattern  = rc.iPattern
  WHERE
           rc.bStart  =  1
       AND rc.iClass  <> rc.iClassRelated
       AND rc.iObject <> rc.iObjectRelated
       AND c.iPattern <> c.iPatternRelated
*/

/*
  -- HACK
  PRINT 'SELECT * FROM #resultCriteria -- after populating bSeed to reflect patterns from original start values'
  SELECT * FROM #resultCriteria
*/
  
  DECLARE @iCriteria INTEGER
  DECLARE @sSqlInsert       VARCHAR( 8000 )  
  DECLARE @sSqlSelect       VARCHAR( 8000 )  
  DECLARE @sSqlFrom         VARCHAR( 8000 )  
  DECLARE @sSqlWhere        VARCHAR( 8000 )
  DECLARE @sSqlWhereStart   VARCHAR( 8000 )
  DECLARE @sCRLF            NVARCHAR( 10  )
  DECLARE @iPattern         INTEGER
  DECLARE @iPatternRelated  INTEGER
  DECLARE @sPattern         NVARCHAR( 100 )
  DECLARE @sPatternRelated  NVARCHAR( 100 )
  DECLARE @sCriteria        NVARCHAR( 100 )
  DECLARE @sAlias           NVARCHAR( 100 )
  DECLARE @sAliasRelated    NVARCHAR( 100 )


  SELECT @sSqlInsert = 'INSERT INTO #batch ( sData )'
  SELECT @sCRLF      = '
'

  DECLARE c CURSOR FOR 
    SELECT DISTINCT iCriteria FROM #resultCriteria
  OPEN c
  FETCH NEXT FROM c INTO @iCriteria
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @sCriteria      = CONVERT( NVARCHAR(32), @iCriteria )
    SELECT @sSqlSelect     = @sCRLF + 'SELECT DISTINCT ''' + @sCriteria + ''''
    SELECT @sSqlFrom       = NULL
    SELECT @sSqlWhere      = NULL
    SELECT @sSqlWhereStart = NULL
    DECLARE p CURSOR FOR 
      SELECT iPattern, iPatternRelated FROM alobListCriteriaAlob
      WHERE
               iCriteria = @iCriteria
    OPEN p
    FETCH NEXT FROM p INTO @iPattern, @iPatternRelated
    WHILE @@FETCH_STATUS <> -1
    BEGIN 
      SELECT @sPattern        = CONVERT( NVARCHAR(32), @iPattern )
      SELECT @sPatternRelated = CONVERT( NVARCHAR(32), @iPatternRelated )
      SELECT @sAlias          = 'r_' + @sPattern
      SELECT @sAliasRelated   = 'r_' + @sPatternRelated
      SELECT @sSqlSelect = @sSqlSelect + ' + ''|'' + CONVERT( NVARCHAR( 32), ' + @sAlias + '.iPattern )'
      SELECT @sSqlSelect = @sSqlSelect + ' + ''|'' + ' + @sAlias + '.iClass'
      SELECT @sSqlSelect = @sSqlSelect + ' + ''|'' + CONVERT( NVARCHAR( 32), ' + @sAlias + '.iObject )'
      IF ( @sSqlFrom IS NULL )
      BEGIN
        SELECT @sSqlFrom = @sCRLF + 'FROM'
      END
      ELSE
      BEGIN
        SELECT @sSqlFrom = @sSqlFrom   + ',' + @sCRLF
      END
      SELECT @sSqlFrom   = @sSqlFrom   + '  #resultCriteria AS ' + @sAlias

      IF ( @sSqlWhere IS NULL ) 
      BEGIN
        SELECT @sSqlWhere = @sCRLF + 'WHERE  ' + @sAlias + '.iCriteria  = '     + @sCriteria
      END
      ELSE
      BEGIN
        SELECT @sSqlWhere = @sSqlWhere + @sCRLF + ' AND ' + @sAlias + '.iCriteria  = '     + @sCriteria
      END
      SELECT @sSqlWhere  = @sSqlWhere  + ' AND ' + @sAlias + '.iPattern  = '      + @sPattern 
      SELECT @sSqlWhere  = @sSqlWhere  + ' AND ' + @sAlias + '.iClassRelated = '  + @sAliasRelated + '.iClass'
      SELECT @sSqlWhere  = @sSqlWhere  + ' AND ' + @sAlias + '.iObjectRelated = ' + @sAliasRelated + '.iObject'

      IF ( @sSqlWhereStart IS NULL )
      BEGIN 
        SELECT @sSqlWhereStart = @sCRLF + ' AND ( ' + @sAlias + '.bSeed = 1'
      END
      ELSE
      BEGIN
        SELECT @sSqlWhereStart = @sSqlWhereStart + ' OR ' + @sAlias + '.bSeed = 1'
      END

      FETCH NEXT FROM p INTO @iPattern, @iPatternRelated
    END
    CLOSE p
    DEALLOCATE p

    SELECT @sSqlWhereStart = @sSqlWhereStart + ' )'

/*
    PRINT 'alobEvaluateCriteria dynamic sql:'
    PRINT @sSqlInsert + @sSqlSelect + @sSqlFrom + @sSqlWhere + @sSqlWhereStart
    PRINT ''
*/    
--    PRINT 'Result: ' 
    EXECUTE( @sSqlInsert + @sSqlSelect + @sSqlFrom + @sSqlWhere + @sSqlWhereStart )
--    PRINT ''

/*
    -- HACK 
    PRINT 'SELECT * FROM #batch'
    SELECT * FROM #batch  
*/
    FETCH NEXT FROM c INTO @iCriteria
  END
  CLOSE c
  DEALLOCATE c

  -- At this point all of the #resultCriteria + alobListCriteriaAlob combinations have been processed
  -- Anything that needs to be enqueued exists in #batch
/*
    CREATE TABLE #bufferQueue
    (
      iBatch       INTEGER,
      iCriteria    INTEGER,
      iPattern     INTEGER,
      iClass       ClassID,
      iObject      ObjID
    )
*/
  -- Populate #bufferQueue
  DECLARE @iBatch        INTEGER
  DECLARE @sData         VARCHAR( 8000 )
  DECLARE @nPosSeparator INTEGER
  DECLARE @iClass        ClassID
  DECLARE @iObject       ObjID

  DECLARE b CURSOR FOR 
    SELECT iBatch, sData FROM #batch
  OPEN b
  FETCH NEXT FROM b INTO @iBatch, @sData 
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    SELECT @nPosSeparator = CHARINDEX( '|', @sData )
    SELECT @iCriteria     = SUBSTRING( @sData, 1, ( @nPosSeparator - 1 ) )
    SELECT @sData         = SUBSTRING( @sData, @nPosSeparator + 1, ( LEN( @sData ) - @nPosSeparator ) )
    WHILE ( '' <> @sData )
    BEGIN
      SELECT @nPosSeparator = CHARINDEX( '|', @sData )
      SELECT @iPattern      = SUBSTRING( @sData, 1, ( @nPosSeparator - 1 ))
      SELECT @sData         = SUBSTRING( @sData, @nPosSeparator + 1, ( LEN( @sData ) - @nPosSeparator ) )

      SELECT @nPosSeparator = CHARINDEX( '|', @sData )
      SELECT @iClass        = SUBSTRING( @sData, 1, ( @nPosSeparator - 1 ))
      SELECT @sData         = SUBSTRING( @sData, @nPosSeparator + 1, ( LEN( @sData ) - @nPosSeparator ) )

      SELECT @nPosSeparator = CHARINDEX( '|', @sData )
      IF ( 0 <> @nPosSeparator ) 
      BEGIN
        SELECT @iObject       = SUBSTRING( @sData, 1, ( @nPosSeparator - 1 ))
        SELECT @sData         = SUBSTRING( @sData, @nPosSeparator + 1, ( LEN( @sData ) - @nPosSeparator ) )
      END
      ELSE
      BEGIN
        SELECT @iObject       = @sData
        SELECT @sData         = ''
      END
      INSERT INTO #bufferQueue (  iBatch,  iCriteria,  iPattern,  iClass,  iObject )
      VALUES                   ( @iBatch, @iCriteria, @iPattern, @iClass, @iObject )
    END
    FETCH NEXT FROM b INTO @iBatch, @sData 
  END
  CLOSE b
  DEALLOCATE b

/*
  -- HACK 
  PRINT 'SELECT * FROM #bufferQueue'
  SELECT * FROM #bufferQueue
*/

  DROP TABLE #start

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobEvaluateCriteria
GO
