-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobputresultcriteria.sql
-- SID: 1.9
-- Date Modified: 04/11/10 08:01:51
--
-- Module Description
--   Defines the alobPutResultCriteria stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- alobPutResultCriteria
PRINT 'Creating procedure alobPutResultCriteria'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobPutResultCriteria' AND type = 'P')
  DROP PROCEDURE alobPutResultCriteria
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/alobputresultcriteria.sql
-- SID: 1.9
-- Date Modified: 04/11/10 08:01:51
CREATE PROCEDURE alobPutResultCriteria
  @bStart INT = 0
  
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobPutResultCriteria
  ELSE
    SAVE TRAN alobPutResultCriteria

  -- Standard Announcement 
  RAISERROR( 'alobPutResultCriteria bStart=%d', 0, 1, @bStart)

  -- The following tables need to pre-exist the call to this stored procedure
  -- #resultCriteria
  -- #resultPattern
  -- #data


  -- create temporary table to split process for populating  
  -- #resultCriteria from new #resultPattern rows
  CREATE TABLE #resultCriteriaTEMP
  (
    iCriteria               INTEGER       NOT NULL,
    iPattern                INTEGER       NOT NULL,
    iClass                  ClassID,
    iObject                 ObjID,
    iClassRelated           ClassID,
    iObjectRelated          ObjID,
    iClassStart             ClassID,
    iObjectStart            ObjID,
    bStart                  INTEGER DEFAULT 0 
  )

  CREATE TABLE #resultCriteriaTEMP2
  (
    iPattern                INTEGER       NOT NULL,
    iClass                  ClassID,
    iObject                 ObjID,
    iClassRelated           ClassID,
    iObjectRelated          ObjID,
    iClassStart             ClassID,
    iObjectStart            ObjID,
  )

  CREATE TABLE #resultCriteriaTEMP3
  (
    iCriteria               INTEGER       NOT NULL,
    iPattern                INTEGER       NOT NULL,
    iClass                  ClassID,
    iObject                 ObjID,
    iClassRelated           ClassID,
    iObjectRelated          ObjID,
    iClassStart             ClassID,
    iObjectStart            ObjID,
    iPatternTemp            INTEGER       NOT NULL 
  )


  -- Seed has been used, time to reset the flag
  UPDATE #resultCriteria
  SET bSeed = 0
  FROM #resultCriteria AS rc
  WHERE
           rc.bSeed = 1
  
  IF ( 1 = @bStart )  -- First pass insert could be merged with the next population of #resultCriteria but would 
                      -- required expensive FULL OUTER JOIN
  BEGIN
    -- Populate #resultCriteria from starting points in #resultPattern and #data
    INSERT INTO #resultCriteria (   iCriteria,    iPattern,    iClass,    iObject,   iClassRelated,   iObjectRelated,  
                                      iClassStart,   iObjectStart,  bStart )
    SELECT      DISTINCT          c.iCriteria, rp.iPattern, rp.iClass, rp.iObject, d.iClassRelated, d.iObjectRelated, 
                                    d.iClassStart, d.iObjectStart, @bStart
    FROM       #resultPattern       AS rp
    INNER JOIN #data                AS d ON d.iClass   = rp.iClass 
                                        AND d.iObject  = rp.iObject
    INNER JOIN alobListCriteriaAlob AS c ON c.iPattern = rp.iPattern
  END

  IF @bStart = 1
  BEGIN
    CREATE INDEX tResultCriterIndex1 ON #resultCriteria(iCriteria,iPattern,iClass,iObject,iClassRelated,iObjectRelated,iClassStart,iObjectStart)
  END


  -- Below two inserts were originally one insert.  However because of performance issues, the insert had
  -- to be split up into two inserts (Defect 109962).  When executing ABSTest script, the original insert would 
  -- take hours to process.  With the below two inserts, the script now runs in minutes for large number of
  -- LOB discovery.
  -- Populate #resultCriteriaTEMP from new #resultPattern rows

  -- Following insert further split for better performance for defect 109967
  --INSERT INTO #resultCriteriaTEMP (   iCriteria,    iPattern,    iClass,    iObject,   iClassRelated,   iObjectRelated,  
  --                                  iClassStart,   iObjectStart,  bStart )
  --SELECT      DISTINCT          c.iCriteria, rp.iPattern, rp.iClass, rp.iObject, d.iClassRelated, d.iObjectRelated, 
  --                              d.iClassStart, d.iObjectStart, @bStart
  --FROM       #resultPattern       AS rp
  --INNER JOIN #data                AS d ON d.iClass   = rp.iClass 
  --                                    AND d.iObject  = rp.iObject
  --INNER JOIN alobListCriteriaAlob AS c ON c.iPattern = rp.iPattern
  --  WHERE NOT EXISTS ( SELECT * FROM #resultCriteria AS rc2
  --                       WHERE    
  --                              rc2.iCriteria      = c.iCriteria
  --                          AND rc2.iPattern       = rp.iPattern
  --                          AND rc2.iClass         = rp.iClass
  --                          AND rc2.iObject        = rp.iObject
  --                          AND rc2.iClassRelated  = d.iClassRelated
  --                          AND rc2.iObjectRelated = d.iObjectRelated
  --                          AND rc2.iClassStart    = d.iClassStart
  --                          AND rc2.iObjectStart   = d.iObjectStart
  --                   )

  INSERT INTO #resultCriteriaTEMP2 (iPattern,    iClass,    iObject,    iClassRelated,   iObjectRelated,  
                                    iClassStart,   iObjectStart)
  SELECT      DISTINCT              rp.iPattern, rp.iClass, rp.iObject, d.iClassRelated, d.iObjectRelated, 
                                    d.iClassStart, d.iObjectStart
  FROM       #resultPattern       AS rp
  INNER JOIN #data                AS d ON d.iClass   = rp.iClass 
                                      AND d.iObject  = rp.iObject

  INSERT INTO #resultCriteriaTEMP ( iCriteria,   iPattern,       iClass,       iObject,       iClassRelated,       iObjectRelated,  
                                    iClassStart,       iObjectStart,       bStart )
  SELECT      DISTINCT              c.iCriteria, temp2.iPattern, temp2.iClass, temp2.iObject, temp2.iClassRelated, temp2.iObjectRelated, 
                                    temp2.iClassStart, temp2.iObjectStart, @bStart
  FROM       #resultCriteriaTEMP2       AS temp2
  INNER JOIN alobListCriteriaAlob AS c ON c.iPattern = temp2.iPattern
    WHERE NOT EXISTS ( SELECT * FROM #resultCriteria AS rc2
                         WHERE    
                                rc2.iCriteria      = c.iCriteria
                            AND rc2.iPattern       = temp2.iPattern
                            AND rc2.iClass         = temp2.iClass
                            AND rc2.iObject        = temp2.iObject
                            AND rc2.iClassRelated  = temp2.iClassRelated
                            AND rc2.iObjectRelated = temp2.iObjectRelated
                            AND rc2.iClassStart    = temp2.iClassStart
                            AND rc2.iObjectStart   = temp2.iObjectStart
                     )


  -- Now populate #resultCriteria from #resultCriteria rows
  INSERT INTO #resultCriteria (   iCriteria,    iPattern,    iClass,    iObject,   iClassRelated,   iObjectRelated,  
                                    iClassStart,   iObjectStart,  bStart )
  SELECT      DISTINCT          rct.iCriteria, rct.iPattern, rct.iClass, rct.iObject, rct.iClassRelated, rct.iObjectRelated, 
                                  rct.iClassStart, rct.iObjectStart, @bStart
  FROM       #resultCriteriaTEMP  AS rct
  INNER JOIN #resultCriteria      AS rcStart ON rcStart.iCriteria = rct.iCriteria
  WHERE     rcStart.bStart = 1

  -- Populate #resultCriteria  based on rows that exist in #resultCriteria, but need to be paired up
  -- SELECT * FROM #resultCriteria -- After the nth population
  -- iCriteria   iPattern    iClass iObject     iClassRelated iObjectRelated bSeed       iNode      bStart      
  -- ----------- ----------- ------ ----------- ------------- -------------- ----------- ---------- ----------- 
  -- 1           3           COMP   1           COMP          1              0           NULL       1
  -- 1           2           LPAR   3           COMP          1              1           NULL       0
  -- SELECT * FROM alobListCriteriaAlob
  -- iCriteria   iPattern    iPatternRelated 
  -- ----------- ----------- --------------- 
  -- 1           3           2( rc. <> 2 )   <- This row exists                      ( rc1,     c1,
  -- 1           2           1( rc. =  3 )   <- This row exists                      ( rc2,     c2, 
  -- 1           3           2               <- This is the row that will be created ( rc1, rc2

  --  following insert split for better performance for defect 109967

  --INSERT INTO #resultCriteria (     iCriteria,     iPattern,     iClass,     iObject,     iClassRelated,     iObjectRelated,  
  --                                   iClassStart,     iObjectStart,  bStart )
  --SELECT      DISTINCT          rc1.iCriteria, rc1.iPattern, rc1.iClass, rc1.iObject,        rc2.iClass,        rc2.iObject, 
  --                               rc2.iClassStart, rc2.iObjectStart, @bStart
  --FROM #resultCriteria            AS rc1
  --INNER JOIN alobListCriteriaAlob AS  c1 ON c1.iCriteria       =  rc1.iCriteria
  --                                      AND c1.iPattern        =  rc1.iPattern
  --INNER JOIN alobListCriteriaAlob AS  c2 ON c2.iCriteria       =  rc1.iCriteria
  --                                      AND c2.iPattern        <> rc1.iPattern
  --                                      AND c2.iPattern        =  c1.iPatternRelated
  --INNER JOIN #resultCriteria      AS rc2 ON rc2.iCriteria      =  rc1.iCriteria
  --                                      AND rc2.iPattern       =  c2.iPattern
  --                                      AND rc2.iClassRelated  =  rc1.iClass
  --                                      AND rc2.iObjectRelated =  rc1.iObject
  --WHERE   NOT EXISTS ( SELECT * FROM #resultCriteria AS dup
  --                     WHERE    
  --                              dup.iCriteria      = rc1.iCriteria
  --                          AND dup.iPattern       = rc1.iPattern
  --                          AND dup.iClass         = rc1.iClass
  --                          AND dup.iObject        = rc1.iObject
  --                          AND dup.iClassRelated  = rc2.iClass
  --                          AND dup.iObjectRelated = rc2.iObject
  --                          AND dup.iClassStart    = rc2.iClassStart
  --                          AND dup.iObjectStart   = rc2.iObjectStart
  --                 )

  INSERT INTO #resultCriteriaTEMP3(  iCriteria,     iPattern,     iClass,     iObject,     iClassRelated, iObjectRelated,  
                                     iClassStart,     iObjectStart,     iPatternTemp )
  SELECT      DISTINCT               rc1.iCriteria, rc1.iPattern, rc1.iClass, rc1.iObject, rc1.iClass,    rc1.iObject, 
                                     rc1.iClassStart, rc1.iObjectStart, c2.iPattern
  FROM #resultCriteria            AS rc1
  INNER JOIN alobListCriteriaAlob AS  c1 ON c1.iCriteria       =  rc1.iCriteria
                                        AND c1.iPattern        =  rc1.iPattern
  INNER JOIN alobListCriteriaAlob AS  c2 ON c2.iCriteria       =  rc1.iCriteria
                                         AND c2.iPattern       <> rc1.iPattern
                                         AND c2.iPattern       =  c1.iPatternRelated

  INSERT INTO #resultCriteria (     iCriteria,       iPattern,       iClass,       iObject,       iClassRelated, iObjectRelated,  
                                    iClassStart,     iObjectStart,     bStart )
  SELECT      DISTINCT              temp3.iCriteria, temp3.iPattern, temp3.iClass, temp3.iObject, rc2.iClass,    rc2.iObject, 
                                    rc2.iClassStart, rc2.iObjectStart, @bStart
  FROM #resultCriteriaTEMP3       AS temp3
  INNER JOIN #resultCriteria      AS rc2 ON rc2.iCriteria      =  temp3.iCriteria
                                        AND rc2.iPattern       =  temp3.iPatternTemp
                                        AND rc2.iClassRelated  =  temp3.iClass
                                        AND rc2.iObjectRelated =  temp3.iObject
  WHERE   NOT EXISTS ( SELECT * FROM #resultCriteria AS dup
                       WHERE    
                                dup.iCriteria      = temp3.iCriteria
                            AND dup.iPattern       = temp3.iPattern
                            AND dup.iClass         = temp3.iClass
                            AND dup.iObject        = temp3.iObject
                            AND dup.iClassRelated  = rc2.iClass
                            AND dup.iObjectRelated = rc2.iObject
                            AND dup.iClassStart    = rc2.iClassStart
                            AND dup.iObjectStart   = rc2.iObjectStart
                     )


  -- Populate #resultCriteria based on rows that exist in #resultCriteria, object that is related to itself
  -- SELECT * FROM #resultCriteria -- After the nth population
  -- iCriteria   iPattern    iClass iObject     iClassRelated iObjectRelated bSeed       iNode      bStart      
  -- ----------- ----------- ------ ----------- ------------- -------------- ----------- ---------- ----------- 
  -- 2           2           LPAR   1           OS            11             1           NULL       0
  -- SELECT * FROM alobListCriteriaAlob
  -- iCriteria   iPattern    iPatternRelated 
  -- ----------- ----------- --------------- 
  -- 2           2           2( rc. <> 2 )   <- This row exists                      ( rc1,     c1,
  -- 2           2           2               <- This is the row that will be created ( rc1, rc2
  INSERT INTO #resultCriteria (     iCriteria,     iPattern,     iClass,     iObject,     iClassRelated,     iObjectRelated,  
                                       iClassStart,     iObjectStart,  bStart )
  SELECT      DISTINCT          rc1.iCriteria, rc1.iPattern, rc1.iClass, rc1.iObject,        rc1.iClass,        rc1.iObject, 
                                   rc1.iClassStart, rc1.iObjectStart,  @bStart
  FROM #resultCriteria            AS rc1
  INNER JOIN alobListCriteriaAlob AS  c1 ON c1.iCriteria       =  rc1.iCriteria
                                        AND c1.iPattern        =  rc1.iPattern
                                        AND c1.iPattern        =  c1.iPatternRelated
  WHERE NOT EXISTS ( SELECT * FROM #resultCriteria AS dup
                     WHERE
                                dup.iCriteria      = rc1.iCriteria
                            AND dup.iPattern       = rc1.iPattern
                            AND dup.iClass         = rc1.iClass
                            AND dup.iObject        = rc1.iObject
                            AND dup.iClassRelated  = rc1.iClass
                            AND dup.iObjectRelated = rc1.iObject
                            AND dup.iClassStart    = rc1.iClassStart
                            AND dup.iObjectStart   = rc1.iObjectStart
                   )




  -- turn off bSeed for those items that have already been checked 
  UPDATE #resultCriteria
  SET  bSeed = 0
  FROM #resultCriteria AS rc1
  WHERE 
           bSeed = 1
       AND EXISTS ( SELECT * FROM #resultCriteria AS rc2
                      WHERE 
                               rc2.iCriteria      =  rc1.iCriteria
                           AND rc2.iPattern       =  rc1.iPattern
                           AND rc2.iClass         =  rc1.iClass
                           AND rc2.iObject        =  rc1.iObject
                           AND rc2.iClassRelated  = rc1.iClassRelated
                           AND rc2.iObjectRelated = rc2.iObjectRelated
--                           AND rc2.bStart    <> rc1.bStart
                           AND rc2.bSeed          = 0
                  )  

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobPutResultCriteria
GO
