-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\ap_getmanagedobjectsplus.sql
-- SID: 1.10.1.11
-- Date Modified: 04/11/03 10:59:44
--
-- Module Description
--   Defines the ap_GetManagedObjectsPlus stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- ap_GetManagedObjectsPlus
PRINT 'Creating procedure ap_GetManagedObjectsPlus'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ap_GetManagedObjectsPlus' AND type = 'P')
  DROP PROCEDURE ap_GetManagedObjectsPlus
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\ap_getmanagedobjectsplus.sql
-- SID: 1.10.1.11
-- Date Modified: 04/11/03 10:59:44
--
-- The routine wrappers the _GetManagedObjects method to provide a superset of 
-- functions to that routine that were not available at the time of this writing. 
-- In some, if not all cases, the added funtion provided by this proc could be moved 
-- into the base method.
--
-- In general the arguments to these 'plus' routines exactly match those of the main
-- proc (in this case _GetManagedObjects) and then 'value-add' plus proc specific
-- attributes follow. 
--
-- This routine will either return the information in a table named #tmpGetMgedObjPlus.
-- If the caller sets the 'returnonly' argument to 1, the assumption is that this table
-- was created by the caller and therefore the data is being returned to the caller. 
--
CREATE PROCEDURE ap_GetManagedObjectsPlus
  ---- first the inputs that mirror _GetManagedObjects
  @cid                ClassID,
  @id                 ObjID,
  @pathid             NVARCHAR(2048) =NULL,
  @LocationPattern    NVARCHAR(255) = NULL,
  @NamePattern        NVARCHAR(255) = NULL,
  @MinAlert           INT = 0,
  @MinPriority        INT = 0,
  @OpenNotes          INT = 2,
  @ObjectType         ClassID = NULL,
  --- now for the new filters that are added on for the web 
  @userid             NVARCHAR(255) = NULL, 
  @plus_owned         INT = 0,
  @plus_owner         NVARCHAR(128)=NULL,
  @plus_rootcauseonly TINYINT=1,  ---------------   NEED TO IMPLEMENT
  @returnonly         TINYINT=0,
  @verbose            TINYINT=0
AS
  SET NOCOUNT ON

  DECLARE @rc INT
  DECLARE @datetime DATETIME, @datetime_str NVARCHAR(100)

  IF @verbose >= 1
  BEGIN
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('ap_GetManagedObjectsPlus: started at %s', 0, 1, @datetime_str) WITH NOWAIT
    RAISERROR('ap_GetManagedObjectsPlus: @cid=%s, @id=%d, @LocationPattern=%s, @NamePattern=%s',
		 0, 1, @cid, @id, @LocationPattern, @NamePattern) WITH NOWAIT
    RAISERROR('ap_GetManagedObjectsPlus: @MinAlert=%d, @MinPriority=%d, @OpenNotes=%d, @ObjectType=%s',
		 0, 1,@MinAlert, @MinPriority, @OpenNotes, @ObjectType) WITH NOWAIT
  END

  -- convert cid to enum value
  DECLARE @objType INT
  SET @objType = 0
  IF @ObjectType IS NOT NULL
  BEGIN
    SELECT @objType = value
      FROM enum_value AS ev JOIN enumeration AS e ON ev.eid = e.eid
      WHERE e.name = 'ManagedObjectTypes' AND ev.val_cid = @ObjectType
  END

  -- This table mirrors the output of _GetManagedObjects.  It must stay that way 
  -- so that the calls to that routine will work 
  CREATE TABLE #tmpGetMgedObj
  (
    _AlertStateID     INT NULL,
    _HasOwnershipNote TINYINT NULL,
    Location          NVARCHAR(255) NULL,
    Type              NVARCHAR(255) NULL,
    name              ObjName NULL,
    _PriorityID       INT NULL,
    _CurrentStateID   INT NULL,
    _DesiredStateID   INT NULL,
    Description       NVARCHAR(255) NULL,
    cid               ClassID,
    id                ObjID,
    _flags            INT NULL
  )

  -- In most cases this table is created by the caller. If not it will be created and 
  -- returned as the result set of this proc. 
  IF @returnonly = 0
  BEGIN 
    CREATE TABLE #tmpGetMgedObjPlus
    (
      myident           INT IDENTITY(1,1),
      _AlertStateID     INT NULL,
      _HasOwnershipNote TINYINT NULL,
      Location          NVARCHAR(255) NULL,
      Type              NVARCHAR(255) NULL,
      name              ObjName NULL,
      _PriorityID       INT NULL,
      _CurrentStateID   INT NULL,
      _DesiredStateID   INT NULL,
      Description       NVARCHAR(255) NULL,
      cid               ClassID,
      id                ObjID, 
      _flags            INT NULL,
      owners            NVARCHAR(1024) NULL
    )
  END 


  -- To handle multiple appends to the output table, get the last identity element 
  -- that is already in the table and make sure any filtering is done on records inserted
  -- after that element
  DECLARE @myTableStartingPoint INT 
  SELECT @myTableStartingPoint=ISNULL(MAX(myident), 0) FROM #tmpGetMgedObjPlus

  IF @cid IS NULL OR @cid='' OR @id=0 
  BEGIN
    RAISERROR('ap_GetManagedObjectsPlus: bad cid/id @cid=%s, @id=%d', 0, 1, @cid, @id) WITH NOWAIT
    GOTO getout
  END

  IF @verbose >= 1
  BEGIN
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('ap_GetManagedObjectsPlus: calling GetManagedObjects at %s', 0, 1, @datetime_str) WITH NOWAIT
  END
  EXEC _GetManagedObjects
         @cid,
         @id,
         @LocationPattern,
         @NamePattern,
         @MinAlert,
         @MinPriority,
         @OpenNotes,
         @objType,
         '#tmpGetMgedObj'

  IF @verbose >= 1
  BEGIN  
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    DECLARE @count INT
    SELECT @count = COUNT(*) FROM #tmpGetMgedObj
    RAISERROR('ap_GetManagedObjectsPlus: GetManagedObjects returned %d rows at %s', 0, 1, @count, @datetime_str) WITH NOWAIT
   --SELECT * FROM #tmpGetMgedObj
  END

  INSERT INTO #tmpGetMgedObjPlus
  (
    _AlertStateID,
    _HasOwnershipNote,
    Location,
    Type,
    name,
    _PriorityID,
    _CurrentStateID,
    _DesiredStateID,
    Description,
    cid,
    id,
    _flags
  )
  SELECT * FROM  #tmpGetMgedObj

  IF @verbose >= 1
  BEGIN  
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('ap_GetManagedObjectsPlus: Done moving data to #tmpGetMgedObjPlus at %s', 0, 1, @datetime_str) WITH NOWAIT
  END

  /* 
     Now concat all userids which own an event for this resource
     - to do this we what we have to do is a step-wise process that issues an update on the temporary
       tables owner column and then continue to issue the update (a slightly different one to handle
       the condition where the owner has already been captured into the table) until none of the rows
       are updated (all owners have been captured).  Each update will only catch one of the owners.)
     - The only other thing to be careful of is running over the size of the owners slot therefore the 
       updates may always return something.  Although this is very unlikely to happen it should still be 
       handled. 
     - last thing that must be done is to handle the case where an ownership note was taken on 
       a higher level object.  There will not be a tie between it and its events rather it will 
       be between it and a note object through the link table with type ATRC
  */ 
  DECLARE @owners nvarchar(1024)
  SET @owners=NULL

  -- get the owners as a comma delimited list of userids. 
  -- NOTE: for LDAP support, we are now using the distinguished name (DN) instead of the userid. Since the
  -- DN composed of comma delimited attributes, we can't use a comma to separate the DNs.
  -- After searching through LDAP documentation, it appears that '|' and '$' are not valid chars in
  -- a DN. But rather than take a chance on a single character delimiter, we'll use '|$|' as
  -- the delimiter bewteen DNs. 
  DECLARE @owner_sep NVARCHAR(3)
  SET @owner_sep = N'|$|'

  --- print ' getting first owners   ' + convert (nvarchar, CURRENT_TIMESTAMP, 109)
  DECLARE @i TINYINT
  SET @i=10
  WHILE (@i>0)
  BEGIN
    SET @i = @i - 1
    --- print ' in loop getting first owners   ' + convert (nvarchar, CURRENT_TIMESTAMP, 109)
    UPDATE #tmpGetMgedObjPlus
      SET owners =
         CASE WHEN tmp.owners IS NULL THEN @owner_sep + n._UserID + @owner_sep 
              ELSE tmp.owners + n._UserID + @owner_sep
         END
      FROM 
        OwnedEvent AS owned,
        #tmpGetMgedObjPlus AS tmp,
        Note_V AS n
      WHERE
        tmp.myident > @myTableStartingPoint
        AND tmp._HasOwnershipNote=1
        AND owned.parentcid = tmp.cid
        AND owned.parentid = tmp.id
        -- AND owned.closetime IS NULL  --- and should probably key off of status  
        AND n._Status = 'Open'
        AND n._Type = 'Ownership'
        AND n.id = owned.noteid  
        AND (tmp.owners IS NULL OR 
              (CHARINDEX(@owner_sep + n._UserID + @owner_sep, tmp.owners) = 0
                 AND (LEN(n._UserID)+ LEN(tmp.owners) + LEN(@owner_sep) < 1024)    --- 1024 is the size of the column
              )
             )

    SET @rc = @@ROWCOUNT
    IF @rc = 0 SET @i=0 

    IF @verbose >= 1
    BEGIN  
      SET @datetime = CURRENT_TIMESTAMP
      SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
      RAISERROR('ap_GetManagedObjectsPlus: Done getting first owners at %s', 0, 1, @datetime_str) WITH NOWAIT
    END
  END 

  -- Now handle ownership notes that were taken on an higher level object without events
  -- once again this should be done in a loop
  -- looking for link of type 'ATRC' linking objects in the temp table with the Notes_C table
  -- that are not deleted 
  SET @i=10
  WHILE (@i>0)
  BEGIN
    SET @i = @i - 1
    UPDATE tmp
      SET owners =
          CASE WHEN tmp.owners IS NULL THEN  @owner_sep + n._UserID + @owner_sep 
               ELSE tmp.owners + n._UserID + @owner_sep
          END
      FROM #tmpGetMgedObjPlus AS tmp
        JOIN Note_V AS n ON (tmp.cid=n._CreatorCID AND tmp.id=n._CreatorObjID AND n.deleted=0 AND n._Status = 'Open' AND n._Type = 'Ownership')      
      WHERE
        tmp.myident > @myTableStartingPoint
        AND tmp._HasOwnershipNote=1
        AND (tmp.owners IS NULL OR 
                (CHARINDEX(@owner_sep + n._UserID + @owner_sep, tmp.owners) = 0
                   AND (LEN(n._UserID)+ LEN(tmp.owners) + LEN(@owner_sep) < 1024)    --- 1024 is the size of the column
                )
             )

    SET @rc = @@ROWCOUNT
    IF @rc=0 SET @i = 0 
  END 

  IF @verbose >= 1
  BEGIN  
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('ap_GetManagedObjectsPlus: Done getting second owners at %s', 0, 1, @datetime_str) WITH NOWAIT
  END
   
  -- now apply plus unique filters if necessary ... 

  -- first for a particular owner
  IF @plus_owner IS NOT NULL AND @plus_owner <> N''
  BEGIN 
    SET @plus_owner = N'%' + @owner_sep + @plus_owner + @owner_sep + N'%'
    DELETE FROM #tmpGetMgedObjPlus WHERE myident > @myTableStartingPoint
					      AND LOWER(owners) NOT LIKE LOWER(@plus_owner)
  END 

  -- now if it must be owned or not ... 
  IF   @plus_owned=1
  BEGIN   --- must be owned
    DELETE FROM #tmpGetMgedObjPlus WHERE  myident > @myTableStartingPoint
      AND  (owners IS NULL OR owners = N'')
  END
  ELSE IF   @plus_owned=2
  BEGIN   --- must be unowned
    DELETE FROM #tmpGetMgedObjPlus WHERE myident > @myTableStartingPoint
      AND (owners IS NOT NULL OR owners <> N'')
  END

  -- now return results 
  getout:
  IF @returnonly = 0
  BEGIN
    SELECT * FROM  #tmpGetMgedObjPlus
  END 

  IF @verbose >= 1
  BEGIN  
    SET @datetime = CURRENT_TIMESTAMP
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('ap_GetManagedObjectsPlus: exit at %s', 0, 1, @datetime_str) WITH NOWAIT
  END
GO
