-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/api_getavaildetails.sql
-- SID: 1.24
-- Date Modified: 04/11/16 14:33:27
-- 
-- Module Description
--   Defines the api_getAvailDetails stored procedure
--

USE Object
GO

-------------------------------------------------------------------------------
-- api_getAvailDetails
-------------------------------------------------------------------------------

PRINT 'Creating procedure api_getAvailDetails'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'api_getAvailDetails' AND type = 'P')
  DROP PROCEDURE api_getAvailDetails
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/api_getavaildetails.sql
-- SID: 1.24
-- Date Modified: 04/11/16 14:33:27
--
-- Retrieves availability data for the tbsm api 
--
CREATE PROCEDURE api_getAvailDetails
  @cid ClassID=NULL,
  @id  INT = 0 , 
  @IncludeClosedProblems  INT = 0,  -- Boolean indicating whether problems that are currently closed are to be reported. If 1 (true) then closed problems are reported.  
  @ClosedProblemSpan      INT = 0,  -- Filter specifying how many hours of past closed problems will be reported. Meaningful only if IncludeClosedProblems=1.  
  @IncludeResources       INT = 1,      -- Boolean indicating whether Event information is to be retrieved.  
  @IncludeNotesType       INT = 1,        -- Boolean indicating whether Note information is to be retrieved.  
  @verbose TINYINT=0
AS
  SET NOCOUNT ON 

  CREATE TABLE #ImpactedPaths
  ( 
    cid   ClassID, 
    id    ObjID, 
    idpath  NVARCHAR(1000) NULL , 
    path  NVARCHAR(1000) NULL,      
    linked_cid ClassID NULL,    -- is the main cid is a service be sure to make this the physical object if its lob is linked to one. 
    linked_id  ObjID  NULL, 
    resourceName NVARCHAR(256) NULL
  ) 

  CREATE TABLE #ExtIncidentRecord
  (
    incident_cid        NVARCHAR(4) NOT NULL,
    incident_id         INT NOT NULL,
    resource_cid        NVARCHAR(4) NULL, 
    resource_id         INT NULL, 
    normalizedState     INT NULL,           --- this is the alert state for the event
    domainState         INT NULL,   		    --- this is the alert state for the event
    eventOperationState INT NULL DEFAULT 0, 		  --- 0 it is open, 1 it is owned, 2 it is closed
    ctime               DATETIME NULL,         --- time event was created or received which one 
    closetime           DATETIME NULL,         --- time event was created or received which one 
    mtime               DATETIME NULL,     
    shortDesc           NVARCHAR(256) NULL,
    longDesc            NVARCHAR(512) NULL,        
    note_cid            NVARCHAR(4) NULL,      --- if this event is tied to a note (multiple notes can be with an event)
    note_id             INT  NULL, 
    ownerName           NVARCHAR(128)  NULL, 
    flags               INT  NULL DEFAULT 0, 
    detailLevel         INT NOT NULL DEFAULT 0  
  )

  CREATE TABLE #ExtNote
  (
    note_cid       ClassID   NOT NULL,
    note_id        ObjID     NOT NULL,
    type           NVARCHAR(40) DEFAULT 'Ownership', 
    ctime          DATETIME  NULL,
    subject        NVARCHAR (255) NULL,
    TicketNumber   NVARCHAR (255) NULL,
    TicketOpenTime DATETIME NULL
  )

  /* #AvailRelationships is the result set asisp_GetAllSubServices will place the results in  */
  CREATE TABLE  #AvailRelationships
  (
    relationship_label NVARCHAR(12) NULL,     -- 'link_type'
    source_cid         ClassID   NULL,     -- ease of use, does not have to be returned to api 
    source_id          ObjID           NULL,
    source_name        NVARCHAR(128) NULL,     -- do not bother doing this for now
    target_cid         ClassID   NULL,     -- ease of use, does not have to be returned to api 
    target_id          ObjID           NULL,
    target_name        NVARCHAR(128) NULL,     -- do not bother doing this for now 
    createTS           DATETIME      NULL,
    modifiedTS         DATETIME      NULL,     -- return same as createTS in link table 
    deletedTS          DATETIME      NULL,     -- return null 
    gen                INT           NULL,     -- 1 for immediate child, 2 for grandchild, etc.
    detailLevel        INT           DEFAULT 100,   -- set to 100, 
    lob_id             ObjID         DEFAULT 0, 
    lob_flags          INT           DEFAULT 0
  )

  CREATE TABLE  #r
  (
    ServiceCID  ClassID NULL, 
    ServiceID   ObjID   NULL, 
    ChildCID    ClassID NULL, 
    ChildID     ObjID   NULL, 
    respath     NVARCHAR(1000),
    RootCause   INT DEFAULT 0  
  )

  CREATE TABLE #TempEvents  
  (  
    cid         ClassID NULL, 
    id          ObjID   NULL  
  )

  DECLARE
    @AlertLevelToReturn INT,
    @count              INT,
    @err                INT,
    @now                NVARCHAR(40),
    @ServiceLobCid      ClassID,
    @ServiceLobId       ObjID,
    @rc                 INT,
    @sub                INT,
    @testPath           NVARCHAR(500)

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Started Processing', 0, 1, @now) WITH NOWAIT
  END

  -- Output Table of Sevice Attributes - cid, id correlates on cid/id of #Services table
  -- The check for the existence the temp tables has been implemented to support the ExecutiveView APIs.
  -- When the APIs call this method it will provide the temp tables and the method will not create
  -- them.

  IF OBJECT_ID('tempdb..#ServicesAttrs') IS NULL
  BEGIN
    CREATE TABLE #ServicesAttrs
    (
      ServiceCID   ClassID NOT NULL,
      ServiceID    ObjID NOT NULL,   
      cid  ClassID NOT NULL,   
      id   ObjID   NOT NULL,
      BusinessRole NVARCHAR(500) NULL,
      RedImpact    NVARCHAR(500) NULL,
      YellowImpact NVARCHAR(500) NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #ServicesAttrs', 0, 1, @now) WITH NOWAIT
    END
  END

  -- Output Table of Notes - ObjectCID, ObjectID correlates on cid/id of #Services table
  -- The check for the existence the temp tables has been implemented to support the ExecutiveView APIs.
  -- When the APIs call this method it will provide the temp tables and the method will not create
  -- them.

  IF OBJECT_ID('tempdb..#Notes') IS NULL
  BEGIN 
    CREATE TABLE #Notes
    (
      ServiceCID     ClassID NOT NULL,
      ServiceID      ObjID NOT NULL,  
      ObjectCID      ClassID NOT NULL,
      ObjectID       ObjID   NOT NULL,
      Id             ObjID   NULL ,
      Type           NVARCHAR(40) DEFAULT 'Ownership', 
      NoteOpenTime   DATETIME NULL,
      NoteSubject    NVARCHAR (255) NULL,
      TicketNumber   NVARCHAR (255) NULL,
      TicketOpenTime DATETIME NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #Notes', 0, 1, @now) WITH NOWAIT
    END
  END 

  -- Output Table of Event Information - ObjectCID, ObjectID, correlates to cid/id of #Services table
  -- The check for the existence the temp tables has been implemented to support the ExecutiveView APIs.
  -- When the APIs call this method it will provide the temp tables and the method will not create
  -- them.
  IF OBJECT_ID('tempdb..#Events') IS NULL
  BEGIN
    CREATE TABLE #Events  
    (  
      ServiceCID    ClassID NULL,  
      ServiceID     ObjID NULL,    
      ObjectCID     ClassID NULL,  
      ObjectID      ObjID NULL,  
      EventCID      ClassID NULL,  
      EventID       ObjID NULL,  
      ParentCID     ClassID NULL,  
      ParentID      ObjID NULL,  
      AlertStateID  INT NULL,  
      EventTime     DATETIME NULL,  
      priority      INT NULL  
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #Events', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#LocalServices') IS NULL
  BEGIN 
    CREATE TABLE #LocalServices
    (
      ServiceCID    ClassID NOT NULL,
      ServiceID     ObjID NOT NULL, 
      cid           ClassID NOT NULL,  
      id            ObjID NOT NULL,
      lob_id        ObjID NOT NULL,
      DisplayName   ObjName NULL,
      Name          NVARCHAR(255) NULL,
      Type          NVARCHAR(40),
      Path          NVARCHAR(1000) NOT NULL,
      ResourceState NVARCHAR(80) NULL,
      AlertStateID  INT DEFAULT 0,
      gen           INT NOT NULL,
      numviolations INT DEFAULT 0,
      lob_flags     INT DEFAULT 0,
      idpath        NVARCHAR(400) NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #LocalServices', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#ExtStatusViewRecord') IS NULL
  BEGIN 
    CREATE TABLE #ExtStatusViewRecord
    (
      cid                       NVARCHAR(4)   NOT NULL, -- ease of use, does not have to be returned to api 
      id                        INT           NOT NULL,
      recordType                NVARCHAR(48)  NOT NULL, -- AVAIL, STATE, SLA
      name                      NVARCHAR(32)  NOT NULL, -- Availability, Object State, SLAState
      description               NVARCHAR(256) NULL,     -- return null 
      state                     INT           NULL,     -- these should probably not be integers need to carry the 
      stateDescription          NVARCHAR(256) NULL,     -- this can be the string for the current stateid and the string for alertstate
      timeEnteredState          DATETIME      NULL,     -- available through history?
      totalNumberOfIncidents    INT           NULL, 
      LatestRootCauseEventTime  DATETIME      NULL,   -- Latest event time  
      prev_state                INT           NULL,     -- these should probably not be integers need to carry the 
      prev_stateDescription     NVARCHAR(256) NULL,     -- this can be the string for the current stateid and the string for alertstate
      detailLevel               INT           NULL      -- set to 100
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #ExtStatusViewRecord', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#ExtBusinessImpactInfo') IS  NULL
  BEGIN 
    CREATE TABLE #ExtBusinessImpactInfo
    (
      lobcid           NVARCHAR(4)   NULL, 
      lobid            INT           NULL, 
      cid              NVARCHAR(4)   NOT NULL,
      id               INT           NOT NULL,
      BusinessRole     NVARCHAR(500) NULL,
      RedImpact        NVARCHAR(500) NULL,
      YellowImpact     NVARCHAR(500) NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #ExtBusinessImpactInfo', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#summary') IS NULL
  BEGIN
    CREATE TABLE #summary
    (
      cid                  ClassID,
      id                   ObjID,
      LastSLAViolationTime DATETIME  DEFAULT NULL NULL,
      NumberSLAViolations  INT       DEFAULT 0 NULL,
      LastSLATrendTime     DATETIME  DEFAULT NULL NULL,
      NumberSLATrends      INT       DEFAULT 0 NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #summary', 0, 1, @now) WITH NOWAIT
    END
  END

---- NOW FOR API OUTPUT TABLES 

  IF OBJECT_ID('tempdb..#ManagedElements') IS NULL
  BEGIN 
    CREATE TABLE #ManagedElements
    (
      cid                NVARCHAR(4)   NOT NULL,  -- ease of use, does not have to be returned to api 
      id                 INT           NOT NULL,
      name               NVARCHAR(128) NULL,
      description        NVARCHAR(256) NULL,
      caption            NVARCHAR(256) NULL,      -- leave in case we use this to put 'other info', return null 
      createTS           DATETIME      NULL,
      modifiedTS         DATETIME      NULL,
      deletedTS          DATETIME      NULL,
      hasChildren        TINYINT       NULL,      -- 0 if false, 1 if true 
      hasSLA             SMALLINT      NULL,      -- ???? negative if not apply, 0  if false, 1 if true
      detailLevel        INT           NOT NULL,  -- set to 100
      physInLob          SMALLINT      NOT NULL,  -- negative if not a physical object, 0 if not associated with an lob, 1 if it is
      lobIsLinked        SMALLINT      NOT NULL,  -- negative if not an LOB, 0 if not associated with a physical object, 1 if it is
      svcIsLinked        SMALLINT      NOT NULL,  -- negative if not a service, 0 if not associated with a physical object, 1 if it is
      linkedCid          ClassID       NULL,      -- for lob and service objects: the class of the physical object it is linked to, if it is linked
      linkedId           ObjID         NULL,      -- for lob and service objects: the object id of the physical object it is linked to, if it is linked
      lobCid             ClassID       NULL,      -- for service objects: the linked lob class
      lobId              ObjID         NULL       -- for service objects: the linked id
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #ManagedElements', 0, 1, @now) WITH NOWAIT
    END
  END 

  IF OBJECT_ID('tempdb..#tempcidids') IS  NULL
  BEGIN 
    CREATE TABLE #tempcidids
    (
      cid ClassID NOT NULL,
      id  ObjID   NOT NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_availDetails: Created worktable #tempcidids', 0, 1, @now) WITH NOWAIT
    END
  END 

  -- Create indexes

  IF NOT EXISTS (SELECT 1 FROM tempdb..sysindexes WHERE id = OBJECT_ID('tempdb..#ManagedElements'))
    CREATE INDEX me_index ON #ManagedElements(cid, id)

  IF NOT EXISTS (SELECT 1 FROM tempdb..sysindexes WHERE id = OBJECT_ID('tempdb..#tempcidids'))
    CREATE INDEX cidid_index ON #tempcidids(cid, id)

  IF NOT EXISTS (SELECT 1 FROM tempdb..sysindexes WHERE id = OBJECT_ID('tempdb..#TempEvents'))
    CREATE INDEX evcidid_index ON #TempEvents(cid, id)

  ---make sure call is for a service ... 
  IF @cid <> 'EXES' GOTO NotService


  ---- ++++++++++ Processing for a cid that is a service  +++++++
  --- get the LOB for the incoming Service 

  SET @ServiceLobCid=NULL
  SET @testPath=NULL
  SET @ServiceLobId=0

  IF @cid='EXES'
  BEGIN 

    --- get the lob cid and id 
    SELECT @ServiceLobCid=dst_cid, @ServiceLobId=dst_id
    FROM link AS l WITH (NOLOCK)
    WHERE l.deleted=0 AND src_cid=@cid AND src_id=@id AND dst_cid='LOB' AND link_type='EVSL'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getAvailDetails: Service LOB id = %d', 0, 1, @now, @ServiceLobId) WITH NOWAIT
    END
        
    IF @ServiceLobId <> 0
    BEGIN 
      SELECT @testPath=idpath
      FROM ObjPathCache AS o WITH (NOLOCK)
      WHERE o.cid=@ServiceLobCid AND o.id=@ServiceLobId
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: api_getAvailDetails: Service LOB path = ''%s''', 0, 1, @now, @testPath) WITH NOWAIT
      END
    END
  
    IF @testPath IS NULL
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: api_getAvailDetails: ObjPathCache has not been setup with Service info yet', 0, 1, @now) WITH NOWAIT
      END
      GOTO GetOut
    END 

  END 

  --- get the root cause events ... 
  
  --- handle not found case later on 
--   PRINT @ServiceLobCid
--   PRINT @ServiceLobId

  EXEC @rc = GetRootCauseResources
    @cid            = @ServiceLobCid, 
    @id             = @ServiceLobId , 
    @cnt            = @sub OUTPUT,  
    @verbose        = @verbose, 
    @rscTableName   = '#r', 
    @getEvents      = 1, 
    @eventTableName = '#TempEvents'

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: GetRootCauseResources returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- set up and call getEventDetails for the cids/ids returned from getallsubservices 
  TRUNCATE TABLE #tempcidids  
  INSERT INTO #tempcidids SELECT cid, id FROM #TempEvents
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Prepared input table for api_getEventDetails with %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  EXEC @rc = api_getEventDetails @verbose = @verbose

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails:  api_getEventDetails returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- now go get the list of subservices under the main service 

  -- The next proc looks for a table #Services with at least and id column for the list of EXES to deal with  
  EXEC @rc = asisp_GetAllSubServices
    @cid                      = @cid,
    @id                       = @id,
    @cnt                      = @count OUTPUT,
    @results                  = 1,
    @Levels                   = 1000,
    @ExcludeServiceResources  = 0,
    @outputType               = 1,
    @outputTable              = '#AvailRelationships',
    @verbose                  = @verbose

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: asisp_GetAllSubServices returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- set up and call getManagedElements for the cids/ids returned from getallsubservices 
  TRUNCATE TABLE #tempcidids
  --- put the list of cids and ids into the list that we will wante returned with in the managed element list 

  SET @AlertLevelToReturn=2 --- says alert states of 2 and above can be returned
  IF @IncludeClosedProblems<>0
  BEGIN 
    SET @AlertLevelToReturn=1 --- says alert states of 1 and above can be returned
  END 

  INSERT INTO #tempcidids 
  SELECT DISTINCT cid, id
  FROM 
  (
    SELECT DISTINCT target_cid AS cid, target_id AS id FROM #AvailRelationships r           ---- the list of all sub services and sub service resources
            JOIN LineOfBusiness_V v ON v.id = r.lob_id AND  _AlertStateID >= @AlertLevelToReturn  
        
    UNION 
    SELECT DISTINCT resource_cid AS cid , resource_id AS id FROM #ExtIncidentRecord     --- this list of cids/ids that actually have events against them 
       WHERE resource_cid <> 'LOB' OR (resource_cid='LOB' AND resource_id NOT IN (SELECT lob_id FROM #AvailRelationships  UNION SELECT @ServiceLobId ) )
   
    UNION 
    SELECT DISTINCT    ChildCID AS cid, ChildID AS id FROM #r WHERE RootCause > 0 AND ChildCID = 'LOB' AND ChildID NOT IN (SELECT lob_id FROM  #AvailRelationships UNION SELECT @ServiceLobId) 

    UNION 
    SELECT @cid AS cid, @id AS id     --- put in starting point                         --- the starting point service 
  ) AS derived
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Prepared input table for api_getManagedElements with %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  EXEC @rc = api_getManagedElements NULL, NULL, @verbose = @verbose
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: api_getManagedElements returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- now get those same resources statuses
  EXEC @rc = api_getStatusView NULL, 0, @verbose = @verbose
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: api_getStatusView returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  ---- now get the impacts that are represented in the events to be returned ... this is the paths from the LOB that is linked to the physical resources back to the starting point service 

  INSERT  INTO #ImpactedPaths (r.cid, r.id, r.idpath)  
    SELECT LTRIM(RTRIM(ChildCID)), ChildID, respath FROM #r AS r WHERE RootCause =1 

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: %d impacted path(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE i SET
    path         = o.resourcepath,
    resourceName = o.resource
  FROM #ImpactedPaths AS i WITH(NOLOCK)
  JOIN ObjPathCache AS o WITH(NOLOCK) ON o.idpath = i.idpath
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Found %d ObjPathCache row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- now update the links   
  UPDATE i SET linked_cid=dst_cid , linked_id=dst_id FROM #ImpactedPaths i WITH(NOLOCK), link l WITH(NOLOCK)
  WHERE i.cid='EXES' AND l.src_cid='EXES'  AND i.id=l.src_id AND l.deleted=0   
  AND link_type='EVSL' AND l.dst_cid = 'LOB'   
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Found %d link row(s)', 0, 1, @now) WITH NOWAIT
  END

  UPDATE i SET linked_cid=dst_cid , linked_id=dst_id FROM #ImpactedPaths i WITH(NOLOCK), link l WITH(NOLOCK)   
  WHERE l.deleted=0 AND l.src_cid='LOB' AND link_type='LOBL' AND l.dst_cid <>'LOB'   
    AND i.cid='LOB' AND i.id=l.src_id 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Found %d LOBL link(s)', 0, 1, @now) WITH NOWAIT
  END

  -- Links up the services and the lobs to their physical object if there is one  
  UPDATE i SET linked_cid=dst_cid , linked_id=dst_id
  FROM #ImpactedPaths i WITH(NOLOCK), link l WITH(NOLOCK)   
  WHERE l.deleted=0 AND l.src_cid='LOB' AND link_type='LOBL' AND l.dst_cid <>'LOB'   
     AND i.cid='EXES' AND i.linked_cid='LOB' AND i.linked_id=l.src_id  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Found %d links to phy objects for services', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF ( @IncludeNotesType = 1)  
  BEGIN     
    -- Only ownership notes whose owning resource is in the impacted paths 

    INSERT INTO #ExtNote ( note_cid, note_id,type,ctime,subject,TicketNumber, TicketOpenTime)        
      SELECT DISTINCT 'NOTE',  v.id, 'Ownership', v._Date, v._Subject, LTRIM(RTRIM(v._ProblemID)), v._Time
 	 FROM  
	    OwnedEvent owned WITH (NOLOCK),   
	    #ExtIncidentRecord e  WITH (NOLOCK),   
	    Note_V v WITH (NOLOCK)
	  WHERE   
	    owned.eventcid = e.incident_cid   
	    AND owned.eventid  = e.incident_id   
	    AND v.id = owned.noteid  

    UPDATE #ExtNote SET TicketNumber=NULL, TicketOpenTime=NULL WHERE TicketNumber = '' OR TicketNumber IS NULL
         
  END  

  -- Now fix up the events for services
  UPDATE e SET 
    resource_cid = m.cid,
    resource_id  = m.id
  FROM #ExtIncidentRecord AS e
  JOIN #ManagedElements AS m ON m.lobCid = e.resource_cid AND m.lobId = e.resource_id
  WHERE m.cid='EXES'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Looked up resource information for %d incident row(s) in result set', 0, 1, @now, @rc) WITH NOWAIT
  END

NotService:
GetOut:

  SELECT
    'ManagedElements',
    'ImpactedPaths',
    'ExtIncidentRecord',
    'ExtNote',
    'ExtStatusViewRecord',
    'ExtBusinessImpactInfo',
    'SLAStatusSummary'

  SELECT * FROM #ManagedElements
  SELECT * FROM #ImpactedPaths
  SELECT * FROM #ExtIncidentRecord
  SELECT * FROM #ExtNote
  SELECT * FROM #ExtStatusViewRecord
  SELECT * FROM #ExtBusinessImpactInfo
  SELECT * FROM #summary

GetOutNoResults:
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getAvailDetails: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
