-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\api_getmanagedelements.sql
-- SID: 1.14
-- Date Modified: 04/11/15 16:14:29
-- 
-- Module Description
--   Defines the api_getManagedElements stored procedure
--

USE Object
GO
   
      
-- api_getManagedElements
PRINT 'Creating procedure api_getManagedElements'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'api_getManagedElements' AND type = 'P')
  DROP PROCEDURE api_getManagedElements
GO 

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/api_getmanagedelements.sql
-- SID: 1.14
-- Date Modified: 04/11/15 16:14:29
--
-- Retrieves Managed Elemenent info for the tbsm api
--
--

--
--
-- Input parameters:
-- @cid = cid (LOB or EXES) of object 
   --
-- @id = id of object
--
-- @verbose >= 1 will execute this procedure in verbose mode
--
-- Optional input temp table with cid/id (or provide @cid and @id)
/*
   CREATE TABLE #tempcidids
   (
   cid      ClassID,
   id       ObjID
   )
*/   
--
CREATE PROCEDURE api_getManagedElements
  @cid     ClassID = NULL,
  @id      ObjID   = NULL,
  @verbose TINYINT = 0
AS
  SET NOCOUNT ON 

  CREATE TABLE #ServiceChildrenInfo
  (
    source_cid        ClassID NULL,
    source_id         INT     NULL, 
    children_type     INT     NULL, 
    numberOfChildren  INT     NULL
  ) 

  DECLARE
    @count          INT,
    @err            INT,
    @now            NVARCHAR(40),
    @rc             INT,
    @sendoutResults TINYINT,
    @sql            NVARCHAR(1000)

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: @cid=''%s'', @id=%d: Starting', 0, 1, @now, @cid, @id) WITH NOWAIT
  END
  
  IF @cid IS NULL
  BEGIN 
    IF OBJECT_ID('tempdb..#tempcidids') IS NULL
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_GetManagedElements requires cid and id to be specified if #tempcidids or in parameters', 11, 1, @now) WITH NOWAIT
      RETURN -1
    END 
  END 
  ELSE
  BEGIN 

    IF OBJECT_ID('tempdb..#tempcidids') IS NULL
    BEGIN
      CREATE TABLE #tempcidids
      (
        cid ClassID NOT NULL,
        id  ObjID   NOT NULL
      ) 
      CREATE INDEX cidid_index ON #tempcidids(cid, id)
    END 
    ELSE
    BEGIN 
      TRUNCATE TABLE #tempcidids
    END 
    
    INSERT INTO #tempcidids VALUES ( @cid, @id )

  END 

  /* #ManagedElements is the result set we'll return to the caller */
  SET @sendoutResults = 0
  IF OBJECT_ID('tempdb..#ManagedElements') IS NULL
  BEGIN 
    CREATE TABLE #ManagedElements
    (
      cid                NVARCHAR(4)   NOT NULL,  -- ease of use, does not have to be returned to api 
      id                 INT           NOT NULL,
      name               NVARCHAR(128) NULL,
      description        NVARCHAR(256) NULL,
      caption            NVARCHAR(256) NULL,      -- leave in case we use this to put 'other info', return null 
      createTS           DATETIME      NULL,
      modifiedTS         DATETIME      NULL,
      deletedTS          DATETIME      NULL,
      hasChildren        TINYINT       NULL,      -- 0 if false, 1 if true 
      hasSLA             SMALLINT      NULL,      -- ???? negative if not apply, 0  if false, 1 if true
      detailLevel        INT           NOT NULL,  -- set to 100
      physInLob          SMALLINT      NOT NULL,  -- negative if not a physical object, 0 if not associated with an lob, 1 if it is
      lobIsLinked        SMALLINT      NOT NULL,  -- negative if not an LOB, 0 if not associated with a physical object, 1 if it is
      svcIsLinked        SMALLINT      NOT NULL,  -- negative if not a service, 0 if not associated with a physical object, 1 if it is
      linkedCid          ClassID       NULL,      -- for lob and service objects: the class of the physical object it is linked to, if it is linked
      linkedId           ObjID         NULL,      -- for lob and service objects: the object id of the physical object it is linked to, if it is linked
      lobCid             ClassID       NULL,      -- for service objects: the linked lob class
      lobId              ObjID         NULL       -- for service objects: the linked id
    )
    CREATE INDEX me_index ON #ManagedElements(cid, id)
    SET @sendoutResults = 1 
  END 

  /* the following are used to capture results from procedures that will be called */

  IF OBJECT_ID('tempdb..#ServiceStatus') IS NULL
  BEGIN
    CREATE TABLE #ServiceStatus
    (
      cid               ClassID NOT NULL,    -- always set to EXES (Service class)
      id                ObjID   NOT NULL,    -- id of the Service
      ParentCID         ClassID NULL,
      ParentID          ObjID   NULL,
      ServicePath       NVARCHAR(1000) NULL, 
      DisplayName       ObjName  NULL,
      flags             TINYINT NULL,
      LOBDeletedTime    DATETIME NULL,
      HasSubServices    TINYINT NULL,
      AlertStateID      INT NULL,
      StatusTimeStamp   DATETIME NULL,
      NumEvents         INT  NULL,
      lob_id            ObjID NOT NULL
    )
  END

  IF OBJECT_ID('tempdb..#ServiceAttrs') IS NULL
  BEGIN
    CREATE TABLE #ServiceAttrs
    (
      cid               ClassID NOT NULL,    -- always set to EXES (Service class)
      id                ObjID   NOT NULL,    -- id of the Service
      BusinessRole      NVARCHAR(500) NULL,
      RedImpact         NVARCHAR(500) NULL,
      YellowImpact      NVARCHAR(500) NULL
    ) 
  END

  INSERT INTO #ManagedElements (cid, id, hasChildren, hasSLA, detailLevel, physInLob, lobIsLinked, svcIsLinked)
  SELECT DISTINCT
    cid, 
    id, 
    0,    --- set children be default to null 
    CASE cid
    WHEN 'EXES' THEN 0
    ELSE -1
    END,
    100,
    -1,
    -1,
    -1
  FROM #tempcidids
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of managed elements', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- first take care of LOBs 
  UPDATE me SET hasChildren=1 
  FROM #ManagedElements AS me
    JOIN LineOfBusiness_C AS l WITH (NOLOCK) ON me.cid='LOB' AND l.id=me.id AND l.nchild>0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated hasChildren for %d LOB row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- now take care of physical objects ... 

  UPDATE me SET hasChildren=
    CASE WHEN EXISTS
    (
      SELECT 1 FROM link AS l WITH (NOLOCK)
        JOIN link_type AS lt WITH (NOLOCK) ON l.link_type = lt.link_type 
      WHERE l.src_cid = cid AND l.src_id = id AND l.deleted = 0 AND lt.flag_parent_child = 1
    )
    THEN 1
    ELSE 0
    END
  FROM #ManagedElements me
  WHERE me.cid NOT IN ('EXES', 'LOB')  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated hasChildren for %d non-LOB row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- lastly take care of services ...

  --- first set up the call to the routine to do the work asisp_GetAllSubServices

  EXEC @rc = asisp_GetAllSubServices
    @cid=null, 
    @id=null,
    @cnt=@count OUTPUT, 
    @results = 1,      -- return result set if set to 1 
    @Levels =1, 
    @outputType=2 , 
    @outputTable = '#ServiceChildrenInfo', 
    @inputTable = '#tempcidids',
    @verbose = @verbose
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: asisp_GetAllSubServices returned %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE me SET hasChildren=1
  FROM #ManagedElements me
    JOIN #ServiceChildrenInfo s ON s.source_cid=me.cid AND s.source_id=me.id AND numberOfChildren>0 
  WHERE me.cid =  'EXES'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated hasChildren for %d row(s) of services', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- set the phsInLob link to 1 if the resource type is not LOB or EXES (therefore it will be a physical object and 
  -- it is linked to a LOB
  UPDATE me SET 
    physInLob = CASE
      WHEN l.link_type = 'LOBL' THEN 1
      ELSE 0
    END
  FROM #ManagedElements AS me
    LEFT OUTER JOIN link AS l WITH(NOLOCK) ON l.dst_cid = me.cid AND l.dst_id = me.id AND l.link_type = 'LOBL' AND l.deleted = 0
  WHERE me.cid NOT IN ('LOB','EXES')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated physInLob for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  /* set the lobIsLinked to 1 if the object is an LOB and it is linked to a physical object */
  UPDATE me SET 
    lobIsLinked = CASE
      WHEN l.link_type = 'LOBL' AND l.dst_cid <> 'LOB' THEN 1
      ELSE 0
    END,
    linkedCid = l.dst_cid,
    linkedId = l.dst_id
  FROM #ManagedElements me
    LEFT OUTER JOIN link l WITH(NOLOCK) ON l.src_cid = me.cid  AND l.src_id = me.id AND l.link_type = 'LOBL' AND l.deleted = 0
  WHERE me.cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated lobIsLinked for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  /* set svcIsLinked to 1 if the cid is a Service (EXES) if the lob it is linked to is linked to a physical object */
  UPDATE me SET 
    svcIsLinked = CASE
      WHEN l2.link_type = 'LOBL' AND l2.dst_cid <> 'LOB' THEN 1
      ELSE 0
    END,
    linkedCid = l2.dst_cid,
    linkedId = l2.dst_id, 
    lobCid    = l1.dst_cid, 
    lobId     = l1.dst_id
  FROM #ManagedElements me
    LEFT OUTER JOIN link l1  WITH(NOLOCK) ON l1.src_cid = me.cid AND l1.src_id = me.id AND l1.link_type = 'EVSL' AND l1.deleted = 0
    LEFT OUTER JOIN link l2  WITH(NOLOCK) ON l2.src_cid = l1.dst_cid AND l2.src_id = l1.dst_id  AND l2.link_type = 'LOBL' AND l2.deleted = 0
  WHERE me.cid = 'EXES'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated svcIsLinked for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  DECLARE class_cursor CURSOR FOR
    SELECT DISTINCT cid FROM #ManagedElements
  OPEN class_cursor
  FETCH NEXT FROM class_cursor INTO @cid
  WHILE @@FETCH_STATUS = 0
  BEGIN
    -- build up end of the SQL statement, which requires getting cname through a SELECT
    SELECT @sql = '
          name        = v.name,
          createTS    = v.ctime,
          modifiedTS  = v.mtime,
          deletedTS   = CASE v.deleted WHEN 1 THEN v.mtime ELSE NULL END
        FROM #ManagedElements AS me
        JOIN ' + cname + '_V AS v WITH(NOLOCK) ON v.id = me.id
        WHERE me.cid = ''' + RTRIM(@cid) + ''''
    FROM Meta..obj_class_table_BASE
    WHERE cid = @cid

    -- if _Desc is available, use it
    IF EXISTS (SELECT 1 
                FROM syscolumns c 
                JOIN sysobjects o ON o.id = c.id
                JOIN Meta..obj_class_table_BASE oc ON o.name = oc.cname + '_V'
                WHERE c.name = '_Desc' AND oc.cid = @cid
              )
      SET @sql = '
          description = v._Desc,' + @sql
    ELSE IF EXISTS
    (
      SELECT 1 
      FROM syscolumns c 
      JOIN sysobjects o ON o.id = c.id
      JOIN Meta..obj_class_table_BASE oc ON o.name = oc.cname + '_V'
      WHERE c.name = 'description' AND oc.cid = @cid
    ) -- otherwise, try to use 'description', which most classes have
      SET @sql = '
          description = v.description,' + @sql

    -- finally, add the unconditional frontend of the statement and run it
    SET @sql = '
        UPDATE me SET' + @sql
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_GetAllSubServices: Updated %d row(s) for class %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END
    FETCH NEXT FROM class_cursor INTO @cid
  END
  DEALLOCATE class_cursor

  -- now set the hasSLA flag for the services 

  UPDATE me SET hasSLA = CASE
                          WHEN (ss._flags & 1) = 1 THEN 1
                          ELSE 0
                        END
    FROM #ManagedElements me
    JOIN EVService_C ss WITH(NOLOCK) ON me.cid = ss.cid AND me.id = ss.id
  WHERE me.cid='EXES'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated hasSLA for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @sendoutResults = 1
  BEGIN 
    SELECT 'ManagedElements'
    SELECT * FROM #ManagedElements
  END 
  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
