-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\api_getstatusview.sql
-- SID: 1.14
-- Date Modified: 04/11/15 15:02:25
-- 
-- Module Description
--   Defines the api_getStatusView stored procedure
--

USE Object
GO

PRINT 'Creating procedure api_getStatusView'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'api_getStatusView' AND type = 'P')
  DROP PROCEDURE api_getStatusView
GO 

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\api_getstatusview.sql
-- SID: 1.14
-- Date Modified: 04/11/15 15:02:25
-- 
-- Retrieves State information for the tbsm api 
--
--
-- Input parameters:
-- @cid = cid (LOB or EXES) of object to retrieve SLA impact info against
--
-- @id = id of object to retrieve SLA impact info against 
--
-- @verbose >= 1 will execute this procedure in verbose mode
--
-- Optional input temp table with cid/id (or provide @cid and @id)
/*
   CREATE TABLE #tempcidids
   (
   cid      ClassID,
   id       ObjID
   )
*/   
--
CREATE PROCEDURE api_getStatusView
  @cid      ClassID = NULL,
  @id       ObjID   = NULL,
  @verbose  TINYINT = 0
AS
  SET NOCOUNT ON 

  CREATE TABLE  #r
  (
    ServiceCID ClassID        NULL, 
    ServiceID  ObjID          NULL, 
    ChildCID   ClassID        NULL, 
    ChildID    ObjID          NULL, 
    respath    NVARCHAR(1000) NOT NULL,
    RootCause  INT            NOT NULL DEFAULT 0  
  )

  CREATE TABLE #RootCauseServices    
  (    
    ServiceCID  ClassID       NULL,    
    ServiceID   ObjID         NULL,    
    cid         ClassID       NOT NULL,    
    id          ObjID         NOT NULL
  )

  DECLARE
    @areServices          TINYINT,
    @cname                NVARCHAR(24),
    @err                  INT,
    @now                  NVARCHAR(40),
    @rc                   INT,
    @sql                  NVARCHAR(2000),
    @sendOutStatusView    TINYINT,
    @serviceContainerMask INT,
    @sub                  INT,
    @timeCutOff           DATETIME

  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Starting', 0, 1, @now) WITH NOWAIT
  END 

  /* The following are the result sets we'll return to the caller */
  SET @sendOutStatusView=0 
  IF OBJECT_ID('tempdb..#ExtStatusViewRecord') IS NULL
  BEGIN 
    CREATE TABLE #ExtStatusViewRecord
    (
      cid                       NVARCHAR(4)   NOT NULL, -- ease of use, does not have to be returned to api 
      id                        INT           NOT NULL,
      recordType                NVARCHAR(48)  NOT NULL, -- AVAIL, STATE, SLA
      name                      NVARCHAR(32)  NOT NULL, -- Availability, Object State, SLAState
      description               NVARCHAR(256) NULL,     -- return null 
      state                     INT           NULL,     -- these should probably not be integers need to carry the 
      stateDescription          NVARCHAR(256) NULL,     -- this can be the string for the current stateid and the string for alertstate
      timeEnteredState          DATETIME      NULL,     -- available through history?
      totalNumberOfIncidents    INT           NULL, 
      LatestRootCauseEventTime  DATETIME      NULL,     -- Latest event time  
      prev_state                INT           NULL,     -- these should probably not be integers need to carry the 
      prev_stateDescription     NVARCHAR(256) NULL,     -- this can be the string for the current stateid and the string for alertstate
      detailLevel               INT           NULL      -- set to 100
    )
    SET @sendOutStatusView=1
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #ExtStatusViewRecord', 0, 1, @now) WITH NOWAIT
    END 
  END 

  IF OBJECT_ID('tempdb..#ExtBusinessImpactInfo') IS NULL
  BEGIN 
    CREATE TABLE #ExtBusinessImpactInfo
    (
      lobcid           NVARCHAR(4)   NULL,
      lobid            INT           NULL,
      cid              NVARCHAR(4)   NOT NULL,
      id               INT           NOT NULL,
      BusinessRole     NVARCHAR(500) NULL,
      RedImpact        NVARCHAR(500) NULL,
      YellowImpact     NVARCHAR(500) NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #ExtBusinessImpactInfo', 0, 1, @now) WITH NOWAIT
    END
  END

  /* the following are used to capture results from procedures that will be called */

  IF OBJECT_ID('tempdb..#summary') IS NULL
  BEGIN
    CREATE TABLE #summary
    (
      cid                  ClassID    NOT NULL,
      id                   ObjID      NOT NULL,
      LastSLAViolationTime DATETIME   NULL DEFAULT NULL,
      NumberSLAViolations  INT        NULL DEFAULT 0,
      LastSLATrendTime     DATETIME   NULL DEFAULT NULL,
      NumberSLATrends      INT        NULL DEFAULT 0
    )  
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #summary', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#ServiceStatus') IS NULL
  BEGIN

    CREATE TABLE #ServiceStatus  
    (  
      cid                       ClassID         NOT NULL,    -- always set to EXES (Service class)  
      id                        ObjID           NOT NULL,    -- id of the Service  
      ParentCID                 ClassID         NULL,
      ParentID                  ObjID           NULL,  
      ServicePath               NVARCHAR(1000)  NULL,   
      DisplayName               ObjName         NULL,
      flags                     TINYINT         NULL,
      LOBDeletedTime            DATETIME        NULL,  
      HasSubServices            TINYINT         NOT NULL DEFAULT 0,  
      AlertStateID              INT             NULL,  
      StatusTimeStamp           DATETIME        NULL,  
      NumEvents                 INT             NULL,
      lob_id                    ObjID           NOT NULL,
      LatestRootCauseEventTime  DATETIME        NULL -- Latest event time  
    )   
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #ServiceStatus', 0, 1, @now) WITH NOWAIT
    END
  END

  IF OBJECT_ID('tempdb..#ServiceAttrs') IS NULL
  BEGIN
    CREATE TABLE #ServiceAttrs
    (
      cid               ClassID       NOT NULL,    -- always set to EXES (Service class)
      id                ObjID         NOT NULL,    -- id of the Service
      BusinessRole      NVARCHAR(500) NULL,
      RedImpact         NVARCHAR(500) NULL,
      YellowImpact      NVARCHAR(500) NULL
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #ServiceAttrs', 0, 1, @now) WITH NOWAIT
    END
  END

  /*
  'LOB', 1 ,  'AVAIL',  'Availability'  ,  NULL,  _AlertStateID,  timeenteredstate
  'LOB', 1,   'STATE'  , 'Object State',       NULL,    , currentStateID


  (gotten from _GetServiceStatus and  _GetSLAImpactSummaryInfo )  see proc for output formatting, 
  'EXES', 1 ,  'AVAIL',  'Availability'  ,  NULL,  _AlertStateID (from _GetServiceStatus),  timeenteredstate
  'EXES', 1,   'SLA',     'SLAState',      NULL,   (derived from _GetSLAImpactSummary)
  */

  /*  FIRST TAKE CARE OF SERVICES   */


  /* see if flag is set to call to get sla info */ 
  -- Set up call to _GetServiceStatus for bulk Services 
  IF OBJECT_ID('tempdb..#ServiceIds') IS NULL
  BEGIN 
    --- for now create it twice, although Latha is changing 
    CREATE TABLE #ServiceIds
    (
      cid ClassID, 
      id  ObjID
    )
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Created worktable #ServiceIds', 0, 1, @now) WITH NOWAIT
    END
  END 
  ELSE
  BEGIN 
    TRUNCATE TABLE #ServiceIds
  END 

  IF @cid IS NULL
  BEGIN 
    IF OBJECT_ID('tempdb..#tempcidids') IS NULL
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView requires cid and id to be specified if #tempcidids or in parameters', 18, 1, @now) WITH NOWAIT
      RETURN -1
    END 
  END 
  ELSE
  BEGIN 

    IF OBJECT_ID('tempdb..#tempcidids') IS NULL
    BEGIN
      CREATE TABLE #tempcidids
      (
        cid ClassID NOT NULL,
        id  ObjID NOT NULL
      ) 
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: api_getStatusView: Created worktable #tempcidids', 0, 1, @now) WITH NOWAIT
      END
      CREATE INDEX cidid_index ON #tempcidids(cid, id)
    END 
    ELSE
    BEGIN 
      TRUNCATE TABLE #tempcidids
    END 

    INSERT INTO #tempcidids VALUES ( @cid, @id )

  END 

  SET @areServices=0

  INSERT INTO #RootCauseServices 
    SELECT t.cid, t.id, l.dst_cid, l.dst_id
    FROM #tempcidids AS t 
      JOIN link AS l WITH (NOLOCK) ON l.link_type='EVSL' AND l.src_cid='EXES' AND l.src_id=t.id AND l.dst_cid='LOB'
    WHERE t.cid = 'EXES'
      AND l.deleted = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Found %d service(s)', 0, 1, @now, @rc) WITH NOWAIT
  END 

  SET @areServices=@rc
  IF @areServices > 0
  BEGIN 

    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Starting to process services', 0, 1, @now) WITH NOWAIT
    END 

    SET @timeCutOff = '2029-01-01 00:00:01.000'
    INSERT INTO #ExtStatusViewRecord ( cid, id, recordType,name,  state, stateDescription, timeEnteredState, totalNumberOfIncidents, prev_state,  detailLevel)
      SELECT DISTINCT d2.cid, d2.id, 'AVAIL', 'Availability', ah.AlertStateID, NULL, ah.ctime, NULL, ah.OldAlertStateID, 100  
      FROM AlertStateHistory AS ah WITH (NOLOCK)
      RIGHT OUTER JOIN 
      ( 
        SELECT t3.cid AS cid, t3.id AS id, ISNULL(derived.ash_id, 0) AS ash_id 
        FROM (
          SELECT a.cid AS cid, a.id AS id, MAX(a.ash_id) AS ash_id
          FROM AlertStateHistory AS a WITH (NOLOCK)
            JOIN #RootCauseServices AS t ON t.cid = a.cid AND t.id = a.id
          WHERE a.ctime < @timeCutOff AND a.AlertStateID <> a.OldAlertStateID 
          GROUP BY a.cid, a.id
          UNION
          SELECT 'AIF' AS cid, 0 AS id, 0 AS ash_id
        ) AS derived 
        RIGHT OUTER JOIN #RootCauseServices AS t3 WITH (NOLOCK) ON t3.cid = derived.cid AND t3.id = derived.id 
      ) AS d2
      ON d2.cid <> 'AIF' AND d2.cid = ah.cid AND d2.id = ah.id AND d2.ash_id = ah.ash_id AND ah.AlertStateID <> ah.OldAlertStateID 
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Inserted %d row(s) from RootCauseServices/AlertStateHistory join', 0, 1, @now, @rc) WITH NOWAIT
    END 

    --- this next statement get something just in case there is not any entries in the AlertStateHistory table though it is not necessarily valid
    ---   maybe in this case leave as is 

    UPDATE e SET
      state = v._AlertStateID,
      timeEnteredState=v.ctime,
      prev_state = NULL
    FROM #ExtStatusViewRecord AS e
      JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.id = e.id 
    WHERE e.state IS NULL
      AND e.cid = 'LOB'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Updated states for %d row(s) that were not in AlertStateHistory table', 0, 1, @now, @rc) WITH NOWAIT
    END 

    --- change cid and ids of services back ... 
    UPDATE s SET cid=t.cid, id=t.id
    FROM #ExtStatusViewRecord AS s
      JOIN link AS l WITH (NOLOCK) ON l.dst_cid=s.cid AND l.dst_id = s.id         
      JOIN #tempcidids AS t ON l.src_cid=t.cid AND l.src_id=t.id 
    WHERE l.deleted = 0 AND l.link_type='EVSL' AND l.src_cid='EXES'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Updated cids and ids for service(s)', 0, 1, @now, @rc) WITH NOWAIT
    END 

    --- NOW FIGURE OUT THE LATEST CHANGE OF EACH OF ITS CHILDREN 
    --- first if the alertstate is green then just use its time and the last change
    --- if not, then get children and check there time change ... 
    --- are any other fields necessary?

    -- first update the availability records where the state is green, just use the time it entered the green state, 
    -- this can be done above 
    UPDATE s SET LatestRootCauseEventTime=s.timeEnteredState FROM #ExtStatusViewRecord AS s WHERE s.cid='EXES'

    --- #RootCauseServices is already set up 

    EXEC @rc = GetRootCauseResources
      @cid          = NULL, 
      @id           = NULL, 
      @cnt          = @sub OUTPUT,  
      @verbose      = @verbose, 
      @rscTableName = '#r'
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: GetRootCauseResources returned %d', 0, 1, @now, @rc) WITH NOWAIT
    END 

    --- now get the lastest time entered state of the service children of the main service that are not service resources
    --- however, ignore service resources

    --- service container mask 
    SET @serviceContainerMask = 8192+2048  --- only want LOBs that are active service containers not service resources 

    UPDATE #ExtStatusViewRecord SET LatestRootCauseEventTime = derived.ctime
    FROM 
      (SELECT r.ServiceCID AS ServiceCID, r.ServiceID AS ServiceID, max(ah.ctime) AS ctime  FROM #r r  -- this query give me all children LOBs that are service containers
        JOIN link AS l WITH (NOLOCK) ON l.deleted=0 AND l.link_type='EVSL' AND l.src_cid='EXES' AND l.dst_cid=r.ChildCID AND l.dst_id=r.ChildID
        JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.deleted=0 AND l.dst_id = v.id AND v._flags&@serviceContainerMask =  @serviceContainerMask 
        JOIN AlertStateHistory AS ah WITH (NOLOCK) ON ah.cid=r.ChildCID AND  ah.id=r.ChildID AND ah.AlertStateID <> ah.OldAlertStateID
      WHERE r.ChildCID='LOB' 
      GROUP BY r.ServiceCID, r.ServiceID) AS derived 
    WHERE #ExtStatusViewRecord.cid='EXES' AND #ExtStatusViewRecord.cid = derived.ServiceCID AND  #ExtStatusViewRecord.id = derived.ServiceID 
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Updated LastRootCauseEventTime on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

/* 141513: This is an alternate decomposition of the above update statement. It may perform better,
   but it has not been determined yet whether the performance problem is here.
    DECLARE @rupdate TABLE
    (
      ServiceCID               NVARCHAR(4) NOT NULL,
      ServiceID                INT         NOT NULL,
      LatestRootCauseEventTime DATETIME    NOT NULL
    )
    INSERT INTO @rupdate
      SELECT DISTINCT
        r.ServiceCID,
        r.ServiceID,
        MAX(ah.ctime)
      FROM #r AS r  -- this query give me all children LOBs that are service containers
      JOIN #ExtStatusViewRecord AS x ON x.cid = r.ServiceCID AND x.cid = r.ServiceID
      JOIN link AS l WITH (NOLOCK) ON l.link_type = 'EVSL' AND l.src_cid = r.ServiceID AND l.dst_cid = r.ChildCID AND l.dst_id = r.ChildID
      JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.deleted = 0 AND l.dst_id = v.id
      JOIN AlertStateHistory AS ah WITH (NOLOCK) ON ah.cid = r.ChildCID AND ah.id = r.ChildID
      WHERE r.ChildCID = 'LOB'
        AND x.cid = 'EXES'
        AND l.deleted = 0
        AND v._flags & @serviceContainerMask = @serviceContainerMask 
        AND ah.AlertStateID <> ah.OldAlertStateID
      GROUP BY r.ServiceCID, r.ServiceID
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Computed %d update(s) for LastRootCauseEventTime', 0, 1, @now, @rc) WITH NOWAIT
    END

    UPDATE x SET LatestRootCauseEventTime = r.LatestRootCauseEventTime
    FROM #ExtStatusViewRecord AS x
    JOIN @rupdate AS r ON r.ServiceCID = x.cid AND r.ServiceID = x.id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Updated LastRootCauseEventTime on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
*/

    INSERT INTO #ServiceIds
    SELECT t.cid, t.id
    FROM #tempcidids AS t 
      JOIN EVService_C AS e ON e.cid='EXES' AND t.id=e.id AND e._flags&1 = 1 
    WHERE t.cid='EXES'

    EXEC @rc = _GetSLAImpactSummaryInfo NULL, 0
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: _GetSLAImpactSummaryInfo returned %d', 0, 1, @now, @rc) WITH NOWAIT
    END 

    INSERT INTO #ExtStatusViewRecord (cid, id, recordType, name, state, stateDescription, timeEnteredState, totalNumberOfIncidents, detailLevel)
      SELECT 'EXES', s.id, 'SLA', 'SLAState', 
            CASE
              WHEN s.NumberSLAViolations > 0 THEN 3
              WHEN s.NumberSLATrends > 0 THEN 2
              ELSE 1
            END,
            CASE
              WHEN s.NumberSLAViolations > 0 THEN 'Red'
              WHEN s.NumberSLATrends > 0 THEN 'Yellow'
              ELSE 'Green'
            END,
            CASE
              WHEN s.NumberSLAViolations > 0 THEN LastSLAViolationTime
              WHEN s.NumberSLATrends > 0 THEN LastSLATrendTime
              ELSE NULL
            END,
            s.NumberSLAViolations + s.NumberSLATrends,
            100
      FROM #summary AS s
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Set SLA state on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
    END 

    --- later on Status View records are updated

    EXEC @rc = api_getBusinessImpact @verbose = @verbose
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: api_getBusinessImpact returned %d', 0, 1, @now, @rc) WITH NOWAIT
    END 

  END --- end of @areServices 

  /* ****** DONE TAKING CARE OF SERVICES */

  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: --- Starting loop for each class not a service.', 0, 1, @now) WITH NOWAIT
  END 

  DECLARE class_cursor CURSOR FOR
    SELECT DISTINCT cid FROM #tempcidids WHERE cid <> 'EXES'
  OPEN class_cursor
  FETCH NEXT FROM class_cursor INTO @cid
  WHILE @@FETCH_STATUS = 0
  BEGIN
    SELECT @cname = cname FROM Meta..obj_class_table_BASE WITH (NOLOCK) WHERE cid = @cid
    SET @sql =
    'INSERT INTO #ExtStatusViewRecord (cid, id, recordType, name, state, stateDescription, timeEnteredState, totalNumberOfIncidents,  detailLevel) ' +
      'SELECT t.cid, t.id, ''AVAIL'', ''Availability'', ' +
            'CASE v.deleted ' +
              'WHEN 0 THEN v._AlertStateID ' +
              'ELSE NULL ' +
            'END, ' +
            'NULL,   /*U ---- get a.name later on ...  */ ' +
            'CASE v.deleted ' +
              'WHEN 0 THEN (SELECT MAX(ctime) FROM AlertStateHistory ash WITH (NOLOCK) WHERE ash.cid = t.cid AND ash.id = t.id AND ash.AlertStateID = v._AlertStateID AND ash.AlertStateID <> ash.OldAlertStateID)  /* --- now using t rather than v for join was evt_ctime but not indexed */ ' +
              'ELSE NULL ' +
            'END, ' +
            'NULL, ' +
            'CASE v.deleted ' +
              'WHEN 0 THEN 100 ' +
              'ELSE NULL ' +
            'END ' +
      'FROM #tempcidids AS t ' +
      'JOIN ' + @cname + '_V AS v ON v.id = t.id ' +
      'WHERE t.cid = ''' + RTRIM(@cid) + ''' '
    IF (@verbose >= 2) PRINT @sql
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Found %d row(s) of availability information for class %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END 

    SET @sql =
      'INSERT INTO #ExtStatusViewRecord (cid, id, recordType, name, state, stateDescription, timeEnteredState, totalNumberOfIncidents, detailLevel) ' +
        'SELECT t.cid, t.id, ''STATE'', ''Object State'', v._CurrentStateID, s._State, ' +
          '(SELECT MAX(ctime) FROM AlertStateHistory ash  WITH (NOLOCK)  WHERE ash.cid = v.cid AND ash.id = v.id AND ash.evt_cid = ''MESG''), NULL, 100 ' +
        'FROM #tempcidids AS t ' +
        'JOIN ' + @cname + '_V AS v WITH (NOLOCK)  ON v.id = t.id ' +
        'JOIN StateType_V AS s WITH (NOLOCK)  ON s.id = v._CurrentStateID ' +
        'WHERE t.cid = ''' + RTRIM(@cid) + ''' AND v.deleted = 0 AND v._CurrentStateID <> 0 '
    IF (@verbose >= 2) PRINT @sql
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN 
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: api_getStatusView: Found %d row(s) of state information for class %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END 

    FETCH NEXT FROM class_cursor INTO @cid
  END
  DEALLOCATE class_cursor

  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Finished looping through classes', 0, 1, @now) WITH NOWAIT
  END

  -- finish setting some of the fields 
  UPDATE s SET stateDescription=a.name 
  FROM #ExtStatusViewRecord AS s 
    JOIN AlertState_C_BASE AS a WITH (NOLOCK) ON a.id = s.state
  WHERE s.recordType = 'AVAIL'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Set stateDescription on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END 
  
  UPDATE s SET prev_stateDescription=a.name
  FROM #ExtStatusViewRecord AS s
    JOIN AlertState_C_BASE AS a WITH (NOLOCK) ON a.id = s.prev_state
  WHERE s.recordType = 'AVAIL'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Set prev_stateDescription on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END 

  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Finished setting state descriptions', 0, 1, @now) WITH NOWAIT
  END

  IF @sendOutStatusView=1
  BEGIN 
    SELECT 'ExtStatusViewRecord', 'ExtBusinessImpactInfo', 'SLAStatusSummary'
    SELECT * FROM #ExtStatusViewRecord
    SELECT * FROM #ExtBusinessImpactInfo
    SELECT * FROM #summary
  END 

  IF @verbose >= 1
  BEGIN 
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: api_getStatusView: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
