-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_descendents_nocache.sql
-- SID: 1.4.1.3
-- Date Modified: 04/12/06 17:35:59
--
-- Module Description
--   Defines the asisp_descendents_nocache stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_descendents_nocache
PRINT 'Creating procedure asisp_descendents_nocache'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_descendents_nocache' AND type = 'P')
  DROP PROCEDURE asisp_descendents_nocache
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/asisp_descendents_nocache.sql
-- SID: 1.4.1.3
-- Date Modified: 04/12/06 17:35:59
--
-- Returns all descendents of the calling object of the class or classes specified by a lookup
-- expression. Unlike asisp_descendents, this procedure does not use the ObjPathCache table.
-- Use this procedure only in cases where the ObjPathCache table cannot be guaranteed to be
-- up-to-date and where that would be a significant problems. Usually, if the procedure is
-- returning results to a user it is okay for the table to be out-of-date since it will soon
-- be updated and the user will then see the correct results.
--
-- Caller must create a table to hold the results, as follows:
/*
  CREATE TABLE #descendent
  (
    cid   ClassID NOT NULL,
    id    ObjID   NOT NULL,
    gen   INT     NOT NULL
  )
*/
CREATE PROCEDURE asisp_descendents_nocache
  @cid            ClassID,
  @id             ObjID,
  @desc_cid_expr  NVARCHAR(1000)=NULL, -- Descendent class id expression. If specified, must be one of the following forms:
                                       --   'IN (<CID1>[,<CID2>...])' OR 'LIKE <wildcard>'
  @verbose        TINYINT=0  -- If >=1 then diagnostic messages are displayed during execution
AS
  SET NOCOUNT ON

  CREATE TABLE #desc
  (
    cid      ClassID NOT NULL,
    id       ObjID   NOT NULL,
    gen      INT     NOT NULL,
    nonleaf  TINYINT NOT NULL
  )

  CREATE TABLE #lob
  (
    src_id  ObjID   NOT NULL,
    dst_id  ObjID   NOT NULL,
    gen     INT     NOT NULL,
    phy_cid ClassID NULL,
    phy_id  ObjID   NOT NULL,
    nonleaf INT     NOT NULL
  )

  DECLARE
    @desc_delete_expr   NVARCHAR(255),
    @diagnostic_expr    NVARCHAR(255),
    @gen                INT,
    @gen_str            NVARCHAR(9),
    @id_str             NVARCHAR(9),
    @link_chain_clause  NVARCHAR(255),
    @now                NVARCHAR(40),
    @obj_where_clause   NVARCHAR(1000),
    @rc                 INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_descendents_nocache @cid=''%s'', @id=%d, @desc_cid_expr=''%s''', 0, 1, @now, @cid, @id, @desc_cid_expr) WITH NOWAIT
  END

  IF @cid = 'LOB'
  BEGIN
    IF @desc_cid_expr IS NULL
    BEGIN
      SET @link_chain_clause = ''
      SET @obj_where_clause = ''
    END
    ELSE
    BEGIN
      SET @link_chain_clause = 'AND (phy_cid ' + @desc_cid_expr + ' OR phy_cid IN (SELECT src_cid FROM link_chain WHERE cid ' + @desc_cid_expr + ') )'
      SET @obj_where_clause =  'WHERE child.phy_cid ' + @desc_cid_expr
    END

    -- Populate worktable with the descendent LOBs that are either linked directly to specified
    -- object classes, or linked to physical objects that can contain objects of those types
    SELECT @id_str = CAST(@id AS NVARCHAR(9))
    EXEC(
    'INSERT INTO #lob ' +
    'SELECT l.src_id, l.dst_id, l.gen, l.phy_cid, l.phy_id, 0 AS nonleaf ' +
    'FROM lob_link l ' +
    'WHERE  ' +
      'src_id = ' + @id_str + ' ' +
      @link_chain_clause + ' '
    )
    
    EXEC(
    'INSERT INTO #lob ' +
    'SELECT l.src_id, l.dst_id, l.gen, l.phy_cid, l.phy_id, 0 AS nonleaf ' +
    'FROM lob_link l ' +
    'WHERE  ' +
      'dst_id = ' + @id_str + ' ' +
      @link_chain_clause + ' ' +
      'AND NOT EXISTS (SELECT 1 FROM #lob AS prev WHERE prev.dst_id = l.dst_id) '
    )

    EXEC(    
    'UPDATE parent SET nonleaf = 1 ' +
      'FROM #lob AS parent  ' +
        'JOIN LOB_LOB_LOBC AS parent_child ON parent.dst_id = parent_child.src_id ' +
        'JOIN #lob AS child ON child.dst_id = parent_child.dst_id ' +
    @obj_where_clause + ' '
    )
    SET @rc = @@ROWCOUNT
    WHILE @rc <> 0
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_descendents_nocache: trimming', 0, 1, @now) WITH NOWAIT
      END
      
      UPDATE parent SET nonleaf = 1
      FROM #lob AS parent 
        JOIN LOB_LOB_LOBC AS parent_child ON parent.dst_id = parent_child.src_id
        JOIN #lob AS child ON child.dst_id = parent_child.dst_id
      WHERE
        parent.nonleaf = 0
        AND child.nonleaf = 1 /* deleted parent from previous generation */
      SET @rc = @@ROWCOUNT
    END
    
    INSERT INTO #desc SELECT DISTINCT phy_cid, phy_id, 0, nonleaf FROM #lob WHERE phy_cid IS NOT NULL
    SET @rc = @@ROWCOUNT
  END
  ELSE
  BEGIN
    INSERT INTO #desc VALUES ( @cid, @id, 0, 0 )
    SET @rc = @@ROWCOUNT
  END

  -- Now iterate through the #descendent table to build the list of all possible containers.
  IF @desc_cid_expr IS NULL
  BEGIN
    SET @link_chain_clause = ''
    SET @desc_delete_expr = ''
  END
  ELSE
  BEGIN
    SET @link_chain_clause = 'AND l.cid ' + @desc_cid_expr
    SET @desc_delete_expr = 'DELETE FROM #desc WHERE NOT (cid ' + @desc_cid_expr + ')'
  END

  SET @gen = 0
  WHILE @rc > 0
  BEGIN
    SET @gen = @gen + 1    
    SET @gen_str = CAST(@gen AS NVARCHAR(9))
    EXEC(
    'INSERT INTO #desc ' +
      'SELECT DISTINCT c.dst_cid, c.dst_id, ' + @gen_str + ', 0 ' +
      'FROM #desc AS d ' +
        'JOIN link AS c ON c.src_cid = d.cid AND c.src_id = d.id ' +
        'JOIN link_chain AS l ON l.dst_cid = c.dst_cid ' +
      'WHERE ' +
        'd.gen = ' + @gen_str + ' - 1 ' +
        'AND c.deleted = 0 ' +
        'AND d.nonleaf = 0 ' +
        'AND NOT EXISTS (SELECT 1 FROM #desc AS prev WHERE prev.cid = c.dst_cid AND prev.id = c.dst_id) ' +
        @link_chain_clause
    )
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_descedents_no_cache: gen=%d, #rows=%d', 0, 1, @now, @gen, @rc) WITH NOWAIT
    END
  END

  IF @desc_delete_expr IS NOT NULL
  BEGIN
    EXEC(@desc_delete_expr)
  END

  INSERT INTO #descendent SELECT DISTINCT cid, id, gen FROM #desc

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_descendents_nocache: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
