-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_getallsubservices.sql
-- SID: 1.12
-- Date Modified: 04/09/17 23:30:23
--
-- Module Description
--   Defines the asisp_GetAllSubServices stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_GetAllSubServices
PRINT 'Creating procedure asisp_GetAllSubServices'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_GetAllSubServices' AND type = 'P')
  DROP PROCEDURE asisp_GetAllSubServices
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_getallsubservices.sql
-- SID: 1.12
-- Date Modified: 04/09/17 23:30:23
--
CREATE PROCEDURE asisp_GetAllSubServices
  @cid                      ClassID,          -- Always EXES 
  @id                       ObjID,            -- Instance id of Service Object
  @cnt                      INT OUTPUT,       -- Total number of subservice found 
  @results                  TINYINT = 0,      -- return result set if set to 1 
  @Levels                   INT     = 1, 
  @outputType               TINYINT = 0,      -- 0 means original style (cid,id, parent), note, does not support a Level > 1  , 
                                              -- 1 means like link table (used by tbsm api)
                                              -- 2 means just indicate whether service has children or not 
  @searchLimit              INT = 500000,     -- search limit defaults to a huge number to get everything
  @ExcludeServiceResources  TINYINT = 1,      -- be default include service resources 
  @outputTable              NVARCHAR(255) = NULL, 
  @inputTable               NVARCHAR(255) = NULL,   -- the table to pull cid and ids from of services 
  @verbose                  TINYINT = 0
AS
  SET NOCOUNT ON

  CREATE TABLE #ServiceObjects    
  (    
    cid          ClassID  NOT NULL  DEFAULT 'EXES',  
    parent       ObjID    NULL,  
    id           ObjID    NOT NULL,         -- Instance id of Services    
    lob_id       ObjID    NULL,             -- Instance Id of the Associated LOB  
    gen          INT      NULL      DEFAULT NULL,
    flags        INT      NULL      DEFAULT NULL,
    found        TINYINT  NOT NULL  DEFAULT 0, 
    retLevel     INT      NOT NULL  DEFAULT 0,               -- jpf
    retParent    ObjID    NOT NULL  DEFAULT 0,               -- jpf 
    ctime        DATETIME NULL      DEFAULT NULL
  )   

  DECLARE
    @cmd                  NVARCHAR(4000),
    @err                  INT,
    @gen                  INT,
    @incomingUserGroup    TINYINT,
    @lobFlagMask          INT,
    @minloops             INT, --- used to handle the case of EXES 0 and multiple levels requested, the loop is entered with already a level 1 in 
    @now                  NVARCHAR(40),
    @rc                   INT,
    @serviceContainerMask INT,
    @usercid              ClassID,
    @userid               ObjID
  
  SET @minloops=0 

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Service Object @id =%d', 0, 1, @now, @id) WITH NOWAIT
  END

  --- Force Levels to 1 if outputType is 0  
  IF @outputType=0
    SET @Levels=1
  
  -- if the request is to know if there are children of the incoming cid/ids make sure the level is 1 
   
  IF @outputType=2
    SET @Levels=1 

  --- service container mask 
  SET @serviceContainerMask = 8192+2048

  /* set up the flag to use to compare with the LOB to decide if it should get included in the results */ 
  SET @lobFlagMask = 8192
  IF @ExcludeServiceResources = 1
  BEGIN
     SET @lobFlagMask = @serviceContainerMask       ---- (8192+2048) means provides lob container service rather than resource service 
  END 
   
  IF @cid IS NOT NULL AND @id IS NOT NULL
  BEGIN  
    INSERT INTO #ServiceObjects (cid,parent,id,lob_id) VALUES ( 'EXES', @id, @id, 0 )
  END
  ELSE
  BEGIN 
    IF @inputTable IS NOT NULL
    BEGIN
      SET @cmd = 'INSERT INTO #ServiceObjects(cid, parent, id, lob_id) SELECT ''EXES'', id, id, 0 from ' + @inputTable + ' WHERE cid = ''EXES'''
      EXEC(@cmd)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of services from input table %s', 0, 1, @now, @rc, @inputTable) WITH NOWAIT
      END
    END
    ELSE
    BEGIN 

      IF OBJECT_ID('tempdb..#Services') IS NOT NULL
      BEGIN
        INSERT INTO #ServiceObjects(cid,parent,id,lob_id) SELECT 'EXES',id,id,0 FROM #Services  
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of services from #Services', 0, 1, @now, @rc) WITH NOWAIT
        END
      END 
    END 

  END

  --- now set up variables to handle user context , first get usercid and id of the user specfied in #tbsm_as_user_info
  SET @incomingUserGroup = 0 


  IF OBJECT_ID('tempdb..#tbsm_as_user_info') IS NOT NULL
  BEGIN
    IF EXISTS( SELECT 1 FROM #tbsm_as_user_info WHERE usergroup IS NOT NULL AND NOT (usergroup='' AND username=''))
    BEGIN
      SELECT @usercid = e.cid, @userid=e.id
      FROM EVITUser_V AS e
      JOIN #tbsm_as_user_info AS t ON ((t.usergroup = N'TBSM_Executives_IT' AND e.cid='EXEI')
                                    OR (t.usergroup = N'TBSM_Executives' AND e.cid='EXEU')
                                    )
                                  AND LOWER(t.username) = LOWER(e._UserID) and e._IsDeleted=0
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: usercid=''%s'', userid=%d', 0, 1, @now, @usercid, @userid) WITH NOWAIT
      END
      SET @incomingUserGroup =1       
    END     

  END 


  --- no handle the exception case where root services are being asked for ... EXES with id of 0
  IF EXISTS (SELECT 1 FROM #ServiceObjects WHERE cid = 'EXES' AND id = 0)
  BEGIN
    SET @minloops=1
    IF @incomingUserGroup=1
    BEGIN 

      INSERT INTO #ServiceObjects (cid, parent, id,  retParent, gen, ctime)  
      SELECT 'EXES', l.src_id, l.dst_id,  0, 1, l.ctime
      FROM  link l WITH (NOLOCK) 
        JOIN link l2 WITH (NOLOCK) ON l2.deleted=0 and l2.link_type='EVSL' and  l2.src_cid = l.dst_cid and l2.src_id=l.dst_id and l2.dst_cid='LOB' --- get LOBs for this Service that matches 
      WHERE l.deleted = 0
        AND l.link_type='EVPL'
        AND l.src_cid = @usercid
        AND l.src_id = @userid
        AND l.dst_cid='EXES'   --- links with the right user parent
----        UPDATE r SET createTS=e.ctime, modifiedTS=e.ctime FROM #Relationships r , EVService_C e where r.createTS is null and r.target_cid=e.cid and r.target_id=e.id
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of services for user %s:%d', 0, 1, @now, @rc, @usercid, @userid) WITH NOWAIT
      END


    END --- end of getting top level services for a specific user 

    ELSE
    BEGIN --- Get all top level Services 
      INSERT INTO #ServiceObjects (cid, parent, id,  retParent, gen, ctime)  
        SELECT derived.src_cid, derived.src_id, derived.dst_id,  0, 1, derived.createTS
        FROM 
        (
          SELECT
            'ROOT' AS link_type,
            'EXES' AS src_cid,
            0 AS src_id,
            e.cid AS dst_cid,
            e.id AS dst_id,
            MIN(l.ctime) AS createTS,
            MIN(l.ctime) AS modifiedTS,
            NULL AS deletedTS,
            1 AS gen,
            100 AS detailLevel ---, max(e.deleted)
          FROM EVService_C AS e  
          LEFT OUTER JOIN link AS l ON e.deleted = 0
                                   AND l.dst_id  = e.id
                                   AND l.dst_cid = 'EXES'
                                   AND link_type = 'EVPL'
                                   AND l.deleted = 0
          WHERE e.deleted=0 AND ( src_cid IN ('EXEI', 'EXEU') OR src_cid IS NULL)
          GROUP BY e.cid, e.id 
        ) AS derived 
        LEFT OUTER JOIN link AS l2 ON l2.deleted      = 0
                                  AND l2.link_type    = 'EVSS'
                                  AND l2.src_cid      = 'EXES'
                                  AND derived.dst_cid = l2.dst_cid
                                  AND derived.dst_id  = l2.dst_id
        WHERE l2.link_type IS NULL
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of top-level services ', 0, 1, @now, @rc) WITH NOWAIT
      END

      --- For the made up links create a link time from the service object time
      UPDATE s SET ctime = e.ctime
      FROM #ServiceObjects AS s
      JOIN EVService_C AS e ON e.cid = s.cid AND e.id = s.id
      WHERE s.ctime IS NULL 
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Updated ctime for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
      END

    END  --- end of else to get all top level services 

    --- now remove EXES, 0 from the processing list 
    DELETE d FROM #ServiceObjects AS d WHERE cid='EXES' AND id=0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_GetAllSubServices: Deleted %d row(s) where id = 0', 0, 1, @now, @rc) WITH NOWAIT
    END

  END   --- end for exists a 'EXES' with id 0 

  UPDATE s SET
    gen=0,  
    lob_id = l.dst_id,  
    flags = v._flags, 
    found = CASE v._flags&@lobFlagMask  WHEN @lobFlagMask  THEN 1 ELSE 0 END,          
    retLevel=0
  FROM #ServiceObjects AS s  
  JOIN link AS l WITH (NOLOCK) ON l.deleted = 0 AND l.link_type='EVSL' AND l.src_id =  s.id  AND l.src_cid='EXES'
  JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.deleted = 0 AND v.id = l.dst_id  AND l.dst_cid='LOB'
  WHERE s.lob_id = 0  --- this will make it only process the entries added in on the input list 
      --- WHERE l.link_type = 'EVSL' AND l.deleted =0  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated %d row(s) of services from entries added on input', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE s SET 
    lob_id = l.dst_id,  
    flags = v._flags  , 
    found = CASE v._flags&@lobFlagMask  WHEN @lobFlagMask  THEN 1 ELSE 0 END,          
    retLevel =  CASE v._flags&@lobFlagMask  WHEN @lobFlagMask  THEN 1 ELSE 0 END
  FROM #ServiceObjects AS s  
  JOIN link AS l WITH (NOLOCK) ON l.deleted = 0 AND l.link_type='EVSL' AND l.src_id =  s.id  AND l.src_cid='EXES'
  JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.deleted = 0 AND v.id = l.dst_id  AND l.dst_cid='LOB'
  WHERE s.lob_id IS NULL --- this will make it only process the ones added from root above 
      ---- WHERE l.link_type = 'EVSL' AND l.deleted =0  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Updated %d row(s) of services from entries added from root', 0, 1, @now, @rc) WITH NOWAIT
  END

  --- set @rc to make it go through loop at least once
  SET @rc = 1   
  SET @gen = 0 

  WHILE @rc > 0  OR @gen<=@minloops  --- either nothing was added to the table or the number of levels has not been hit yet  
  BEGIN

    SET @gen = @gen + 1


    INSERT INTO #ServiceObjects  
      SELECT  'EXES', s.id, e.dst_id, v.id, @gen, v._flags, 
        CASE v._flags&@lobFlagMask WHEN @lobFlagMask  THEN 1 ELSE 0 END,          
        CASE s.flags&@lobFlagMask  WHEN @lobFlagMask  THEN s.retLevel+1 ELSE s.retLevel END, 
        CASE s.flags&@lobFlagMask  WHEN @lobFlagMask  THEN e.src_id ELSE s.retParent END,
        l.ctime
      FROM  link AS e  WITH (NOLOCK)   
        JOIN #ServiceObjects AS s WITH (NOLOCK) ON s.id = e.src_id  
        JOIN link AS l  WITH (NOLOCK) ON l.deleted=0 and l.src_id = e.dst_id    
        JOIN LineOfBusiness_V  AS v  WITH (NOLOCK) ON v.id = l.dst_id
      WHERE e.link_type = 'EVSS'   
        AND e.deleted = 0      
        AND l.deleted = 0  
        AND l.link_type = 'EVSL'  
        AND retLevel <= @Levels 
        AND s.gen = @gen - 1     
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) of services descendent from generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
    END

    IF
    (
      SELECT COUNT(cid)
      FROM #ServiceObjects WITH (NOLOCK) 
      WHERE gen <> 0
        AND flags & @lobFlagMask = @lobFlagMask
        AND retLevel > 0
        AND retLevel <= @Levels
        AND found = 1
    ) > @searchLimit
    BEGIN 
      UPDATE #ServiceObjects SET found = 0 WHERE gen = @gen-1
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Reset found on %d row(s) because search limit of %d row(s) has been exceeded', 0, 1, @now, @rc, @searchLimit) WITH NOWAIT
      END
      GOTO GETOUT     
    END 

    /* DELETE AFTER I THINK THIS WORKS WITHOUT IT 
    IF @rc > 0
        SELECT @rc = COUNT(*)   
        FROM #ServiceObjects    WITH (NOLOCK) 
        WHERE gen = @gen OR ((retLevel < @Levels) OR ( flags & @lobFlagMask <> @lobFlagMask)   )     
    */

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_GetAllSubServices: Locating Subservices. Rowcount = %d, gen = %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
    END
  END

GETOUT:

  IF @outputType = 0
  BEGIN 
    IF @results = 1
    BEGIN 
      SELECT cid ,id, retParent    --- used to be parent
      FROM #ServiceObjects    WITH (NOLOCK) 
      WHERE gen <> 0 AND flags & @lobFlagMask = @lobFlagMask  and retLevel > 0 and retLevel <= @Levels AND found=1
      SET @cnt = @@ROWCOUNT
    END 
    ELSE
    BEGIN
      SELECT @cnt = COUNT(cid)
      FROM #ServiceObjects    WITH (NOLOCK) 
      WHERE gen <> 0 AND flags & @lobFlagMask = @lobFlagMask  and retLevel > 0 and retLevel <= @Levels AND found=1
    END 
  END 
  ELSE IF @outputType = 1
  BEGIN   --- outputType = 1 
    IF @results = 1
    BEGIN 
      --- return a result similar to a link table enty except add in an indication of whether is is a service resource or a service container 
      SET @cmd =
      'SELECT ''EVSS'' AS relationship_label, ''EXES'' AS source_cid, retParent AS source_id, cid AS target_cid, id AS target_id, ctime AS createTS, lob_id AS lob_id, flags AS lob_flags, retLevel AS gen, ctime AS modifiedTS ' +
      'FROM #ServiceObjects WITH (NOLOCK) ' +
      'WHERE found=1 AND gen <> 0 AND flags & '  +  CAST(@lobFlagMask AS NVARCHAR(9)) + ' = ' + CAST(@lobFlagMask AS NVARCHAR(9)) + '  and retLevel > 0 and retLevel <= ' +  CAST (@Levels AS NVARCHAR(9)) + ' '

      IF @outputTable IS NOT NULL
      BEGIN 
        SET @cmd = 'INSERT INTO ' + @outputTable + '(relationship_label, source_cid, source_id, target_cid, target_id, createTS, lob_id, lob_flags, gen, modifiedTS) ' + @cmd
      END 

      EXEC(@cmd)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) into output table %s', 0, 1, @now, @rc, @outputTable) WITH NOWAIT
      END
      SET @cnt = @rc

    END 
    ELSE
    BEGIN 
      SELECT @cnt = COUNT(*) 
      FROM #ServiceObjects WITH (NOLOCK)
      WHERE gen <> 0
        AND flags & @lobFlagMask = @lobFlagMask
        AND retLevel > 0
        AND retLevel <= @Levels
        AND found=1
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_GetAllSubServices: Count = %d', 0, 1, @now, @cnt) WITH NOWAIT
      END

    END
  END 
  ELSE
  BEGIN ---- outputType = 2
    --- first change the flags column to only indicate whether it is a service container or a service resource
    UPDATE #ServiceObjects SET
      flags = CASE flags & @serviceContainerMask  WHEN @serviceContainerMask THEN @serviceContainerMask ELSE 8192 END 
    WHERE found=1 

    --- return a results either in the table provided or just select them to a result set 

    SET @cmd = 
    'SELECT ''EXES'' as source_cid, retParent as source_id , flags as children_type , count(retParent) as numberOfChildren ' +
    'FROM #ServiceObjects WITH (NOLOCK) ' +
    'WHERE retLevel = 1  AND found=1 ' +
    'GROUP BY retParent, flags '

    IF @outputTable IS NOT NULL
    BEGIN 
      SET @cmd = 'INSERT INTO ' + @outputTable + '( source_cid, source_id, children_type, numberOfChildren) ' + @cmd
    END 

    EXEC(@cmd)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_GetAllSubServices: Inserted %d row(s) into output table %s', 0, 1, @now, @rc, @outputTable) WITH NOWAIT
    END
    SET @cnt = @rc
  END 

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_GetAllSubServices: Finished', 0, 1, @now) WITH NOWAIT
  END

  RETURN @cnt
GO
