-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_processstagedexcps.sql
-- SID: 1.33.1.30
-- Date Modified: 04/11/10 11:57:01
--
-- Module Description
--   Defines the asisp_processStagedEXCPs stored procedure

USE Object
GO
SET LANGUAGE 'English'
GO
SET QUOTED_IDENTIFIER  OFF
SET ANSI_NULLS  ON
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedEXCPs
PRINT 'Creating procedure asisp_processStagedEXCPs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedEXCPs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedEXCPs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_processstagedexcps.sql
-- SID: 1.33.1.30
-- Date Modified: 04/11/10 11:57:01
-- Processes the records in the StagedEXCP table, up to the specified @maxrows
CREATE PROCEDURE asisp_processStagedEXCPs
  @maxrows  INT = 100,    -- Maximum number of rows to process
  @verbose  TINYINT = 0   -- If >0 then verbose progress messages are printed during execution
AS
  SET NOCOUNT ON

  -- Temporary table declarations
  DECLARE @defer_excp TABLE (timestamp BINARY(8) NOT NULL)
  DECLARE @defer_singleton TABLE (timestamp BINARY(8) NOT NULL, cid NVARCHAR(4) NOT NULL, id INT NOT NULL)
  DECLARE @ownedSingletons TABLE (id INT, deleted TINYINT, new_timestamp BINARY(8), new_id INT NULL)

  -- Variable declartions
  DECLARE
    @err                    INT,
    @rc                     INT,
    @now                    NVARCHAR(40),
    @tcount                 INT,
    @exteventtype           INT,
    @owner_event_count      INT,
    @extended_event_count   INT,
    @max_timestamp          TIMESTAMP,
    @num_events_to_process  INT,
    @num_events_processed   INT,
    @eventretained          NVARCHAR(5),
    @maxts                  BINARY(8),
    @deleted_count          INT,
    @exch_base_id           ObjID,
    @date                   DATETIME,
    @updated_count          INT,
    @base_id                ObjID,
    @inserted_count         INT

  IF NOT EXISTS (SELECT 1 FROM StagedEXCP)
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: no events to process', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0
  END

  IF EXISTS (SELECT 1 FROM SyncEvent WITH (NOLOCK))
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: SyncEvent record(s) exist, returning', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs @maxrows=%d, @verbose=%d', 0, 1, @now, @maxrows, @verbose) WITH NOWAIT
  END

  -- Repopulate work table with ids of all EXCP events that are available for processing, 'available' being defined
  -- as a EXCP whose parent object does not have any earlier MESG events that have not been processed.
  SET ROWCOUNT @maxrows
  TRUNCATE TABLE StagedEXCPInProgress
  INSERT INTO StagedEXCPInProgress
    SELECT
      0,
      timestamp,
      e.ctime,
      e.parent_cid,
      e.parent_id,
      TIME,
      ISNULL(e.SEQ_NO,''),
      ISNULL(e.OBJECT_NAME,''),
      ISNULL(e.NATIVE_KEY,''),
      ISNULL(e.EXCP_OBJ_TYPE,''),
      ISNULL(e.EXCP_OBJ_NAME,''),
      ISNULL(e.EXCP_NAME,''),
      ISNULL(e.EXCP_CD,''),
      1, -- Was nEXCH. But by definition each staged exception can only contribute 1 to the count
      e.AlertState,
      e.Priority,
      e.Flags
    FROM
      StagedEXCP AS e
    ORDER BY e.timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error %d creating worktable from first %d rows of StagedEXCP', 3, 1, @now, @err, @maxrows) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: procesing %d event(s) in StagedEXCP', 0, 1, @now, @rc) WITH NOWAIT
  END

  SET ROWCOUNT 0

  --
  -- Populate the work table holding the extended attributes    KDM
  -- from StagedEEVT associated with the rows just taken from
  -- the StagedEXCP table

  TRUNCATE TABLE StagedEEVTInProgress
  INSERT INTO StagedEEVTInProgress
  (
    timestamp,
    ctime,
    ExtEvType,
    ExtEvBinData,
    ExtEvLongTxt,
    ExtEvSlot00,
    ExtEvSlot01,
    ExtEvSlot02,
    ExtEvSlot03,
    ExtEvSlot04,
    ExtEvSlot05,
    ExtEvSlot06,
    ExtEvSlot07,
    ExtEvSlot08,
    ExtEvSlot09,
    ExtEvSlot10,
    ExtEvSlot11,
    ExtEvSlot12,
    ExtEvSlot13
  )
  SELECT
    ee.timestamp,
    ee.ctime,
    ExtEvType,
    ExtEvBinData,
    ExtEvLongTxt,
    ExtEvSlot00,
    ExtEvSlot01,
    ExtEvSlot02,
    ExtEvSlot03,
    ExtEvSlot04,
    ExtEvSlot05,
    ExtEvSlot06,
    ExtEvSlot07,
    ExtEvSlot08,
    ExtEvSlot09,
    ExtEvSlot10,
    ExtEvSlot11,
    ExtEvSlot12,
    ExtEvSlot13
  FROM
    StagedEEVT AS ee
    JOIN StagedEXCPInProgress AS xx ON ee.timestamp = xx.timestamp
  ORDER BY ee.timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking up StagedEEVT records. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d matching row(s) in StagedEEVT', 0, 1, @now, @rc) WITH NOWAIT
  END
  SET @extended_event_count = @rc

  --
  -- If we have some extended events then check to see if any
  -- of them are ownership events
  IF @extended_event_count > 0
  BEGIN

    -- OA07460 we need to add an SC to the close EXCP_CD
    UPDATE s SET EXCP_CD='SC '+s.EXCP_CD
    FROM StagedEXCPInProgress AS s
    JOIN StagedEEVTInProgress AS e ON s.timestamp=e.timestamp
    WHERE e.ExtEvSlot03='CLOSED'

    -- Lookup the value from the ownership enumeration
    SELECT @exteventtype = value
    FROM enumeration AS e WITH (NOLOCK) JOIN enum_value AS ev WITH (NOLOCK) ON e.eid = ev.eid
    WHERE e.name = 'ExtendedEventType' AND ev.name = 'TECO'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error looking up ExtendedEventType enumeration. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: enumeration value ExtendedEventType.TECO = [%d]', 0, 1, @now, @exteventtype) WITH NOWAIT
    END
    IF @rc = 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processstagedEXCPs: Setting of field TECO is not supported.', 3, 1, @now) WITH NOWAIT
      RETURN -30000
    END

    SELECT @owner_event_count = COUNT(*) FROM StagedEEVTInProgress WHERE ExtEvType=@exteventtype
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error selecting count from worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: owner_event_count = [%d]', 0, 1, @now, @owner_event_count) WITH NOWAIT
    END

    --
    -- If we have ownership records, copy them from StagedEXCPInProgress to
    -- ensure that they do not get deleted (singleton, etc, etc)
    IF @owner_event_count > 0
    BEGIN
      TRUNCATE TABLE StagedEXCPOwnerInProgress
      INSERT INTO StagedEXCPOwnerInProgress
        SELECT
          0,
          ee.timestamp,
          ee.ctime,
          parent_cid,
          parent_id,
          TIME,
          SEQ_NO,
          OBJECT_NAME,
          NATIVE_KEY,
          EXCP_OBJ_TYPE,
          EXCP_OBJ_NAME,
          EXCP_NAME,
          EXCP_CD,
          nEXCH,
          AlertState,
          Priority
        FROM
          StagedEXCPInProgress AS ee
        JOIN StagedEEVTInProgress AS yy ON
          ee.timestamp = yy.timestamp
          AND yy.ExtEvType = @exteventtype
        ORDER BY ee.timestamp
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: error retrieving ownership records from StagedEXCPInProgress. Error=%d', 3, 1, @now, @err) WITH NOWAIT
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: found TEC Ownership %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
      END
    END
  END
  ELSE
    SET @owner_event_count=0
  ----  END KDM

  SET ROWCOUNT 0

  -- For EXCP that have earlier unprocessed StagedMESG records, we will defer processing these EXCP until next SEL cycle (after StagedMESG has been processed again)
  INSERT INTO @defer_excp (timestamp)
    SELECT DISTINCT p.timestamp
      FROM StagedEXCPInProgress AS p
      JOIN StagedMESG           AS m ON m.parent_cid = p.parent_cid AND m.parent_id = p.parent_id
    WHERE m.timestamp < p.timestamp
  SELECT @rc = @@ROWCOUNT
  IF @rc > 0
  BEGIN
    DELETE FROM p
      FROM StagedEXCPInProgress AS p
      JOIN StagedMESG           AS m ON m.parent_cid = p.parent_cid AND m.parent_id = p.parent_id
    WHERE m.timestamp < p.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing rows which there are earlier unprocessed StagedMESG records. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) for which there are earlier unprocessed StagedMESG records', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- For situations when there are more than 1 Singleton exception that have same name, resource cid and id
  INSERT INTO @defer_singleton (timestamp, cid, id)
    SELECT DISTINCT new.timestamp, new.parent_cid, new.parent_id
    FROM StagedEXCPInProgress AS prev
      JOIN StagedEXCPInProgress AS new ON
             new.parent_cid = prev.parent_cid
         AND new.parent_id  = prev.parent_id
         AND new.EXCP_NAME  = prev.EXCP_NAME
         AND new.timestamp  > prev.timestamp
         AND (new.AlertState <> prev.AlertState OR new.Priority <> prev.Priority)
         AND (prev.Flags & 1 = 1 AND new.Flags & 1 = 1)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking for singletons with the same name, resource cid & id. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d singletons with the same name, resource cid & id', 0, 1, @now, @rc) WITH NOWAIT
  END
  IF @rc > 0  
  BEGIN
    -- If we have a situation where there are more than 1 Singleton exception with the same name, resource cid and id,
    -- we have to defer all other excpetions for that resource as well, otherwise the final state of the resource
    -- could be incorrect.
    DECLARE @defer_ts BINARY(8)
    SELECT @defer_ts = MIN(timestamp) FROM @defer_singleton
    INSERT INTO @defer_singleton (timestamp, cid, id)
      SELECT DISTINCT p.timestamp, p.parent_cid, p.parent_id
      FROM StagedEXCPInProgress AS p
        JOIN @defer_singleton AS d ON d.cid = p.parent_cid AND d.id = p.parent_id
      WHERE
            p.timestamp > @defer_ts
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error looking for singletons that should be deferred. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: found %d singletons that should be deferred', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Remove deferred exceptions from the InProgress tables

    DELETE FROM StagedEXCPInProgress 
    WHERE timestamp IN (SELECT DISTINCT timestamp FROM @defer_singleton)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing rows from StagedEXCPInProgress for deferred singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) from StagedEXCPInProgress for deferred singletons', 0, 1, @now, @rc) WITH NOWAIT
    END

    DELETE FROM StagedEXCPOwnerInProgress 
    WHERE timestamp IN (SELECT DISTINCT timestamp FROM @defer_singleton)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing rows from StagedEXCPOwnerInProgress for deferred singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) from StagedEXCPOwnerInProgress for deferred singletons', 0, 1, @now, @rc) WITH NOWAIT
    END

    DELETE FROM StagedEEVTInProgress 
    WHERE timestamp IN (SELECT DISTINCT timestamp FROM @defer_singleton)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing rows from StagedEEVTInProgress for deferred singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) from StagedEEVTInProgress for deferred singletons', 0, 1, @now, @rc) WITH NOWAIT
    END

    IF @verbose >= 2
      SELECT '@defer_singleton'='@defer_singleton',* FROM @defer_singleton
  END

  SELECT @max_timestamp = MAX(timestamp) FROM StagedEXCPInProgress
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error selecting MAX(timestamp) from worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  SELECT @num_events_to_process = COUNT(*) FROM StagedEXCPInProgress

  -- OA02933: Made this check exclude StagedEXCPInProgress records that have corresponding rows
  -- in StagedEXCPOwnerInProgress. This is instead of the pruning of the StagedEXCPInProgress
  -- pruning that was going on before (see comments above).
  SELECT @num_events_processed = COUNT(*) FROM StagedEXCPInProgress AS p
  WHERE timestamp NOT IN (SELECT timestamp FROM StagedEXCPOwnerInProgress)
  IF @num_events_processed = 0
  BEGIN
    IF @owner_event_count > 0
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: going to PROCESS_OWNERSHIP', 0, 1, @now) WITH NOWAIT
      END
      GOTO PROCESS_OWNERSHIP            -- KDM
    END
    ELSE
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: going to CLEANUP', 0, 1, @now) WITH NOWAIT
      END
      GOTO CLEANUP
    END
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('INITIAL' AS NVARCHAR(16)) AS 'INITIAL', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Delete any events with 'Eliminate' priority
  DELETE FROM StagedEXCPInProgress WHERE Priority = 6
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error removing rows with ''Eliminate'' priority. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) with ''Eliminate'' priority', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Delete any exceptions belonging to a deleted resource
  DELETE s
  FROM StagedEXCPInProgress AS s
    JOIN containment AS c ON c.dst_cid = s.parent_cid AND c.dst_id = s.parent_id
  WHERE c.deleted = 1

  -- Update the AlertState/Priority/Flags values of all rows with NULL values
  UPDATE s SET
    AlertState = ISNULL(s.AlertState, t.AlertStateID),
    Priority   = ISNULL(s.Priority, t.PriorityID),
    Flags      = ISNULL(s.Flags, t.Flags)
  FROM StagedEXCPInProgress AS s
    JOIN ExceptionType_T AS t ON t.cid = s.parent_cid AND s.EXCP_NAME LIKE t.Exception
  WHERE
    (s.AlertState IS NULL OR s.Priority IS NULL OR s.Flags IS NULL)
    AND NOT EXISTS
    (
      SELECT 1 FROM ExceptionType_T AS tplus
      WHERE
        tplus.cid = s.parent_cid
        AND s.EXCP_NAME LIKE tplus.Exception
        AND tplus.Salience > t.Salience
    )
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking up AlertState/Priority/Flags values from ExceptionType. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: updated AlertState/Priority/Flags values on %d row(s) from ExceptionType', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Replace nulls with 0s for AlertState/Priority
  UPDATE StagedEXCPInProgress SET
    AlertState = ISNULL(AlertState, 0),
    Priority = ISNULL(Priority, 0),
    Flags = ISNULL(Flags, 0)
  WHERE AlertState IS NULL OR Priority IS NULL OR Flags IS NULL
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error replacing NULL AlertState/Priority/Flags values. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: replaced %d NULL AlertState/Priority/Flags value(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  /* 125296 - SJ
    If Duplicate events come in from TEC , we keep the last event in the first batch. In any of the following
    batches, we only update the exception counter. However, in TEC, one would either keep the first event
    or the last event, as a global or per class. To be in synch with TEC, a SystemConfiguration setting called
    TEC_DUPS will be used to determine whether the fisrt event or the last event slots are used or if it is not
    set, the default behavior will be used.
  */
  SELECT @eventretained=propertyvalue FROM SystemConfiguration where groupname='TEC_DUPS' and propertyname='EventRetained'
  IF (@eventretained='FIRST')
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Retrieved SystemConfiguration value for TEC_DUPS : FIRST', 0, 1, @now) WITH NOWAIT
    END
    --If the Exception already exists, update this batch with the slots from ExtendedEvent
    UPDATE ee SET
      ExtEvSlot00 = e._ExtEvSlot00,
      ExtEvSlot01 = e._ExtEvSlot01,
      ExtEvSlot02 = e._ExtEvSlot02,
      ExtEvSlot04 = e._ExtEvSlot04
    FROM
      StagedEXCPInProgress AS p
      JOIN Exception_V AS o WITH (NOLOCK) ON o._ParentCID = p.parent_cid
                                         AND o._ParentID  = p.parent_id
                                         AND o._EXCP_NAME = p.EXCP_NAME
      JOIN ExtendedEvent_V AS e ON e.id = o.id
      JOIN StagedEEVTInProgress AS ee ON ee.timestamp = p.timestamp
    WHERE o.deleted=0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating StagedEEVTInProgress with slots from ExtendedEvent_V. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Updated %d StagedEEVTInProgress rows with slots from ExtendedEvent_V', 0, 1, @now, @rc) WITH NOWAIT
    END
    -- Update this batch with the slots values of the first duplicate 
    UPDATE later_e SET 
      ExtEvSlot00 = earlier_e.ExtEvSlot00, 
      ExtEvSlot01 = earlier_e.ExtEvSlot01, 
      ExtEvSlot02 = earlier_e.ExtEvSlot02, 
      ExtEvSlot04 = earlier_e.ExtEvSlot04
    FROM
      StagedEXCPInProgress AS earlier
      JOIN StagedEEVTInProgress AS earlier_e ON earlier_e.timestamp = earlier.timestamp
      JOIN StagedEXCPInProgress AS later ON later.parent_cid = earlier.parent_cid
                                        AND later.parent_id  = earlier.parent_id
                                        AND later.EXCP_NAME  = earlier.EXCP_NAME
      JOIN StagedEEVTInProgress AS later_e ON later_e.timestamp = later.timestamp
    WHERE earlier.timestamp  < later.timestamp
      AND earlier.Flags & 1 = 1
      AND later.Flags & 1 = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating StagedEEVTInProgress slots to match the first duplicate event in the batch. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Updated %d StagedEEVTInProgress slots to match the first duplicate event in the batch', 0, 1, @now, @rc) WITH NOWAIT
    END
  END
  ELSE IF (@eventretained='LAST')-- Keep Last
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Retrieved SystemConfiguration value for TEC_DUPS : LAST', 0, 1, @now) WITH NOWAIT
    END
    SET @maxts=0
    --If the Exception already exists, update ExtendedEvent with slot values of the last event in this batch
    SELECT @maxts=MAX(p.timestamp)  
    FROM StagedEXCPInProgress AS p
      JOIN Exception_V AS e ON e._ParentCID = p.parent_cid
                           AND e._ParentID  = p.parent_id
                           AND e._EXCP_NAME = p.EXCP_NAME
    WHERE
      e.deleted = 0
      AND p.Flags & 1 = 1
    IF @maxts<>0
    BEGIN
      UPDATE e SET 
        _ExtEvSlot00 = ee.ExtEvSlot00, 
        _ExtEvSlot01 = ee.ExtEvSlot01, 
        _ExtEvSlot02 = ee.ExtEvSlot02, 
        _ExtEvSlot04 = ee.ExtEvSlot04
      FROM
        StagedEXCPInProgress AS p
        JOIN Exception_V AS o ON o._ParentCID = p.parent_cid
                             AND o._ParentID = p.parent_id
                             AND o._EXCP_NAME = p.EXCP_NAME
        JOIN ExtendedEvent_V AS e ON e.id = o.id
        JOIN StagedEEVTInProgress AS ee ON ee.timestamp = p.timestamp
      WHERE p.timestamp=@maxts
        AND o.deleted=0
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: error updating ExtendedEvent with slots from StagedEEVTInProgress. Error=%d', 3, 1, @now, @err) WITH NOWAIT
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: Updated %d ExtendedEvent rows with slots from StagedEEVTInProgress', 0, 1, @now, @rc) WITH NOWAIT
      END
    END
    -- Update this batch with the slots values of the last duplicate 
    UPDATE earlier_e SET 
      ExtEvSlot00 = later_e.ExtEvSlot00,
      ExtEvSlot01 = later_e.ExtEvSlot01,
      ExtEvSlot02 = later_e.ExtEvSlot02,
      ExtEvSlot04 = later_e.ExtEvSlot04
    FROM
      StagedEXCPInProgress AS earlier
      JOIN StagedEEVTInProgress AS earlier_e ON earlier_e.timestamp=earlier.timestamp
      JOIN StagedEXCPInProgress AS later ON later.parent_cid = earlier.parent_cid
                                        AND later.parent_id  = earlier.parent_id
                                        AND later.EXCP_NAME  = earlier.EXCP_NAME
      JOIN StagedEEVTInProgress AS later_e ON later_e.timestamp=later.timestamp 
    WHERE earlier.timestamp  < later.timestamp
      AND earlier.Flags & 1 = 1
      AND later.Flags & 1 = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating StagedEEVTInProgress slots to match the last duplicate event in the batch. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: Updated %d StagedEEVTInProgress slots to match the last duplicate event in the batch', 0, 1, @now, @rc) WITH NOWAIT
    END
  END
  --END 125296

  -- Check for 'singleton' exceptions within this batch. These are exceptions whose ExceptionType
  -- records have the Singleton bit set in the Flags attribute. If this batch contains any such
  -- records, we make sure that only the most recent one is actually created.
  DELETE FROM prev
  FROM StagedEXCPInProgress AS prev
  JOIN StagedEXCPInProgress AS new ON new.parent_cid = prev.parent_cid
                                   AND new.parent_id = prev.parent_id
                                   AND new.EXCP_NAME = prev.EXCP_NAME
  WHERE new.timestamp > prev.timestamp
    AND (new.AlertState <> prev.AlertState OR new.Priority <> prev.Priority)
    AND prev.Flags & 1 = 1
    AND new.Flags & 1 = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error removing duplicate singleton exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: removed %d duplicate singleton exception(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Look for existing singleton exceptions that are owned by a note and have new exceptions with a different alert state or priority.

  INSERT INTO @ownedSingletons (id, deleted, new_timestamp, new_id)
    SELECT o.eventid, e.deleted, new.timestamp, NULL
    FROM OwnedEvent o
    JOIN Exception_V e ON e.id = o.eventid
    JOIN StagedEXCPInProgress new ON new.parent_id  = e._ParentID
                                 AND new.parent_cid = e._ParentCID
                                 AND new.EXCP_NAME  = e._EXCP_NAME
    WHERE o.status = 0
      AND o.eventcid = 'EXCP'
      AND new.Flags & 1 = 1
      AND ((new.AlertState <> e._AlertState AND new.AlertState <> -1) OR (new.Priority <> e._Priority AND new.Priority <> -1))
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking for existing owned singleton exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d existing owned singleton exception(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
    SELECT '@ownedSingletons'='@ownedSingletons',* FROM @ownedSingletons

  -- Standard transaction logic. We start modifying permanent and public tables beyond this point.
  -- Prior to this we were only modifying our own temporary and private tables.
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedEXCPs
  ELSE
    SAVE TRANSACTION asisp_processStagedEXCPs

  -- Update owned singletons that have been cleared and bypass further processing of the new exception
  IF EXISTS (SELECT 1 FROM @ownedSingletons WHERE deleted = 1)
  BEGIN
    UPDATE e SET
      _AlertState = new.AlertState,
      _Priority   = new.Priority,
      _nEXCH      = 1
    FROM Exception_V AS e
      JOIN @ownedSingletons AS o ON o.id = e.id
      JOIN StagedEXCPInProgress AS new ON new.timestamp = o.new_timestamp
    WHERE o.deleted = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating cleared owned singleton exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated %d cleared owned singleton exception(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- If an owned cleared singleton exception is now green, it should not keep the note open
    UPDATE o SET status = 1
    FROM OwnedEvent AS o
      JOIN @ownedSingletons AS os ON os.id = o.eventid
      JOIN Exception_V AS e ON e.id = os.id
    WHERE o.eventcid = 'EXCP' AND e._AlertState = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating notes for green singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated %d notes that now own green singletons', 0, 1, @now, @rc) WITH NOWAIT
    END

    DELETE new
      FROM StagedEXCPInProgress AS new
      JOIN @ownedSingletons AS o ON o.new_timestamp = new.timestamp
      WHERE o.deleted = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error deleting new exceptions for cleared owned singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: deleted %d new exceptions for cleared owned singletons', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- Check for old 'singleton' exceptions. These are previously created exceptions of the singleton
  -- type. If the batch contains any of these exceptions, we have to make sure that any previously
  -- generated exceptions of this type whose alertstate and/or priority don't match are cleared.
  -- But do not match no clear events in the batch.
  INSERT INTO StagedEXCPInProgress(id,timestamp,ctime,parent_cid,parent_id,TIME,SEQ_NO,OBJECT_NAME,NATIVE_KEY,EXCP_OBJ_TYPE,EXCP_OBJ_NAME,EXCP_NAME,EXCP_CD,nEXCH,AlertState,Priority)
    SELECT
      e.id,p.timestamp,p.ctime,p.parent_cid,p.parent_id,p.TIME,p.SEQ_NO,p.OBJECT_NAME,p.NATIVE_KEY,p.EXCP_OBJ_TYPE,p.EXCP_OBJ_NAME,p.EXCP_NAME,
      'SC - SINGLETON EXCEPTION CLOSED BY NEW EXCEPTION WITH ALERTSTATE=' + CAST(p.AlertState AS NVARCHAR) + ', PRIORITY=' + CAST(p.Priority AS NVARCHAR) + ' AT ' + CONVERT(NVARCHAR(40), p.TIME, 121),
      p.nEXCH,e._AlertState,e._Priority
    FROM StagedEXCPInProgress AS p
      JOIN Exception_V AS e ON e._ParentCID = p.parent_cid
                           AND e._ParentID  = p.parent_id
                           AND e._EXCP_NAME = p.EXCP_NAME
    WHERE
      e.deleted = 0
      AND p.Flags & 1 = 1
      AND ((e._AlertState <> p.AlertState AND p.AlertState <> -1) OR (e._Priority <> p.Priority AND p.Priority <> -1))
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking up prior singleton exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d prior singleton exception(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Remove all rows from StagedEXCPInProgress where there is a subsequent 'clear' event
  DELETE FROM cleared
  FROM
    StagedEXCPInProgress AS cleared
    JOIN StagedEXCPInProgress AS clearing ON clearing.parent_cid = cleared.parent_cid
                                         AND clearing.parent_id = cleared.parent_id
                                         AND clearing.EXCP_NAME = cleared.EXCP_NAME
  WHERE (clearing.AlertState = cleared.AlertState OR clearing.AlertState = -1)
    AND (clearing.Priority = cleared.Priority OR clearing.Priority = -1)
    AND clearing.timestamp > cleared.timestamp
    AND (clearing.EXCP_CD = 'OK' OR clearing.EXCP_CD = 'SC' OR clearing.EXCP_CD like 'SC[ ]%')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error removing records with subsequent clear events. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: removed %d record(s) with subsequent clear events', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('CLEARED_REMOVED' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- The staged exceptions which have an AlertState or Priority of -1 and EXCP_CD = 'OK' indicate
  -- clears where we want to do a wildcard match on the column(s) with a -1 value.
  INSERT INTO StagedEXCPInProgress(id,timestamp,ctime,parent_cid,parent_id,TIME,SEQ_NO,OBJECT_NAME,NATIVE_KEY,EXCP_OBJ_TYPE,EXCP_OBJ_NAME,EXCP_NAME,EXCP_CD,nEXCH,AlertState,Priority)
    SELECT e.id,p.timestamp,p.ctime,p.parent_cid,p.parent_id,p.TIME,p.SEQ_NO,p.OBJECT_NAME,p.NATIVE_KEY,p.EXCP_OBJ_TYPE,p.EXCP_OBJ_NAME,p.EXCP_NAME,p.EXCP_CD,p.nEXCH,e._AlertState,e._Priority
    FROM StagedEXCPInProgress AS p
    JOIN Exception_V AS e ON e._ParentCID = p.parent_cid
                         AND e._ParentID  = p.parent_id
                         AND e._EXCP_NAME = p.EXCP_NAME
    WHERE
      (p.AlertState = -1 OR p.Priority = -1)
      AND e.deleted = 0
      AND (e._AlertState = p.AlertState OR p.AlertState = -1)
      AND (e._Priority = p.Priority OR p.Priority = -1)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error performing wildcard clear lookups. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d wildcard clear(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- The wildcard rows have now been replaced with 'clear' rows for the matching exceptions,
  -- so we can get rid of them
  DELETE FROM StagedEXCPInProgress WHERE AlertState = -1 OR Priority = -1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error deleting wildcard clears. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: removed %d wildcard clear(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('WILDCARD_REMOVED' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- At this point for each object there may one 'clear' event followed by 0 or more observations
  -- We find these by looking up the ids of the previously generated nondeleted exceptions that
  -- match them
  -- OA08983 - this now ensures that any TEC 'CLOSE' events that don't have a corresponding 'ACK' event
  -- do not get updated as this would cause them to go through the exception deleted code path, whereas
  -- they should be handled by the PROCESS_OWNERSHIP: code path
  UPDATE p SET id = e.id
  FROM StagedEXCPInProgress AS p
  JOIN Exception_V AS e ON e._ParentCID  = p.parent_cid
                       AND e._ParentID   = p.parent_id
                       AND e._EXCP_NAME  = p.EXCP_NAME
                       AND e._AlertState = p.AlertState
                       AND e._Priority   = p.Priority
  WHERE
    (p.EXCP_CD = 'OK' OR p.EXCP_CD = 'SC' OR p.EXCP_CD like 'SC[ ]%')
    AND e.deleted = 0
    AND p.timestamp NOT IN (SELECT timestamp FROM StagedEXCPOwnerInProgress)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking up ids for cleared exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d id(s) for cleared exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Check for owned exceptions
  UPDATE p SET id = e.id
  FROM
    StagedEXCPInProgress AS p
    JOIN Exception_V AS e ON e._ParentCID  = p.parent_cid
                         AND e._ParentID   = p.parent_id
                         AND e._EXCP_NAME  = p.EXCP_NAME
                         AND e._AlertState = p.AlertState
                         AND e._Priority   = p.Priority
    JOIN OwnedEvent AS o ON o.eventcid = 'EXCP'
                        AND o.eventid  = e.id
  WHERE
    e.deleted = 1
    AND p.EXCP_CD like 'SC[ ]%'
    AND o.status = 0
    AND p.id = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error updating owned exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d owned exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('AFTER_OWNED' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- Check for clears of owned exceptions. For these we just clear the OwnedEvent records
  TRUNCATE TABLE StagedEXCPClearedOwned
  INSERT INTO StagedEXCPClearedOwned
    SELECT e.id
    FROM
      StagedEXCPInProgress AS p
      JOIN Exception_A AS e ON e._ParentCID = p.parent_cid
                           AND e._ParentID  = p.parent_id
                           AND e._EXCP_NAME = p.EXCP_NAME
      JOIN OwnedEvent AS o ON o.eventcid = 'EXCP'
                          AND o.eventid  = e.id
    WHERE
      o.status = 0
      AND (p.EXCP_CD = 'OK' OR (p.Flags & 1 = 1 AND p.AlertState = 1 )) ---  or condition since singletons do not have to have a description of 'OK'
      AND p.EXCP_CD NOT like 'SC[ ]%'     --- excludes system generated take ownership events
      AND ( (e._AlertState = p.AlertState AND e._Priority = p.Priority)
            OR
            p.Flags & 1 = 1 )
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error checking for clears of owned exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d owned exceptions to clear', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE OwnedEvent SET status = 1 FROM StagedEXCPClearedOwned WHERE eventcid='EXCP' AND eventid=id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error updating OwnedEvent table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: updated %d row(s) in OwnedEvent', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Remove all 'false' clears. These are rows that have a EXCP_CD indicating a cleared exception
  -- but a zero id which tells us that there is no corresponding Exception_V record.
  DELETE FROM StagedEXCPInProgress WHERE id = 0 AND (EXCP_CD = 'OK' OR EXCP_CD = 'SC' OR EXCP_CD like 'SC[ ]%')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error removing false clears. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: removed %d false clear(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('FALSE_CLEARS' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- Delete all cleared exceptions
  TRUNCATE TABLE StagedEXCPDeletedHistory
  INSERT INTO StagedEXCPDeletedHistory(exch_id, excp_id)
    SELECT 0, id FROM StagedEXCPInProgress
    WHERE EXCP_CD = 'OK' OR EXCP_CD = 'SC' OR EXCP_CD like 'SC[ ]%'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error computing set of exceptions to delete. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d exception(s) to delete', 0, 1, @now, @rc) WITH NOWAIT
  END

  SET @deleted_count = @rc
  IF @deleted_count > 0
  BEGIN
    -- Create history records
    SELECT @date = CURRENT_TIMESTAMP

    INSERT INTO ExceptionHistory_C (ctime, _TIME, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
    SELECT @date, TIME, SEQ_NO, EXCP_NAME, EXCP_CD, excp_id
    FROM StagedEXCPDeletedHistory AS h
      JOIN StagedEXCPInProgress AS p ON p.id = h.excp_id
    WHERE NOT EXISTS (SELECT 1 FROM StagedEXCPInProgress AS later WHERE p.id = later.id AND later.timestamp > p.timestamp)
    ORDER BY exch_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error inserting into ExceptionHistory_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into ExceptionHistory_C', 0, 1, @now, @rc) WITH NOWAIT
    END
    -- Get the identity record
    IF @rc > 0
      SET @exch_base_id = IDENT_CURRENT('ExceptionHistory_C') - @rc + 1
    ELSE
      SET @exch_base_id = 0

    -- Update the staging table with the new id values
    UPDATE StagedEXCPDeletedHistory SET exch_id = row_id + @exch_base_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating EXCH ids in worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated EXCH ids for %d row(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Mark exception instances as deleted
    UPDATE Exception_C SET deleted = 1 FROM StagedEXCPDeletedHistory WHERE id = excp_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error marking exceptions as deleted. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: marked %d exception(s) as deleted', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Remove the StagedEXCPInProgress records for the exceptions just deleted
    DELETE FROM StagedEXCPInProgress FROM StagedEXCPDeletedHistory WHERE id = excp_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing StagedEXCPInProgress records for deleted exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: removed %d StagedEXCPInProgress record(s) for deleted exceptions', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('CLEARED_DELETED' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Update the ids of the StagedEXCPInProgress rows to the ids of previously generated
  -- exceptions that match them
  UPDATE p SET id = e.id
  FROM
    StagedEXCPInProgress AS p
    JOIN Exception_V AS e ON
      e._ParentCID      = p.parent_cid
      AND e._ParentID   = p.parent_id
      AND e._EXCP_NAME  = p.EXCP_NAME
      AND e._AlertState = p.AlertState
      AND e._Priority   = p.Priority
  WHERE
    e.deleted = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error looking up ids of previous exceptions to be updated. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: found %d id(s) of previous exceptions to be updated', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('PREV_IDS_LOOKUP' AS NVARCHAR(16)) AS STEP, * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Any rows in StagedEXCPInProgress that have nonzero ids are updated to existing exceptions
  TRUNCATE TABLE StagedEXCPUpdatedHistory
  INSERT INTO StagedEXCPUpdatedHistory(timestamp, exch_id, excp_id)
    SELECT timestamp, 0, id
    FROM StagedEXCPInProgress
    WHERE id > 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error creating table of updates to previous exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: created table of %d update(s) to previous exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT @updated_count = @rc

  IF @updated_count > 0
  BEGIN
    -- Create history records
    SET @date = CURRENT_TIMESTAMP

    -- insert the records
    INSERT INTO ExceptionHistory_C (ctime, _TIME, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
    SELECT
        @date, TIME, SEQ_NO, EXCP_NAME, EXCP_CD, excp_id
    FROM StagedEXCPUpdatedHistory AS h
      JOIN StagedEXCPInProgress     AS a ON a.timestamp = h.timestamp
    ORDER BY row_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error inserting into ExceptionHistory_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into ExceptionHistory_C', 0, 1, @now, @rc) WITH NOWAIT
    END
    IF @rc > 0
      SET @exch_base_id = IDENT_CURRENT('ExceptionHistory_C') - @rc + 1
    ELSE
      SET @exch_base_id = 0

    -- Update the staging table with the new id values
    UPDATE StagedEXCPUpdatedHistory SET exch_id = row_id + @exch_base_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating EXCH ids in worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated EXCH ids for %d row(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
    END

    IF @verbose >= 2
    BEGIN
        PRINT ' '
        PRINT 'Distinct StagedEXCPUpdatedHistory'
        SELECT DISTINCT exch_id, COUNT(*) AS 'COUNT' FROM StagedEXCPUpdatedHistory GROUP BY exch_id HAVING COUNT(*)>1
        PRINT ' '
        PRINT 'Selected StagedEXCPUpdatedHistory based upon id in ExceptionHistory_C'
        SELECT * FROM StagedEXCPUpdatedHistory WHERE exch_id IN (SELECT id FROM ExceptionHistory_C)
        PRINT ' '
        PRINT 'All StagedEXCPUpdatedHistory'
        SELECT * FROM StagedEXCPUpdatedHistory
        PRINT ' '
        PRINT 'MAX(id) in ExceptionHistory_C'
        SELECT MAX(id) AS 'MAX(id)' FROM ExceptionHistory_C
        PRINT ' '
        PRINT 'EXCH_ID'
        SELECT last_id FROM EXCH_ID
    END

    -- Update temporal attributes of exception
    UPDATE Exception_C SET
      _TIME    = a.TIME,
      _SEQ_NO  = a.SEQ_NO,
      _EXCP_CD = a.EXCP_CD,
      _nEXCH   = _nEXCH + (SELECT SUM(nEXCH) FROM StagedEXCPInProgress AS a2 WHERE a2.id = a.id)
    FROM
      StagedEXCPInProgress AS a
    WHERE
      Exception_C.id = a.id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating attributes of previous exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated attributes of %d previous exception(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Remove the StagedEXCPInProgress records for the exceptions just updated
    DELETE FROM StagedEXCPInProgress WHERE id > 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error removing processed rows from worktable for updated exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: removed %d processed row(s) from worktable for updated exceptions', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT 'UPDATED_REMOVED' AS STEP, * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Remove any exceptions with 'Eliminate' priority
  DELETE FROM StagedEXCPInProgress WHERE Priority=6
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error removing rows with ''Eliminate'' priority. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: Removed %d row(s) with ''Eliminate'' priority', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- StagedEXCPInserted is a worktable that is to contain the distinct exceptions that are to
  -- be created.
  TRUNCATE TABLE StagedEXCPInserted
  INSERT INTO StagedEXCPInserted(excp_timestamp)
    SELECT latest.timestamp
    FROM StagedEXCPInProgress AS latest
    WHERE NOT EXISTS
    (
      SELECT 1
      FROM StagedEXCPInProgress later
      WHERE
        later.parent_cid     = latest.parent_cid
        AND later.parent_id  = latest.parent_id
        AND later.EXCP_NAME  = latest.EXCP_NAME
        AND later.AlertState = latest.AlertState
        AND later.Priority   = latest.Priority
        AND later.timestamp  > latest.timestamp
    )
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error populating table of distinct exceptions to create. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: populated table of %d distinct exception(s) to create', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Update the nEXCH column of the new exceptions to be equal to the sum of all observations for each
  -- distinct set of parent_cid, parent_id, EXCP_NAME, AlertState, and Priority
  UPDATE e SET nEXCH =
    (
      SELECT SUM(nEXCH)
      FROM StagedEXCPInProgress e2
      WHERE
        e2.parent_cid = e.parent_cid
        AND e2.parent_id = e.parent_id
        AND e2.EXCP_NAME = e.EXCP_NAME
        AND e2.AlertState = e.AlertState
        AND e2.Priority = e.Priority
        AND e2.timestamp <= e.timestamp
    )
  FROM StagedEXCPInProgress AS e
    JOIN StagedEXCPInserted AS i ON i.excp_timestamp = e.timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error updating count of exceptions. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: updated count for %d new exception(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT @inserted_count = COUNT(*) FROM StagedEXCPInserted
  IF @inserted_count = 0 
  BEGIN 
    IF @owner_event_count > 0 GOTO PROCESS_OWNERSHIP            -- KDM
    ELSE GOTO CLEANUP
  END

  -- Insert Exception class records
  -- Insert Exception attribute records
  INSERT INTO Exception_C
  (
    _TIME,            -- 3
    _SEQ_NO,          -- 5
    _OBJECT_NAME,     -- 6
    _NATIVE_KEY,      -- 7
    _EXCP_NAME,       -- 10
    _EXCP_CD,         -- 11
    _nEXCH,           -- 12
    _AlertState,      -- 13
    _Priority,        -- 14
    _ParentCID,       -- 15
    _ParentID,        -- 16
    _flags            -- 17
  )
  SELECT
    e.TIME,                 -- 3
    e.SEQ_NO,               -- 5
    e.OBJECT_NAME,          -- 6
    e.NATIVE_KEY,           -- 7
    e.EXCP_NAME,            -- 10
    e.EXCP_CD,              -- 11
    e.nEXCH,                -- 12
    ISNULL(e.AlertState,0), -- 13
    ISNULL(e.Priority,0),   -- 14
    e.parent_cid,           -- 15
    e.parent_id,            -- 16
    e.Flags                 -- 17
  FROM
    StagedEXCPInProgress AS e
    JOIN StagedEXCPInserted AS i ON i.excp_timestamp = e.timestamp
  ORDER BY i.row_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error inserting into Exception_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into Exception_C', 0, 1, @now, @rc) WITH NOWAIT
  END
  -- Get the identity value
  IF @rc > 0
    SET @base_id = IDENT_CURRENT('Exception_C') - @rc + 1
  ELSE
    SET @base_id = 0

  -- Update the id field for the new exceptions. This has to be done for all rows in StagedEXCPInProgress,
  -- not just the rows that are being used to create the exceptions. We need to do this because we use
  -- all of the rows for creating the ExceptionHistory records for the new exceptions.
  UPDATE e SET id = @base_id + i.row_id
  FROM
    StagedEXCPInProgress AS e
    JOIN StagedEXCPInProgress eprimary ON
      eprimary.parent_cid     = e.parent_cid
      AND eprimary.parent_id  = e.parent_id
      AND eprimary.EXCP_NAME  = e.EXCP_NAME
      AND eprimary.AlertState = e.AlertState
      AND eprimary.Priority   = e.Priority
    JOIN StagedEXCPInserted AS i ON i.excp_timestamp = eprimary.timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error updating EXCP ids in worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: updated EXCP ids for %d row(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    SELECT CAST('NEW_EXCP' AS NVARCHAR(16)) AS STEP, *
    FROM
      StagedEXCPInProgress AS e
      JOIN StagedEXCPInserted AS i ON i.excp_timestamp = e.timestamp
    SET NOCOUNT ON
  END

  -- Old singleton exceptions that are owned but have not been cleared may be replaced by new exceptions.
  -- We need to adjust the OwnedEvent and NoteDependent tables to reflect the new exception ids.
  IF EXISTS (SELECT 1 FROM @ownedSingletons WHERE deleted = 0)
  BEGIN
    -- Back-fill the object ids for the new exceptions
    UPDATE o SET new_id = new.id
      FROM @ownedSingletons o
      JOIN StagedEXCPInProgress new ON o.new_timestamp = new.timestamp
      WHERE new.id > 0 AND o.deleted = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating @ownedSingletons with new object ids. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated %d rows in @ownedSingletons with new object ids', 0, 1, @now, @rc) WITH NOWAIT
    END

    UPDATE oe SET eventid = os.new_id
      FROM OwnedEvent oe
      JOIN @ownedSingletons os ON os.id = oe.eventid
      WHERE os.new_id > 0 AND os.deleted = 0 AND oe.eventcid = 'EXCP'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating OwnedEvent for new singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated %d rows in OwnedEvent for new singletons', 0, 1, @now, @rc) WITH NOWAIT
    END

    UPDATE nd SET eventid = os.new_id
      FROM NoteDependent nd
      JOIN @ownedSingletons os ON os.id = nd.eventid
      WHERE os.new_id > 0 AND os.deleted = 0 AND nd.eventcid = 'EXCP'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error updating NoteDependent for new singletons. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: updated %d rows in NoteDependent for new singletons', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  --
  -- For any exceptions with extended attributes, create
  -- an EEXC record                                              KDM
  IF @extended_event_count > 0
  BEGIN
    INSERT INTO ExtendedEvent_C(id, name, eventcid)
      SELECT e.id, CONVERT(NVARCHAR(54), e.EXCP_NAME), yy.eventcid
      FROM StagedEXCPInProgress AS e
      JOIN StagedEXCPInserted AS i ON i.excp_timestamp = e.timestamp
      JOIN StagedEEVTInProgress AS yy ON yy.timestamp = e.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error inserting into ExtendedEvent_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into ExtendedEvent_C', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Insert ExtendedEvent attribute records
    INSERT INTO ExtendedEvent_A
    (
      id,
      eventcid,
      _ExtEvBinData,
      _ExtEvLongTxt,
      _ExtEvSlot00,
      _ExtEvSlot01,
      _ExtEvSlot02,
      _ExtEvSlot03,
      _ExtEvSlot04,
      _ExtEvSlot05,
      _ExtEvSlot06,
      _ExtEvSlot07,
      _ExtEvSlot08,
      _ExtEvSlot09,
      _ExtEvSlot10,
      _ExtEvSlot11,
      _ExtEvSlot12,
      _ExtEvSlot13,
      _ExtEvType,
      _ParentCID,
      _ParentID
    )
      SELECT
        yy.id,
        e.eventcid,
        ExtEvBinData,
        ExtEvLongTxt,
        ExtEvSlot00,
        ExtEvSlot01,
        ExtEvSlot02,
        ExtEvSlot03,
        ExtEvSlot04,
        ExtEvSlot05,
        ExtEvSlot06,
        ExtEvSlot07,
        ExtEvSlot08,
        ExtEvSlot09,
        ExtEvSlot10,
        ExtEvSlot11,
        ExtEvSlot12,
        ExtEvSlot13,
        ExtEvType,
        parent_cid,
        parent_id
      FROM StagedEEVTInProgress AS e
      JOIN StagedEXCPInserted AS i ON i.excp_timestamp = e.timestamp
      JOIN StagedEXCPInProgress AS yy ON yy.timestamp = e.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error inserting into ExtendedEvent_A. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into ExtendedEvent_A', 0, 1, @now, @rc) WITH NOWAIT
    END
  END
  -- End of Extended Excepton processing                         KDM

  -- Create history records
  SET @date = CURRENT_TIMESTAMP
  SELECT @inserted_count = COUNT(*) FROM StagedEXCPInProgress
  TRUNCATE TABLE StagedEXCPInsertedHistory
  INSERT INTO StagedEXCPInsertedHistory(timestamp)
    SELECT timestamp FROM StagedEXCPInProgress
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error computing set of ExceptionHistory records to create. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: computed set of %d ExceptionHistory record to create', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert the records
  INSERT INTO ExceptionHistory_C (ctime, _TIME, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
  SELECT @date, p.TIME, p.SEQ_NO, p.EXCP_NAME, p.EXCP_CD, p.id
  FROM StagedEXCPInsertedHistory AS h
    JOIN StagedEXCPInProgress AS p ON p.timestamp = h.timestamp
  ORDER BY h.rowno
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error inserting into ExceptionHistory_A. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: inserted %d row(s) into ExceptionHistory_A', 0, 1, @now, @rc) WITH NOWAIT
  END
  -- Get the identity value
  IF @rc > 0
    SET @base_id = IDENT_CURRENT('ExceptionHistory_C') - @rc + 1
  ELSE
    SET @base_id = 0

PROCESS_OWNERSHIP:
  -- Process TEC ownership events
  IF @owner_event_count > 0
  BEGIN
    EXEC @rc = asisp_processStagedTECOwnership @exteventtype = @exteventtype, @verbose = @verbose
    IF @rc < 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: asisp_processStagedTECOwnership failed with return code ', 3, 1, @now, @rc) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN @rc
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: asisp_processStagedTECOwnership succeeded with return code %d', 0, 1, @now, @rc) WITH NOWAIT
    END
  END -- Process TEC ownership events

  --
  -- Cleanup the StagedEVT table                            KDM
  IF @extended_event_count > 0
  BEGIN
    DELETE FROM pp
    FROM StagedEEVT AS pp
    JOIN StagedEEVTInProgress AS qq ON
      qq.timestamp = pp.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: error pruning records from StagedEEVT. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedEXCPs: pruned %d record(s) from StagedEEVT', 0, 1, @now, @rc) WITH NOWAIT
    END

    --
    -- Prune the ownership events from the StagedEXCP table

    IF @owner_event_count > 0
    BEGIN
      DELETE FROM pp
      FROM StagedEXCP AS pp
      JOIN StagedEXCPOwnerInProgress AS qq ON
        qq.timestamp = pp.timestamp
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: error pruning ownership records from StagedEXCP. Error=%d', 3, 1, @now, @err) WITH NOWAIT
        ROLLBACK TRANSACTION asisp_processStagedEXCPs
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedEXCPs: pruned %d ownership record(s) from StagedEXCP', 0, 1, @now, @rc) WITH NOWAIT
      END
    END
  END
  --                                                        KDM

CLEANUP:
  -- Remove processed records from StagedEXCP
  DELETE FROM s 
    FROM StagedEXCP AS s 
    WHERE timestamp <= @max_timestamp
      AND NOT EXISTS (SELECT 1 FROM @defer_excp AS d WHERE d.timestamp = s.timestamp)
      AND NOT EXISTS (SELECT 1 FROM @defer_singleton AS t WHERE t.timestamp = s.timestamp)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: error deleting processed records from StagedEXCP. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: deleted %d processed record(s) from StagedEXCP', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedEXCPs

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedEXCPs: Finished. Number of records processed = %d', 0, 1, @now, @num_events_to_process) WITH NOWAIT
  END
  RETURN @num_events_to_process
GO
