-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM
--
-- Restricted Materials of IBM
--
-- 5724-C51
--
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedtecownership.sql
-- SID: 1.3
-- Date Modified: 04/09/15 16:33:49
--
-- Module Description
--   Defines the asisp_processStagedTECOwnership stored procedure
USE Object
GO

-- asisp_processStagedTECOwnership
PRINT 'Creating procedure asisp_processStagedTECOwnership'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedTECOwnership' AND type = 'P')
  DROP PROCEDURE asisp_processStagedTECOwnership
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM
--
-- Restricted Materials of IBM
--
-- 5724-C51
--
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedtecownership.sql
-- SID: 1.3
-- Date Modified: 04/09/15 16:33:49
--
-- Helper procedure for asisp_processStagedEXCPs.
-- This procedure handles creating notes in TBSM for events that have been owned in TEC.
--
-- Returns:
--   >= 0 - the number of events taken ownership of
--   < 0  - error
--
-- There is no transaction logic in this procedure since it is always being called from within
-- an outer transaction, and a failure in this procedure implies a failure in the caller.
CREATE PROCEDURE asisp_processStagedTECOwnership
  @exteventtype INT,
  @verbose      TINYINT = 0
AS    
  CREATE TABLE #BulkEvents -- Input table for _CreateOwnershipNotesBulk
  (
    noteid      INT           NULL,
    obj_cid     ClassID       NOT NULL,
    obj_id      ObjID         NOT NULL,
    evt_cid     ClassID       NOT NULL,
    evt_id      ObjID         NOT NULL,
    Status      NVARCHAR(16)  NOT NULL,
    UserID      NVARCHAR(255) NOT NULL,
    Flags       INT           NOT NULL
  )

  CREATE TABLE #Objects  -- Input table for _SetMNO2Flags
  (
    ident       INT           NOT NULL IDENTITY (1,1),
    cid         ClassID       NOT NULL,
    id          ObjID         NOT NULL,
    value       TINYINT       NULL DEFAULT NULL,
    rCorr       INT           NULL DEFAULT NULL,
    errorId     INT           NULL DEFAULT NULL
  )

  -- Privately used table
  DECLARE @SetMNO2Flag TABLE 
  (
    ident       INT           NOT NULL IDENTITY (1,1),
    cid         NVARCHAR(4)   NOT NULL,
    id          INT           NOT NULL,
    field       NVARCHAR(100) NOT NULL,
    value       TINYINT       NOT NULL
  )

  -- Variable declarations
  DECLARE
    @err            INT,
    @now            NVARCHAR(40),
    @rc             INT,
    @owner          NVARCHAR(100),
    @body           NVARCHAR(100),
    @subject        NVARCHAR(100),
    @resolution     NVARCHAR(110),
    @event_id       ObjID,
    @obj_cid        ClassID,
    @obj_id         ObjID,
    @status         NVARCHAR(16),
    @eevt_timestamp BINARY(8),
    @serverhandle   NVARCHAR(16),
    @eventhandle    NVARCHAR(16),
    @datereception  NVARCHAR(32),
    @tecstatus      NVARCHAR(32),
    @tecclass       NVARCHAR(255),
    @administrator  NVARCHAR(128),
    @note_id        ObjID,
    @mno2_flag_name NVARCHAR(100),
    @excp_name      NVARCHAR(255),
    @admin          NVARCHAR(128),
    @clearAlerts    NVARCHAR(3),
    @flags          INT,
    @nNotes         INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECOwnership: Starting', 0, 1, @now) WITH NOWAIT
  END

  SET @nNotes = 0 -- this variable will be the return code of the procedure if successful
  
  -- Lookup SystemConfiguration values for creating notes from TEC ownership events
  SELECT @owner       = RTRIM(propertyvalue) FROM SystemConfiguration WITH (NOLOCK) WHERE groupname = 'TEC_EVENT_OWNERSHIP' AND propertyname = 'TEC_NOTE_OWNER'
  SELECT @resolution  = RTRIM(propertyvalue) FROM SystemConfiguration WITH (NOLOCK) WHERE groupname = 'TEC_EVENT_OWNERSHIP' AND propertyname = 'TEC_NOTE_RESOLUTION'
  SELECT @clearAlerts = RTRIM(propertyvalue) FROM SystemConfiguration WITH (NOLOCK) WHERE groupname = 'TEC_EVENT_OWNERSHIP' AND propertyname = 'CLEAR_ALERTS_ON_OWNERSHIP_DEFAULT' 
  IF @clearAlerts = 'YES'
    SET @flags = 1
  ELSE
    SET @flags = 0

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECOwnership: TEC_NOTE_OWNER                    = [%s]', 0, 1, @now, @owner       ) WITH NOWAIT
    RAISERROR('%s: asisp_processStagedTECOwnership: TEC_NOTE_RESOLUTION               = [%s]', 0, 1, @now, @resolution  ) WITH NOWAIT
    RAISERROR('%s: asisp_processStagedTECOwnership: CLEAR_ALERTS_ON_OWNERSHIP_DEFAULT = [%s]', 0, 1, @now, @clearAlerts ) WITH NOWAIT
  END

  -- 137789 is for excessive blocking between the console (eg TakeOwnership) and the SEL processing of TEC events
  -- in StagedEXCP/MESGs.  This was related to these procs calling _SetMNO2Flags per event.
  -- To avoid this, requests for _SetMNO2Flags are now buffered in @SetMNO2Flag.  If any entries are buffered,
  -- #Objects is populated and _SetMNO2Flags is invoked per MNO2 Flag name to process all entries in #Object.
  -- This is more efficient since it minimizes the number of separate updates to <cname>_C tables.
  -- In addition these updates are done at the end of cursor processing, so the time the update locks are
  -- held is reduced.
  --
  -- As a further performance improvement we also collect all of the events to be owned into the #BulkEvents
  -- table and call _CreateOwnershipNotesBulk once at the end of the procedure if necessary.

  -- OA08983 - Joining on StagedEXCPOwnerInProgress as it is possible for multiple TEC events to have the 
  -- same eventhandle, serverhandle, datereception, tecclass. The code was opening/closing ownership notes 
  -- for the wrong event/resource, but we now get the unique event id for which the ownership note has to
  -- be opened/closed.

  DECLARE EXCPOwnedEvent_C INSENSITIVE CURSOR FOR
  SELECT e.timestamp, e.ExtEvSlot00, e.ExtEvSlot01, e.ExtEvSlot02, e.ExtEvSlot03, 
         e.ExtEvSlot04, e.ExtEvSlot06, o.parent_cid, o.parent_id
  FROM StagedEEVTInProgress AS e
    JOIN StagedEXCPOwnerInProgress AS o ON o.timestamp = e.timestamp
  WHERE e.ExtEvType=@exteventtype
  OPEN EXCPOwnedEvent_C
  FETCH NEXT FROM EXCPOwnedEvent_C INTO @eevt_timestamp, @eventhandle, @serverhandle, @datereception, @tecstatus, 
                                        @tecclass, @administrator, @obj_cid, @obj_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @event_id = NULL
    SELECT @event_id=id
    FROM ExtendedEvent_A
    WHERE _ExtEvSlot00 = @eventhandle 
      AND _ExtEvSlot01 = @serverhandle 
      AND _ExtEvSlot02 = @datereception 
      AND _ExtEvSlot04 = @tecclass
      AND _ParentCID   = @obj_cid
      AND _ParentID    = @obj_id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: unable to retrieve eventid for ownership events. Error=%d. eventhandle=%s, serverhandle=%s, datereception=%s, tecclass=%s',
                3, 1, @now, @err, @eventhandle, @serverhandle, @datereception, @tecclass) WITH NOWAIT
      DEALLOCATE EXCPOwnedEvent_C
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: marked %d owned exception(s) as closed', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- OA02933: Added singleton check to handle case where existing extended event does not match
    -- the extended event that created the most recent instance of the singleton. This can happen
    -- if the TEC rule that generates theclose  event use the slots from the oldest event instead
    -- of from the most recent.
    IF @event_id IS NULL
    BEGIN
      -- Singleton check
      SET @excp_name = NULL
      SELECT @excp_name = EXCP_NAME, @obj_cid = parent_cid, @obj_id = parent_id
        FROM StagedEXCPInProgress
        WHERE timestamp = @eevt_timestamp AND Flags & 1 = 1
      IF @excp_name IS NOT NULL
      BEGIN
        -- The StagedEXCP record tells us this is a singleton event, so see if there is
        -- a current exception on the object with this name (we don't care about the
        -- extended attributes since by definition there must be only one singleton
        -- of a given name on a given object).
        SELECT @event_id = e.id 
        FROM Exception_V AS e
        WHERE _ParentCID = @obj_cid AND _ParentID = @obj_id AND _EXCP_NAME = @excp_name
      END
    END

    -- For debug purposes, generate RAISERROR when event not found
    IF @event_id IS NULL
      RAISERROR('%s: asisp_processStagedTECOwnership: No matching event found for ownership event. eventhandle=%s, serverhandle=%s, datereception=%s, tecstatus=%s, tecclass=%s, administrator=%s',
                0, 1, @now, @eventhandle, @serverhandle, @datereception, @tecstatus, @tecclass, @administrator) WITH NOWAIT

    IF @event_id IS NOT NULL
    BEGIN
      -- Did a TEC operator take ownership or close the event
      IF @tecstatus = 'ACK' OR @tecstatus = 'CLOSED' OR @tecstatus = 'ACN'
      BEGIN
        --
        -- Check to see if we already have a note for this event, TEC could
        -- have acknowledged the event earlier and now closing it.
        SELECT @note_id=noteid
        FROM OwnedEvent
        WHERE eventid=@event_id AND parentcid=@obj_cid AND parentid=@obj_id
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_processStagedTECOwnership: error retrieving note id. Error=%d', 3, 1, @now, @err) WITH NOWAIT
          DEALLOCATE EXCPOwnedEvent_C
          RETURN -30000
        END
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          IF @rc = 1
            RAISERROR('%s: asisp_processStagedTECOwnership: noteid lookup found note %d', 0, 1, @now, @note_id) WITH NOWAIT
          ELSE
            RAISERROR('%s: asisp_processStagedTECOwnership: noteid lookup returned %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
        END

        IF @note_id IS NULL
        BEGIN
          IF @administrator IS NULL OR @administrator = '' SET @admin = @owner
          ELSE SET @admin = @administrator

          SET @status = 'Open'

          IF @tecstatus = 'ACN'
            INSERT INTO #BulkEvents VALUES ( NULL, @obj_cid, @obj_id, 'EXCP', @event_id, @status, @admin,  0 )
          ELSE
          BEGIN
            IF @tecstatus = 'CLOSED'
              SET @status = 'Closed'
            INSERT INTO #BulkEvents VALUES ( NULL, @obj_cid, @obj_id, 'EXCP', @event_id, @status, @admin,  @flags )
          END
          SELECT @err = @@ERROR, @rc = @@ROWCOUNT
          IF @err <> 0
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_processStagedTECOwnership: Error %d creating record for ownership of EXCP %d', 3, 1, @now, @err, @event_id) WITH NOWAIT
            DEALLOCATE EXCPOwnedEvent_C
            RETURN -30000
          END
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_processStagedTECOwnership: Taking ownership of EXCP %d on %s %d', 0, 1, @now, @event_id, @obj_cid, @obj_id) WITH NOWAIT
          END
        END
        ELSE
        BEGIN
          --
          -- If we already had a note, then either a TBSM and TEC operator took ownership
          -- at the same time, in which case TEC wins because we can not force the issue
          -- on their side, or an earlier ACK did not make it to TEC.  Either case, TEC wins.
          -- I need to think about this a bit more ????
          IF @tecstatus = 'ACK'
          BEGIN
            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: asisp_processStagedTECOwnership: _CommitNote received an ACK for message %d already owned.', 0, 1, @now, @event_id) WITH NOWAIT
            END
          END
        END

        IF @tecstatus = 'CLOSED'
        BEGIN
          IF @note_id IS NOT NULL
          BEGIN
            UPDATE Note_A SET _Resolution = @resolution, _Status = N'Closed' WHERE id = @note_id
            SET @err = @@ERROR
            IF @rc < 0 OR @err <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: asisp_processStagedTECOwnership: error updating Resolution field on note %d. RC=%d, Error=%d', 3, 1, @now, @note_id, @rc, @err) WITH NOWAIT
              DEALLOCATE EXCPOwnedEvent_C
              RETURN -30000
            END
          END

          --Remove automation icon is possible.
          UPDATE AutomationFlag SET autocount = autocount - 1 WHERE cid = @obj_cid AND id = @obj_id AND autocount > 0
          SELECT @err = @@ERROR, @rc = @@ROWCOUNT
          IF @err <> 0
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_processStagedTECOwnership: error updating autocount in AutomationFlag. Error=%d', 3, 1, @now, @err) WITH NOWAIT
          END
          IF @rc = 0
          BEGIN
            INSERT INTO @SetMNO2Flag(cid, id, field, value) 
              VALUES ( @obj_cid, @obj_id, 'AutomationStarted', 0 )
          END
          IF (SELECT af.autocount FROM AutomationFlag AS af WHERE af.cid = @obj_cid AND af.id = @obj_id) = 0
          BEGIN
            DELETE FROM AutomationFlag WHERE cid = @obj_cid AND id = @obj_id AND autocount = 0
            SELECT @err = @@ERROR, @rc = @@ROWCOUNT
            IF @err <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: asisp_processStagedTECOwnership: error deleting autocount records from AutomationFlag. Error=%d', 3, 1, @now, @err) WITH NOWAIT
            END
            INSERT INTO @SetMNO2Flag(cid, id, field, value) 
              VALUES ( @obj_cid, @obj_id, 'AutomationStarted', 0 )
          END
          INSERT INTO @SetMNO2Flag(cid, id, field, value) 
            VALUES ( @obj_cid, @obj_id, 'Problem', 0 )
        END
      END
      ELSE
      BEGIN
        -- Is automation taking care of the problem at TEC
        IF @tecstatus = 'RESPONSE'
        BEGIN
          --Keep a count for number of events setting the automation icon.
          UPDATE AutomationFlag SET autocount = autocount + 1 WHERE cid = @obj_cid AND id = @obj_id
          SELECT @err = @@ERROR, @rc = @@ROWCOUNT
          IF @err <> 0
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_processStagedTECOwnership: error updating autocount in AutomationFlag. Error=%d', 3, 1, @now, @err) WITH NOWAIT
          END
          IF @rc  = 0
          BEGIN
            INSERT INTO AutomationFlag (cid, id, autocount) VALUES (@obj_cid, @obj_id, 1)
            SELECT @err = @@ERROR, @rc = @@ROWCOUNT
            IF @err <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: asisp_processStagedTECOwnership: error %d inserting into AutomationFlag', 3, 1, @now, @err) WITH NOWAIT
            END
          END
          INSERT INTO @SetMNO2Flag(cid, id, field, value) 
            VALUES (  @obj_cid, @obj_id, 'AutomationStarted', 1 )
        END
        ELSE
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_processStagedTECOwnership - unrecognized status %s', 2, 1, @now, @tecstatus) WITH NOWAIT
        END
      END -- Did a TEC operator take ownership or close the event
    END -- We should only have one event that matched the above attributes

    FETCH NEXT FROM EXCPOwnedEvent_C INTO @eevt_timestamp, @eventhandle, @serverhandle, @datereception, @tecstatus, 
                                          @tecclass, @administrator, @obj_cid, @obj_id
  END
  DEALLOCATE EXCPOwnedEvent_C

  -- Create ownership notes for events
  IF EXISTS (SELECT 1 FROM #BulkEvents)
  BEGIN
    SELECT @body    = RTRIM(propertyvalue) FROM SystemConfiguration WITH (NOLOCK) WHERE groupname = 'TEC_EVENT_OWNERSHIP' AND propertyname = 'TEC_NOTE_BODY'
    SELECT @subject = RTRIM(propertyvalue) FROM SystemConfiguration WITH (NOLOCK) WHERE groupname = 'TEC_EVENT_OWNERSHIP' AND propertyname = 'TEC_NOTE_SUBJECT'
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: TEC_NOTE_BODY    = [%s]', 0, 1, @now, @body) WITH NOWAIT
      RAISERROR('%s: asisp_processStagedTECOwnership: TEC_NOTE_SUBJECT = [%s]', 0, 1, @now, @subject) WITH NOWAIT
    END

    EXEC @rc = _CreateOwnershipNotesBulk @Subject = @subject, @Body = @body, @Resolution = @resolution, @verbose = @verbose
    IF @rc < 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: _CreateOwnershipNotesBulk failed with return code %d', 3, 1, @now, @rc) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: Created %d ownership note(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
    SET @nNotes = @rc
  END -- Create ownership notes for events

  -- Call _SetMNO2Flags for objects
  IF EXISTS (SELECT 1 FROM @SetMNO2Flag)
  BEGIN
    DELETE dup   -- remove entries that have been superseeded by a later setting
      FROM @SetMNO2Flag AS f
      JOIN @SetMNO2Flag AS dup ON dup.cid = f.cid AND dup.id = f.id AND dup.field = f.field
                              AND dup.ident < f.ident
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECOwnership: removed %d redundant row(s) from @SetMNO2Flag', 0, 1, @now, @rc) WITH NOWAIT
    END
    DECLARE cur_setmno2flag INSENSITIVE CURSOR FOR 
      SELECT DISTINCT field FROM @SetMNO2Flag 
    OPEN cur_setmno2flag
    FETCH NEXT FROM cur_setmno2flag INTO @mno2_flag_name
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      TRUNCATE TABLE #Objects 
      INSERT INTO #Objects (cid, id, value, rCorr, errorId)
        SELECT cid, id, value, 0, 0 
          FROM @SetMNO2Flag 
          WHERE field = @mno2_flag_name
          ORDER BY ident
      SET @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedTECOwnership: inserted %d row(s) into #Objects for %s', 0, 1, @now, @rc, @mno2_flag_name) WITH NOWAIT
      END
      EXEC @rc = _SetMNO2Flag 'ROOT',1,@mno2_flag_name, 0
      IF @rc < 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedTECOwnership: Error %d from _SetMNO2Flag for %s', 0, 1, @now, @rc, @mno2_flag_name) WITH NOWAIT
      END
      FETCH NEXT FROM cur_setmno2flag INTO @mno2_flag_name
    END
    DEALLOCATE cur_setmno2flag
  END -- Call _SetMNO2Flags for objects
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECOwnership: Finished. Number of notes created = %d', 0, 1, @now, @nNotes) WITH NOWAIT
  END

  RETURN @nNotes
GO

