-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_register_rodm_networks.sql
-- SID: 1.2.4.7
-- Date Modified: 04/12/06 17:36:58
-- 
-- Module description
--   Defines the asisp_register_RODM_networks stored procedure

USE Object
GO

--------------------------------------------------------------------------------------------
-- asisp_register_RODM_networks
PRINT 'Create procedure asisp_register_RODM_networks'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_register_RODM_networks' AND type = 'P')
  DROP PROCEDURE asisp_register_RODM_networks
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_register_rodm_networks.sql
-- SID: 1.2.4.7
-- Date Modified: 04/12/06 17:36:58
-- Called from scheduled task created by RODM_object_discovery_backend to register SNANetworks
-- asynchronously.
CREATE PROCEDURE asisp_register_RODM_networks
  @verbose  TINYINT = 0
AS
  SET NOCOUNT ON

  DECLARE
    @cid                ClassID,
    @id                 ObjID,
    @now                NVARCHAR(40),
    @RegistrationScope  TINYINT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_register_RODM_networks: Starting', 0, 1, @now) WITH NOWAIT
  END

  -- Call _RODMRegister for all networks that need it.  The list
  -- of networks is in a table that's maintained by 
  -- RODM_object_discovery_backend.  This stored procedure is
  -- designed to be executed from a SQL job - it might have been
  -- nice to have done what this procedure does as part of the
  -- discovery backend but the korn script that gets called
  -- can get locked out by a SQL transaction and create a deadlock
  -- situation.  The purpose of this procedure is so that it can
  -- be run asynchronously

  DECLARE regc CURSOR FOR
    SELECT cid, id FROM RODMNetworksToRegister
  OPEN regc
  FETCH NEXT FROM regc INTO @cid, @id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- RODMNetworksToRegister may contain non SNNT objects placed by RODM_object_discovery_backend
    --   make sure that these do NOT register their descendents.
    SET @RegistrationScope = CASE WHEN 'SNNT' = @cid THEN 3 WHEN 'GMFF' = @cid THEN 3 ELSE 1 END
    EXEC _RODMRegister @cid, @id,
      @RegistrationStatus = 2,                     -- re-register
      @RegistrationScope  = @RegistrationScope,    -- root and descendents
      @ResponseType       = 1                      -- exceptions only
    DELETE FROM RODMNetworksToRegister WHERE cid=@cid AND id=@id
    FETCH NEXT FROM regc INTO @cid, @id
  END
  DEALLOCATE regc

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_register_RODM_networks: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
