-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_uploadgetrodmobjects.sql
-- SID: 1.16
-- Date Modified: 04/12/06 17:41:19
--
-- Module description
--   Defines the asisp_asisp_uploadGetRODMObjects stored procedure

USE Object
GO

--------------------------------------------------------------------------------------------
-- asisp_uploadGetRODMObjects
PRINT 'Create procedure asisp_uploadGetRODMObjects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_uploadGetRODMObjects' AND type = 'P')
  DROP PROCEDURE asisp_uploadGetRODMObjects
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM
--
-- Restricted Materials of IBM
--
-- 5724-C51
--
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_uploadgetrodmobjects.sql
-- SID: 1.16
-- Date Modified: 04/12/06 17:41:19
-- This stored procedure is called from the SendMVSRODMRegisterObjects shell script.  The script
-- will find generate the list of all objects that need to be registered, unregistered, or
-- re-registered, and update the RODMManagedObject table with the appropriate registration status.
CREATE PROCEDURE asisp_uploadGetRODMObjects
  @cid                ClassID,
  @id                 ObjID,
  @RegistrationStatus TINYINT,  /* 0=unregister, 1=register, 2=re-register */
  @RegistrationScope  TINYINT,  /* 1=root obj only, 2=only descendents, 3=root AND descendents */
  @ResponseType       TINYINT,  /* 0=OFF (never), 1=EX (exception only), 2=DEF (always) */
  @verbose            TINYINT = 0
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET ANSI_DEFAULTS OFF
  SET NOCOUNT ON

  CREATE TABLE #descendent -- Table to hold the objects we need to register
  (
    cid   ClassID  NOT NULL,
    id    ObjID    NOT NULL,
    gen   INT      NOT NULL
  )

  DECLARE
    @snnt_id          ObjID,
    @gmff_id          ObjID,
    @msg_txt          NVARCHAR(255),
    @now              NVARCHAR(49),
    @ResponseTypeStr  NVARCHAR(9)

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_uploadGetRODMObjects: @cid=''%s'', @id=%d, @RegistrationStatus=%d, @RegistrationScope=%d, @ResponseType=%d',
              0, 1, @now, @cid, @id, @RegistrationStatus, @RegistrationScope, @ResponseType) WITH NOWAIT
  END
  
  IF @RegistrationScope <> 1 -- include descendent objects
  BEGIN
    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
    IF @cid = 'OS'
    BEGIN
      -- If the calling object is an operating system then we want to perform the registration
      -- operation on all SNANetworks that are monitored by this OS. We iterate through all
      -- these networks and build up the table of descendent objects so we can do one monster
      -- registration.
      DECLARE osc INSENSITIVE CURSOR FOR SELECT id FROM SNANetwork_V WHERE _MonitoringOSID = @id
      OPEN osc
      FETCH NEXT FROM osc INTO @snnt_id
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        EXEC asisp_descendents_nocache 'SNNT', @snnt_id, 'IN (SELECT cid FROM isa_chain WITH (NOLOCK) WHERE base_cid = ''RDMO'')', @verbose = @verbose -- find RODMStatesObjects
        FETCH NEXT FROM osc INTO @snnt_id
      END
      DEALLOCATE osc
      -- Retrieve the GMFHSAggregate objects for this OS
      SELECT @gmff_id = id from GMFHSAggregateFolder_V where deleted = 0
      IF @gmff_id IS NOT NULL
         EXEC asisp_descendents_nocache 'GMFF', @gmff_id, 'IN (SELECT cid FROM isa_chain WITH (NOLOCK) WHERE base_cid = ''RDMO'')', @verbose = @verbose -- find RODMStatesObjects
    END
    ELSE
    BEGIN
      -- The calling object is expected to have a direct parent-child chain to RODMManagedObjects.
      EXEC asisp_descendents_nocache @cid, @id, 'IN (SELECT cid FROM isa_chain WITH (NOLOCK) WHERE base_cid = ''RDMO'')', @verbose = @verbose -- find RODMManagedObjects
    END
    SET TRANSACTION ISOLATION LEVEL READ COMMITTED
  END
  
  -- include the base object (unless it's an OS)
  IF @RegistrationScope <> 2 AND @cid <> 'OS'
    INSERT INTO #descendent VALUES ( @cid, @id, 0 )

  -- exclude the base object if registration scope says "children only"
  IF @RegistrationScope = 2
    DELETE FROM #descendent WHERE cid = @cid AND id = @id

  -- Filter out objects by registration status. If we're asked to reregister objects then send
  -- registration statements only for objects that have _RegistrationStatus=1. This will avoid
  -- floods of unregister statements.
  IF @RegistrationStatus = 2
    DELETE d FROM #descendent AS d
    JOIN RODMManagedObject AS rmo ON d.cid = rmo.cid AND d.id = rmo.id
    WHERE rmo._RegistrationStatus = 0

  -- Make any necessary change to RegistrationStatus
  IF @RegistrationStatus <> 2
    UPDATE rmo SET _RegistrationStatus = @RegistrationStatus
    FROM RODMManagedObject AS rmo
    JOIN #descendent d ON rmo.cid = d.cid AND rmo.id = d.id

  -- Remove GMFA that are marked deleted = 1
  DELETE d FROM #descendent AS d
    JOIN RODMManagedObject AS rmo ON d.cid = rmo.cid AND d.id = rmo.id
    WHERE rmo.cid = 'GMFA' 
      AND rmo.deleted = 1

  -- set LastMonitoringOSID for GMFA objects getting registered

  UPDATE rmo SET _LastMonitoringOSID = gmff._MonitoringOSID
    FROM RODMManagedObject      AS rmo
    JOIN #descendent            AS d    ON rmo.cid = d.cid AND rmo.id = d.id
    JOIN GMFHSAggregateFolder_V AS gmff ON gmff.deleted = 0
    WHERE rmo.cid = 'GMFA' AND rmo._RegistrationStatus = 1

  -- If this is a re-register we don't want to send up unregister statements
  CREATE TABLE #monitored_fields
  (
    cid         ClassID      NOT NULL,
    field_name  NVARCHAR(40) NOT NULL
  )
  INSERT INTO #monitored_fields VALUES ( 'RDSO', 'states' )
  INSERT INTO #monitored_fields VALUES ( 'RDMO', 'DisplayStatus' )

  SELECT @ResponseTypeStr = v.name FROM enum_value AS v WITH (NOLOCK) JOIN enumeration AS e WITH (NOLOCK) ON e.eid = v.eid
  WHERE e.name ='RODMResponseType' AND v.value = @ResponseType
  IF @ResponseTypeStr IS NULL
  BEGIN
    EXEC Object..asisp_getTBSMMessage @cid,@id,112,'RO','English','asisp_uploadGetRODMObjects',@msg_txt OUTPUT
    SELECT @msg_txt
    IF @msg_txt IS NOT NULL 
       RAISERROR(@msg_txt,0,1)
    ELSE
       RAISERROR('asisp_uploadGetRODMObjects: Invalid response type: %d', 0, 1, @ResponseType)
    RETURN -30000
  END

  -- We want override the response type of any object that previously failed in a registration
  -- so that we will get a definite response for the object.  When a registration fails, we receive
  -- a \05\03 message from the host and we will create a 'RODMREGISTER FAILED' exception for the
  -- object.  If registration has previously failed, we will request a definite response in a
  -- subsequent registration request (regardless of the response type specified).  If the subsequent
  -- request is successful, we will get a successful response and will clear the 'RODMREGISTER FAILED'
  -- exception.
  --
  -- The first step to overriding the response type will be to see what objects (if any) have an
  -- exception indicating that registration failed:

  CREATE TABLE #failed_regs
  (
    cid   ClassID NOT NULL,
    id    ObjID   NOT NULL,
    _count INT    NOT NULL
  )

  INSERT INTO #failed_regs
    SELECT o.cid, o.id, CASE WHEN excp.id IS NULL THEN 0 ELSE COUNT(*) END
      FROM #descendent AS o
      LEFT OUTER JOIN Exception_V AS excp ON excp._ParentCID = o.cid AND excp._ParentID = o.id AND 
                                             excp.deleted = 0 AND excp.name = 'RODMREGISTER FAILED'
    GROUP BY o.cid, o.id, excp.id

  -- Select results in upload format
  SELECT DISTINCT CAST(os.id AS NVARCHAR(9)) + '|'
       + '05\02' -- RODM INTERFACE\OBJECT REGISTRATION
       + '\73' + map.RODMClass -- PARM_1
       + '\74' + m._RODMMyName -- PARM_2
       + '\47' + CASE m._RegistrationStatus WHEN 1 THEN 'ON' ELSE 'OFF' END -- SWITCH
       + '\84id=' + CAST(m.id AS NVARCHAR(9)) + -- CORRELATION_ID - object id & ...
         ' action=' + CASE m._RegistrationStatus WHEN 1 THEN 'register' ELSE 'unregister' END + -- ... registration action
       + '\85' + CASE
                   WHEN fr._count = 0 THEN @ResponseTypeStr
                   ELSE 'DEF'
                 END
       + '\75' + mf.field_name -- PARM_3
       + '~'
       AS DATA_ROW
  FROM
    RODMManagedObject AS m
      JOIN #descendent AS d ON d.cid = m.cid AND d.id = m.id
      JOIN #failed_regs AS fr ON fr.cid = d.cid AND fr.id = d.id
      JOIN SNANetwork_V AS net ON m._NetworkID LIKE '%' + net.name + '%'
      JOIN OS_V AS os ON os.id = net._MonitoringOSID
      JOIN RODMClassMap AS map WITH (NOLOCK) ON map.cid = m.cid
      JOIN isa_chain AS ic WITH (NOLOCK) ON ic.cid = m.cid
      JOIN #monitored_fields AS mf ON mf.cid = ic.base_cid
  WHERE
    os._UploadEnabled = 1
    AND m.cid <> 'GMFA'
  UNION
  SELECT DISTINCT CAST(os.id AS NVARCHAR(9)) + '|'
       + '05\02' -- RODM INTERFACE\OBJECT REGISTRATION
       + '\73' + map.RODMClass -- PARM_1
       + '\74' + m._RODMMyName -- PARM_2
       + '\47' + CASE m._RegistrationStatus WHEN 1 THEN 'ON' ELSE 'OFF' END -- SWITCH
       + '\84id=' + CAST(m.id AS NVARCHAR(9)) + -- CORRELATION_ID - object id & ...
         ' action=' + CASE m._RegistrationStatus WHEN 1 THEN 'register' ELSE 'unregister' END + -- ... registration action
       + '\85' + CASE
                   WHEN fr._count = 0 THEN @ResponseTypeStr
                   ELSE 'DEF'
                 END
       + '\75' + mf.field_name -- PARM_3
       + '~'
       AS DATA_ROW
  FROM
    RODMManagedObject AS m
      JOIN #descendent AS d ON d.cid = m.cid AND d.id = m.id
      JOIN #failed_regs AS fr ON fr.cid = d.cid AND fr.id = d.id
      JOIN link AS l ON l.src_cid = 'GMFF' AND l.dst_cid = m.cid AND l.dst_id = m.id
      JOIN GMFHSAggregateFolder_V AS gfolder ON gfolder.id = l.src_id
      JOIN OS_V AS os ON os.id = gfolder._MonitoringOSID
      JOIN RODMClassMap AS map WITH (NOLOCK) ON map.cid = m.cid
      JOIN isa_chain AS ic WITH (NOLOCK) ON ic.cid = m.cid
      JOIN #monitored_fields AS mf ON mf.cid = ic.base_cid
  WHERE
    os._UploadEnabled = 1
    AND m.cid = 'GMFA'
    AND l.deleted = 0

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_uploadGetRODMObjects: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
