-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/at_create.sql
-- SID: 1.15.1.14
-- Date Modified: 04/11/11 15:24:16
--
-- Module Description
-- Called to send a status updaet to TEC for events that originated from TEC

USE Object
GO

---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure AT_Create'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'AT_Create' AND type = 'P')
  DROP PROCEDURE AT_Create
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/at_create.sql
-- SID: 1.15.1.14
-- Date Modified: 04/11/11 15:24:16
--
-- Module Description:
--   This procedure is responsible for the creation of a problem ticket, Ownership or Problem note and setting the Problem flag
--   to display the Problem icon. It will also add a row to the AT_Created table to indicate a problem was created.
--   Input will be the PendingAutoTicketFilteredEvent table
--   It uses the _OpenIntegrationRequest, _AddRequestField and _CommitIntegrationRequest to create the <GUID>.in file and invoke
--   the appropriate Request Processor, contained in the SystemConfiguration table, which will create the Problem ticket in the
--   target Problem product.
--   Note: The Request Processor is created by the Tivoli Services organization in conjunction with the customer.

-- Input Parameters:
--   @cid      - The class id passed from a wrapper proc
--   @id       - The object id passed from a wrapper proc
--   @eventno   - The id of Event from the PendingAutoTicketFilteredEvent table 
--   @verbose  - The flag to activate more detailed information concerning the processing within this proc
--
-- Output:
--   @err       - Zero or negative value to indicate success or failure
CREATE PROCEDURE AT_Create
   @cid     NVARCHAR(255),
   @id      INT,
   @eventno INT,
   @verbose INT = 0
AS
  DECLARE @now NVARCHAR(40)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create:, @cid=%s, @id=%d, @eventno=%d, @verbose=%d', 0, 1, @now, @cid, @id, @eventno, @verbose) WITH NOWAIT
  END
  
-- PendingAutoTicketFilteredEvent table variables
  DECLARE @ctime DATETIME
  DECLARE @obj_cid ClassID
  DECLARE @obj_id ObjID
  DECLARE @obj_name ObjName
  DECLARE @event_cid  ClassID
  DECLARE @event_id ObjID
  DECLARE @event_name ObjName
  DECLARE @event_ctime DATETIME
  DECLARE @event_detail NVARCHAR(255)
  DECLARE @event_alertstate INT
  DECLARE @event_priority INT
  DECLARE @event_state INT
  DECLARE @rule_id INT

--  AT_Rule table variables
  DECLARE @note_type NCHAR(16)
  DECLARE @note_expiration DATETIME
  DECLARE @note_owner NVARCHAR(255)
  DECLARE @note_subject NVARCHAR(255)
  DECLARE @note_body NVARCHAR (255)

-- AT_Rule_Detail table variables
  DECLARE @name NVARCHAR(255)
  DECLARE @value NVARCHAR(255)

-- _OpenIntegrationRequest proc parameters
  DECLARE @request_type NVARCHAR (30)
  DECLARE @request_method NVARCHAR(30)
  DECLARE @request_guid NVARCHAR(255)

-- _AddRequestField proc parameters
  DECLARE @request_field_name NVARCHAR (120)
  DECLARE @request_field_value NVARCHAR(2000)
  DECLARE @request_error_no INT

-- Miscellaneous
  DECLARE @ProblemID    NVARCHAR (255)
  DECLARE @noteid       NVARCHAR (255)
  DECLARE @err          INT
  DECLARE @rc           INT
  DECLARE @rp_rc        INT
  DECLARE @length       INT
  DECLARE @event_parent INT

  DECLARE @event_owned      NVARCHAR(3) 
  DECLARE @event_problem_id NVARCHAR(255) 
  DECLARE @event_owner      NVARCHAR(255) 
  DECLARE @event_active     NVARCHAR(3) 
  DECLARE @return_code      INT

  SET NOCOUNT ON
  IF @verbose <> 1
    SET @verbose = 0

--  Standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  IF @tcount = 0
    BEGIN TRANSACTION AT_Create
  ELSE
    SAVE TRANSACTION AT_Create

--
--  Standard transaction logic starts here
--
--  query PendingAutoTicketFilteredEvent table to obtain event data and auto ticket rule id
  SELECT
    @ctime = ctime, @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name,
    @event_cid = event_cid, @event_id = event_id, @event_name = event_name,
    @event_ctime = event_ctime, @event_detail = event_detail, @event_alertstate = event_alertstate,
    @event_priority = event_priority, @event_state = event_state, @rule_id = rule_id
  FROM
    PendingAutoTicketFilteredEvent
  WHERE
    @eventno = eventno
  SET @rc = @@ROWCOUNT
  IF @rc = 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Could not find event in PendingAutoTicketFilteredEvent table - Note/Ticket note created, eventno=%d, obj_cid=%s, obj_id=%d',
      11, 1, @now, @eventno, @obj_cid, @obj_id) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
    RETURN -30000
  END

  -- Attempt to lookup resource name is it wasn't populated in the event. Make sure the resource
  -- class exist before calling asisp_getobjname. It can't tolerate non-existent classes.
  IF EXISTS(SELECT * FROM obj_class WHERE LTRIM(RTRIM(cid)) = LTRIM(RTRIM(@obj_cid)))
  BEGIN
    IF @obj_name = ''
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: Attempting to resolve resource name. It was empty in the event.', 0, 1, @now) WITH NOWAIT
      END
      EXEC asisp_getobjname @obj_cid, @obj_id, @obj_name OUTPUT
      SET @obj_name = ISNULL(@obj_name,'N/A')
    END
  END

  -- If resource name is still empty at this point, then set it to default value.
  IF @obj_name = ''
    SET @obj_name = 'N/A'

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: query AT_Rule table to obtain note information', 0, 1, @now) WITH NOWAIT
  END
  SELECT
    @note_type = note_type, @note_expiration = note_expiration, @note_owner = note_owner,
    @note_subject = note_subject, @note_body = note_body
  FROM
    AT_Rule
  WHERE
    @rule_id = rule_id
  SET @rc = @@ROWCOUNT
  IF @rc = 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Could not find rule_id in AT_Rule table Note/Ticket not created , rule_id=%d, eventno=%d',
      11, 1, @now, @rule_id, @eventno) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
    RETURN -30000
  END

  IF @note_type = 'O'
    SET @note_type = 'Ownership'
  ELSE IF @note_type = 'P'
    SET @note_type = 'Problem'
  ELSE
    SET @note_type = 'None'

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: CREATE and COMMIT OWNERSHIP OR PROBLEM NOTE BASED ON Note Type', 0, 1, @now) WITH NOWAIT
  END

-- ****  had to convert event alert state to char to match parameter of called procedures
  SET @event_alertstate = CONVERT(NVARCHAR(16),@event_alertstate)

  IF @note_type = 'Ownership'
  BEGIN
    -- Must get _ParentID of exception history record ad pass that id to _TakeOwnership
    IF @event_cid = 'EXCP'
    BEGIN
      SELECT @event_parent = _ParentID FROM ExceptionHistory_V WHERE id = @event_id
      IF @event_parent IS NULL
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: AT_Create failed - Could not get parent ID for ExceptionHistory id=%d.', 11, 1, @now, @event_id) WITH NOWAIT
        ROLLBACK TRANSACTION AT_Create
        RETURN -30000
      END
    END
    ELSE
  SET @event_parent = @event_id

    -- Get the REQUEST_CLEAR_EVENTS option from the SystemConfiguration table, which will 
    -- tell us to either clear the events once the note is committed or not - SR (APAR OA03810)
    DECLARE @clearevents NVARCHAR(4), @clear_int INT
    SELECT @clearevents = propertyvalue FROM SystemConfiguration WITH (NOLOCK)
    WHERE 
      groupname = 'REQUEST_AUTOTICKET'
      AND propertyname = 'REQUEST_CLEAR_EVENTS'
      AND propertydatatype = 'INT'

    IF (@clearevents IS NULL OR (@clearevents <> '0' AND @clearevents <> '1'))
      SET @clearevents = '1'

    SET @clear_int = CAST(@clearevents AS INT)

    -- The events to take ownership of are passed to _CreateOwnershipNote through the following table
    CREATE TABLE #Events
    (
      cid ClassID,
      id  ObjID
    )
    INSERT INTO #Events VALUES (@event_cid , @event_parent) 

    DECLARE @t_noteid INT -- the note id is also returned as an output parm

    EXEC @noteid = _CreateOwnershipNote @obj_cid, @obj_id, @note_owner, 
                        @note_subject, @note_body, '', @t_noteid, @clear_int
    IF @noteid <= 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: _CreateOwnershipNote failed - Ownership Note/Ticket not created, note_id=%d, obj_cid=%s, obj_id=%d, event_cid=%s, event_id=%d,@eventno',
          11, 1, @now, @noteid, @obj_cid, @obj_id, @event_cid, @event_id, @eventno) WITH NOWAIT
      ROLLBACK TRANSACTION AT_Create
      RETURN @noteid
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: Create Ownership Note was successful ', 0, 1, @now) WITH NOWAIT
    END
  END
  ELSE IF @note_type = 'Problem'
  BEGIN
    -- Added a blank value for _AlertState parameter after the note status value. Without this parameter the note subject value
    -- was being passed which causes the column contraint to fail.
    EXEC @noteid = asisp_createNOTE @obj_name, @obj_cid, @obj_id, '', '', 'TBSM','Problem', 'Open','',@note_subject, @note_body, '', @obj_id, @obj_cid
    IF @noteid < 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: asisp_createNOTE failed - Problem Note/Ticket not created, note_id=%d, obj_cid=%s, obj_id=%d, event_cid=%s, event_id=%d, @eventno',
        11, 1, @now, @noteid, @obj_cid, @obj_id, @event_cid, @event_id, @eventno) WITH NOWAIT
      ROLLBACK TRANSACTION AT_Create
      RETURN @noteid
    END

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: _asisp_createNOTE - Problem Note ID = %d', 0, 1, @now, @noteid) WITH NOWAIT
    END
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Note process was successful proceed with Problem Ticket creation.', 0, 1, @now) WITH NOWAIT
  END
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: call _OpenIntegrationRequest to add request and obtain a GUID value', 0, 1, @now) WITH NOWAIT
  END
  SET @request_type = 'REQUEST_AUTOTICKET'
  SET @request_method = 'REQUEST_ADD'

  EXEC _OpenIntegrationRequest @obj_cid, @obj_id, @request_type, @request_method, @request_guid OUT
  IF @request_guid = ''
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: _OpenIntegrationRequest did not return a GUID - Note/Ticket not created, obj_cid=%s, obj_id=%d',
      11, 1, @now, @obj_cid, @obj_id) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Request GUID = %s', 0, 1, @now, @request_guid) WITH NOWAIT
  END
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: call _AddRequestField to add name/value pairs', 0, 1, @now) WITH NOWAIT
  END
  
   -- ************************
  -- Add event status information to integration request
    -- Initialize output variable values.
    SET @event_owned = ''
    SET @event_problem_id = ''
    SET @event_owner = ''
    SET @event_active = ''
    EXEC @return_code = AT_GetEventStatus @obj_cid, @obj_id, @event_cid, @event_id,  
         @event_owned OUTPUT,@event_problem_id OUTPUT,@event_owner OUTPUT, @event_active OUTPUT
    
    -- Add all of the data about the request to a temp statging table which will be used 
    -- later to bulk insert into the IntegrationFields table for the request.
    CREATE TABLE #int_fields(
      field_name   NVARCHAR(240),
      field_value  NVARCHAR(2000) NULL
    )
  
    -- **************************************************************************************************
    -- For enhancement request MR1119023325 a call to AT_addExceptionInfo was added to insert the 
    -- exception propagation matrix and exception occurrance count into the <guid>.in file.
    -- **************************************************************************************************
    -- If the event that was passed was a Exception, add exception matrix information to the input files.
    -- **************************************************************************************************
    IF @event_cid = 'EXCP'
    BEGIN

      SELECT @event_parent = _ParentID FROM ExceptionHistory_V WHERE id = @event_id

      IF @event_parent IS NULL
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_addExceptionInfo failed - Could not get parent ID for ExceptionHistory id=%d.', 0,1, @now, @event_id)
        ROLLBACK TRANSACTION AT_Create
        RETURN -3000
      END
    
      -- Add extended Exception information.
      EXEC AT_addExceptionInfo @event_parent,@obj_cid,@obj_id,@request_error_no OUT

    END
  
    INSERT INTO #int_fields VALUES('REQUEST_AUTO_CREATE','YES')
    INSERT INTO #int_fields VALUES('REQUEST_OBJECT_NAME',@obj_name)
    INSERT INTO #int_fields VALUES('REQUEST_USER_ID','')
    INSERT INTO #int_fields VALUES('REQUEST_PASSWORD','')
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_CID',@event_cid)
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_ID',@event_id)
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_DETAIL',@event_detail)
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_NAME',@event_name)
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_STATE',@event_state)
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_TIME',@event_ctime) 
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_PRIORITY',@event_priority) 
    INSERT INTO #int_fields VALUES('REQUEST_EVENT_ALERT_STATE',@event_alertstate)
    
    INSERT INTO #int_fields VALUES('EVENT_OWNED',@event_owned) 
    INSERT INTO #int_fields VALUES('EVENT_PROBLEM_ID',@event_problem_id) 
    INSERT INTO #int_fields VALUES('EVENT_OWNER',@event_owner) 
    INSERT INTO #int_fields VALUES('EVENT_ACTIVE',@event_active) 

    INSERT INTO IntegrationFields  (request_guid, field_name, field_value)
         SELECT @request_guid, field_name, field_value from #int_fields
  
    IF @@ERROR <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _AddRequestField insert failed - Note/Ticket not created, obj_cid=%s, obj_id=%d, request_field_name=%s',
          0,1, @now, @obj_cid, @obj_id, @request_field_name)
      ROLLBACK TRANSACTION AT_Create
      RETURN -30000
    END
  
  -- ************************
  
  --
  -- call _CommitIntegrationRequest to create the <GUID>.in file and call Request Processor to create Problem Ticket
  -- in the target Problem product
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Call _CommitIntegrationRequest', 0, 1, @now) WITH NOWAIT
  END
  CREATE TABLE #atcreateout
  (
    outputdata NVARCHAR(4000)
  )

  EXEC _CommitIntegrationRequest @obj_cid, @obj_id, @request_guid, @request_error_no OUT, 1


  IF @request_error_no <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: _CommitIntegrationRequest failed - Note/Ticket not created, obj_cid=%s, obj_id=%d, request_guid=%s',
      11, 1, @now, @obj_cid, @obj_id, @request_guid) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: **** read results set to obtain id of Problem Ticket just created', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @output NVARCHAR(4000)
  DECLARE @rp_rc_char NVARCHAR(255)

  IF @verbose >= 2
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Contents of <GUID>.out file.', 0, 1, @now) WITH NOWAIT
    SELECT * FROM #atcreateout
  END

  SELECT @output = outputdata FROM #atcreateout WHERE outputdata like 'REQUEST_RETURN_CODE%'

  -- Must anticipate that the value of the REQUEST_RETURN_CODE may be a GTMPE9999? type return number.
  -- These numbers indicate that the request failed and are handled as if a non-zero REQUEST_RETURN_CODE was returned.
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: First line of request output file := %s', 0, 1, @now, @output) WITH NOWAIT
  END

  IF CHARINDEX('|GTM',@output,1) <> 0
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: Request Processor return code was a GTM type. Setting @rp_rc = 9999.', 0, 1, @now) WITH NOWAIT
    END
    SET @rp_rc = 9999
  END
  ELSE
  BEGIN
    -- If the REQUEST_RETURN_CODE is not a GTM type, make sure it is numeric before assigning it to 
    -- the @rp_rc integer variable.
    SET @rp_rc_char = LTRIM(RTRIM(SUBSTRING(@output,25,12)))
    IF ISNUMERIC(LTRIM(RTRIM(@rp_rc_char))) = 0 
    BEGIN
      SET @rp_rc = 9999
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: Request Processor return code was not numeric. Setting @rp_rc = 9999.', 0, 1, @now) WITH NOWAIT
      END
    END
    ELSE
      SET @rp_rc = LTRIM(RTRIM(SUBSTRING(@output,25,12)))
  END
 
  IF @rp_rc <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Request Processor return code was not 0. Request Processor failed - Note/Ticket not created, obj_cid=%s, obj_id=%d, request_guid=%s, rp_rc=%d',
      11, 1, @now, @obj_cid, @obj_id, @request_guid, @rp_rc) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create:  get Problem ID returned from Request Processor and update Note if rc is zero', 0, 1, @now) WITH NOWAIT
  END
  IF @rp_rc = 0
  BEGIN
    SET @output = ''
    SELECT @output = outputdata FROM #atcreateout WHERE outputdata LIKE 'REQUEST_PROBLEM_ID%'
    IF @output <> ''
    BEGIN
      SET @output = SUBSTRING(@output,24,60)
      SET @length = LEN(@output)
      SET @ProblemID = SUBSTRING(@output,1,@length)

      -- The request processor has signaled that the Note should not be created.
      IF LTRIM(RTRIM(@ProblemID)) = 'NO_NOTE'
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: Request processor has signaled that a Note should not be created., obj_cid=%s, obj_id=%d, note_id=%d',
          11, 1, @now, @obj_cid, @obj_id, @noteid) WITH NOWAIT
        ROLLBACK TRANSACTION AT_Create
        RETURN 0
      END

 -- Check to make sure the event is still ownable before completing the Note updates.
      -- Add event status information to integration request
      -- Initialize output variable values.
      SET @event_owned = ''
      SET @event_problem_id = ''
      SET @event_owner = ''
      SET @event_active = ''
      EXEC @return_code = AT_GetEventStatus @obj_cid, @obj_id, @event_cid, @event_id, 
           @event_owned OUTPUT,@event_problem_id OUTPUT,@event_owner OUTPUT, @event_active OUTPUT

      IF @return_code <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: Call to AT_GetEventStatus failed with return code = %d.', 11, 1, @now, @return_code) WITH NOWAIT
        ROLLBACK TRANSACTION AT_Create
        RETURN -30000
      END

      -- Event was owned while the integration request was being processed so don't create the Note.
      IF @event_owned = 'YES'
      BEGIN
         
        UPDATE IntegrationRequest SET status = 112 WHERE request_guid = @request_guid
        SET @err = @@ERROR
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_Create: Update of status column in IntegrationRequest table failed with error=%d. Note not created.', 11, 1, @now, @err) WITH NOWAIT
          ROLLBACK TRANSACTION AT_Create
          RETURN -30000
        END
      END

      -- Event became inactive while the integration request was being processed so don't create the Note.
      IF @event_owned = 'NO' AND @event_active = 'NO'
      BEGIN
        UPDATE IntegrationRequest SET status = 113 WHERE request_guid = @request_guid
        SET @err = @@ERROR
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_Create: Update of status column in IntegrationRequest table failed with error=%d. Note not created.', 11, 1, @now, @err) WITH NOWAIT
          ROLLBACK TRANSACTION AT_Create
          RETURN -30000
        END
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_Create: Update the Note with the Problem ID returned from the target Problem product.', 0, 1, @now) WITH NOWAIT
      END
      IF @note_type <> 'None'
      BEGIN
        UPDATE Note_A SET _ProblemID = @ProblemID WHERE id = @noteid
        SET @err = @@ERROR
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_Create: Update Note with Problem ID failed - Note/Ticket not created, obj_cid=%s, obj_id=%d, note_id=%d',
            11, 1, @now, @obj_cid, @obj_id, @noteid) WITH NOWAIT
          ROLLBACK TRANSACTION AT_Create
          RETURN -30000
        END
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_Create: If the Problem Ticket create was successful set problem flag to display problem icon on object', 0, 1, @now) WITH NOWAIT
        END
        EXEC @err = _SetMNO2Flag @obj_cid, @obj_id, 'Problem', 1
      END
    END
  END

  -- Clean up temp table.
  DROP TABLE #atcreateout

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: If a Problem note is being create update the note subject and body from the data in the AT_Rule table.', 0, 1, @now) WITH NOWAIT
  END

  IF @note_type = 'Problem'
  BEGIN
    UPDATE Note_A SET _Subject = @note_subject, _Body = @note_body WHERE id = @noteid
    SET @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: Update Note with subject, body failed - Problem Note/Ticket not created, obj_cid=%s, obj_id=%d, note_id=%d',
        11, 1, @now, @obj_cid, @obj_id, @noteid) WITH NOWAIT
      ROLLBACK TRANSACTION AT_Create
      RETURN -30000
    END
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Insert processed event into AT_Created table', 0, 1, @now) WITH NOWAIT
  END

  -- If there is already a row in the AT_Created table for the resource and the event, just update the event_ctime
  -- to reflect the last time a ticket was created for the resource and event.
  IF EXISTS( SELECT * FROM AT_Created WHERE 
             (LTRIM(RTRIM(obj_cid)) = LTRIM(RTRIM(@obj_cid))) AND 
             (obj_id = @obj_id) AND
             (LTRIM(RTRIM(event_name)) = LTRIM(RTRIM(@event_name))) AND 
             (event_alertstate = @event_alertstate) AND
             (event_priority = @event_priority) AND 
             ((event_state = @event_state) OR ((event_state IS NULL) AND (@event_state IS NULL))))
  BEGIN

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: ++++ Update existing row with a new event time. ++++', 0, 1, @now) WITH NOWAIT
    END

    UPDATE AT_Created SET event_ctime = @event_ctime, event_id = @event_id 
    WHERE (LTRIM(RTRIM(obj_cid)) = LTRIM(RTRIM(@obj_cid))) AND 
          (obj_id = @obj_id) AND
          (LTRIM(RTRIM(event_name)) = LTRIM(RTRIM(@event_name))) AND 
          (event_alertstate = @event_alertstate) AND
          (event_priority = @event_priority) AND 
          ((event_state = @event_state) OR ((event_state IS NULL) AND (@event_state IS NULL)))

  END

  ELSE

  BEGIN

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: ++++ Inserting first time AT_Created row. ++++', 0, 1, @now) WITH NOWAIT
    END

    -- Insert new row for the resource and event.
    INSERT INTO AT_Created (ctime, obj_cid, obj_id, obj_name, event_cid, event_id, event_name, event_ctime, event_detail,
      event_alertstate, event_priority, event_state, deleted)
    VALUES ( CURRENT_TIMESTAMP, @obj_cid, @obj_id, @obj_name, @event_cid, @event_id, @event_name, @event_ctime, @event_detail, @event_alertstate,
      @event_priority, @event_state, 0 )
    SET @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: Could not insert AT_Created for filtered event', 0, 1, @now) WITH NOWAIT
    END
  END

--
-- Standard transaction logic ends here
--
-- Finish up transaction

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: Ending transaction', 0, 1, @now) WITH NOWAIT
  END

  IF (@err <> 0)
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_Create: ERROR WAS ENCOUNTERED ROLLBACK TRANSACTION', 0, 1, @now) WITH NOWAIT
    ROLLBACK TRANSACTION AT_Create
  END
  ELSE IF @tcount = 0
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_Create: SUCCESSFUL PROCESS COMMIT TRANSACTION', 0, 1, @now) WITH NOWAIT
    END    
    COMMIT TRANSACTION AT_Create
  END