-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/at_createinputfiles.sql
-- SID:  1.8 
-- DATE Modified:  04/11/11 15:24:17
--
-- Module Description:
--   This procedure is responsible for the creation of a <GUID>.in file for each row in the 
--   referenced temp table #PendingAutoTicketFilteredEvent.
--   The temp table is created in the AT_CreateInputFiles stored procedure. If DIRECT_REQUEST_READ is enabled
--   in the system configuration file, the pending request are created in the IntegrationRequest table with a 
--   status value of 100 but the <GUID>.in file is not created. 
-- Input Parameters:
--   @cid      - The class id passed from a wrapper proc
--   @id       - The object id passed from a wrapper proc
--   @verbose  - The flag to activate more detailed information concerning the processing within this proc
--
-- Returns:
--   @err       - Zero or negative value to indicate success or failure.
USE Object
GO

-------------------------------------------------------------------------------------------------------
PRINT 'Creating procedure AT_CreateInputFiles'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'AT_CreateInputFiles' AND type = 'P')
  DROP PROCEDURE AT_CreateInputFiles
GO

-- File: archive/db/sql/at_createinputfiles.sql
-- SID:  1.8 
-- DATE Modified:  04/11/11 15:24:17
--
-- Module Description:
--   This procedure is responsible for the creation of a <GUID>.in file for each row in the 
--   referenced temp table #PendingAutoTicketFilteredEvent.
--   The temp table is created in the AT_CreateInputFiles stored procedure. If DIRECT_REQUEST_READ is enabled
--   in the system configuration file, the pending request are created in the IntegrationRequest table with a 
--   status value of 100 but the <GUID>.in file is not created. 
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- Input Parameters:
--   @cid      - The class id passed from a wrapper proc
--   @id       - The object id passed from a wrapper proc
--   @verbose  - The flag to activate more detailed information concerning the processing within this proc
--
-- Returns:
--   @err       - Zero or negative value to indicate success or failure.
CREATE PROCEDURE AT_CreateInputFiles
   @cid     VARCHAR(255),
   @id      INT,
   @verbose INT = 0
AS

  -- PendingAutoTicketFilteredEvent table variables
  DECLARE @ctime DATETIME
  DECLARE @obj_cid ClassID
  DECLARE @obj_id ObjID
  DECLARE @obj_name ObjName
  DECLARE @event_cid  ClassID
  DECLARE @event_id ObjID
  DECLARE @event_name ObjName
  DECLARE @event_ctime DATETIME
  DECLARE @event_detail NVARCHAR(255)
  DECLARE @event_alertstate INT
  DECLARE @event_priority INT
  DECLARE @event_state INT
  DECLARE @rule_id INT

--  AT_Rule table variables
  DECLARE @note_type NCHAR(16)
  DECLARE @note_expiration DATETIME
  DECLARE @note_owner NVARCHAR(60)
  DECLARE @note_subject NVARCHAR(255)
  DECLARE @note_body NVARCHAR (255)

-- AT_Rule_Detail table variables
  DECLARE @name NVARCHAR(255)
  DECLARE @value NVARCHAR(255)

-- _OpenIntegrationRequest proc parameters
  DECLARE @request_type NVARCHAR (30)
  DECLARE @request_method NVARCHAR(30)
  DECLARE @request_guid NVARCHAR(255)

-- _AddRequestField proc parameters
  DECLARE @request_field_name NVARCHAR (120)
  DECLARE @request_field_value NVARCHAR(2000)
  DECLARE @request_error_no INT

-- Miscellaneous
  DECLARE @ProblemID NVARCHAR (255)
  DECLARE @noteid NVARCHAR (255)
  DECLARE @err INT
  DECLARE @rp_rc INT
  DECLARE @length INT
  DECLARE @event_parent INT

  DECLARE @duplicate_age INT
  DECLARE @duplicate_age_char NVARCHAR(255)
  DECLARE @direct_request_read NVARCHAR(255)

-- Declares form _CommitIntegrationRequest
  DECLARE @phy_cid ClassID
  DECLARE @phy_id ObjID
 
  DECLARE @TIME_OUT_CUSTOM_MSG NVARCHAR(4000)

  DECLARE @OAerr_output NVARCHAR(255)
  DECLARE @OAerr_source NVARCHAR(255)
  DECLARE @OAerr_description NVARCHAR(255)
  DECLARE @OAreturn_code INT
  DECLARE @OIntegrationServer AS INT

  DECLARE @debug_mode AS NVARCHAR(3)
  DECLARE @timeout_value_string AS NVARCHAR(255)
  DECLARE @timeout_value AS INT
  DECLARE @request_field_row_count INT
  DECLARE @request_processor_language NVARCHAR(255)
  DECLARE @field_count INT
  DECLARE @current_field_name NVARCHAR(255)
  DECLARE @return_code INT
  DECLARE @request_command_processor AS NVARCHAR(255)
  DECLARE @err_description NVARCHAR(255)

  DECLARE @userid NVARCHAR(255)
  DECLARE @usergroup NVARCHAR(255)
  DECLARE @Verno NVARCHAR(5)

  DECLARE @eventno INT

  DECLARE @event_owned      NVARCHAR(3) 
  DECLARE @event_problem_id NVARCHAR(255) 
  DECLARE @event_owner      NVARCHAR(255) 
  DECLARE @event_active     NVARCHAR(3) 

  DECLARE @now NVARCHAR(40)

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Called with @cid=%s, @id=%d, @verbose=%d', 0, 1, @now, @cid, @id, @verbose ) WITH NOWAIT
  END

  SET NOCOUNT ON

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT

  IF @tcount = 0
    BEGIN TRANSACTION AT_CreateInputFiles
  ELSE
    SAVE TRANSACTION AT_CreateInputFiles

  -- Obtain userid/usergroup to determine if 1.5 console is being used with higher release database
  EXEC asisp_getContextUserInfo @userid OUTPUT, @usergroup OUTPUT

  IF @usergroup IS NULL
      SET @Verno = 'V15'
    ELSE
      SET @Verno = 'V21'

  -- Initialize variables.
  SET @OAreturn_code = 0
  SET @request_error_no = 0

  -- CREATE an instanace of the CIntegrationManger class.
  EXEC @OAreturn_code = sp_OACreate 'IntegrationManager.CIntegrationManager', @OIntegrationServer OUT
  IF @OAreturn_code <> 0
  BEGIN

    -- Try to get the error information FOR the failed CREATE.
    EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT

    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Could NOT CREATE instance of Integration Manager Class. Probable cause class IS NOT registered. Error description = %s. Source := %s.',0,1,@now, @OAerr_description,@OAerr_source) WITH NOWAIT

    ROLLBACK TRANSACTION AT_CreateInputFiles
    RETURN -3000

  END

  EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'SetRequestTypeFileInfo', NULL , @sNewInputFileExt = '.batch.in', @sNewInputFilePath = ''
  IF @OAreturn_code <> 0
  BEGIN

    -- Try to get the error information FOR the failed CREATE.
    EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT

    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method SetRequestTypeFileInfo() failed when setting the input file extension and input file path. Decsription = %s. Source = %s. @OAreturn_code = %d.', 0, 1,
              @now,@request_field_name,@OAerr_description,@OAerr_source,@OAreturn_code ) WITH NOWAIT

    ROLLBACK TRANSACTION AT_CreateInputFiles
    RETURN -3000
  
  END

  -- Add all of the data about the request to a temp statging table which will be used 
  -- later to bulk insert into the IntegrationFields table for the request.
  CREATE TABLE #int_fields(
    field_name   NVARCHAR(240),
    field_value  NVARCHAR(2000) NULL
  )

  CREATE TABLE #field_stats
    (
      field_name NVARCHAR(255),
      field_count INT
    )

  -- See IF customer is reading the IntegrationRequest table directly.
  SELECT @direct_request_read = Rtrim(propertyvalue) FROM SystemConfiguration WHERE groupname = 'AUTO_PROBLEM' and propertyname = 'DIRECT_REQUEST_READ'
  SET @direct_request_read = ISNULL(@direct_request_read,'OFF')

  -- Display the direct request read configuration values.
  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Direct request read = %s',0,1,@now,@direct_request_read) WITH NOWAIT
  END

  -- See IF there is a duplicate age value.
  SELECT @duplicate_age_char = Rtrim(propertyvalue) FROM SystemConfiguration WHERE groupname = 'AUTO_PROBLEM' and propertyname = 'DUPLICATE_AGE'
  SET @duplicate_age_char = ISNULL(@duplicate_age_char,'5')

  -- Make sure duplicate age value is numeric. If not set it to the default value to 5 minutes.
  IF ISNUMERIC(@duplicate_age_char) <> 1
    SET @duplicate_age = 5
  ELSE
    BEGIN
      -- Make sure duplicate age value is a valid integer.
      -- If not set it to the default value to 5 minutes.
      IF CONVERT(INT,@duplicate_age_char) = 0
        SET @duplicate_age = 5
      ELSE
        SET @duplicate_age = CONVERT(INT,@duplicate_age_char)
    END

  -- Display the duplicate ticket age value.
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Duplicate ticket age = %d',0,1,@now,@duplicate_age) WITH NOWAIT
  END

  --  query PendingAutoTicketFilteredEvent table to obtain event data and auto ticket rule id
  SET @eventno = 0

  DECLARE pendingfilteredevent_cursor CURSOR FOR 
    SELECT eventno, obj_cid, obj_id, event_name, event_ctime, event_alertstate, event_priority, event_state
    FROM #PendingAutoTicketFilteredEvent
    ORDER BY eventno

  OPEN pendingfilteredevent_cursor
  
  FETCH NEXT FROM pendingfilteredevent_cursor 
    INTO @eventno, @obj_cid, @obj_id, @event_name, @event_ctime, @event_alertstate, @event_priority, @event_state

  -- For each pending filtered event create a <GUID>.in file.
  WHILE @@FETCH_STATUS = 0
  BEGIN

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
      RAISERROR('%s: AT_CreateInputFiles: Processing eventno = %d',0,1,@now,@eventno) WITH NOWAIT
    END

    -- Do a duplicate age check for each event that was filtered.
    IF EXISTS( SELECT * FROM AT_Created 
      WHERE (LTRIM(RTRIM(obj_cid)) = LTRIM(RTRIM(@obj_cid))) AND 
            (obj_id = @obj_id) AND 
            (LTRIM(RTRIM(event_name)) = LTRIM(RTRIM(@event_name))) AND 
            (event_alertstate = @event_alertstate) AND 
            (event_priority = @event_priority) AND 
            ((event_state = @event_state) or ((event_state IS NULL) AND (@event_state IS NULL))) AND
            (DATEDIFF(minute,event_ctime,@event_ctime) < @duplicate_age))
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
        RAISERROR('%s: AT_CreateInputFiles: Filtered event failed duplicate age test, eventno = %d.',0,1,@now,@eventno) WITH NOWAIT
      END
    END

    ELSE
      BEGIN

        SELECT @ctime = ctime, @obj_cid = obj_cid, @obj_id = obj_id, @obj_name = obj_name,
               @event_cid = event_cid, @event_id = event_id, @event_name = event_name,
               @event_ctime = event_ctime, @event_detail = event_detail, @event_alertstate = event_alertstate,
               @event_priority = event_priority, @event_state = event_state, @rule_id = rule_id
        FROM PendingAutoTicketFilteredEvent
        WHERE @eventno = eventno

        IF @@rowcount = 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
          RAISERROR('%s: AT_CreateInputFiles: Could not find event in PendingAutoTicketFilteredEvent table - Note/Ticket note created, eventno=%d, obj_cid=%s, obj_id=%d',
                     0,1, @now, @eventno, @obj_cid, @obj_id) WITH NOWAIT
        END

        SET @phy_cid = @obj_cid
        SET @phy_id = @obj_id
          
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
          RAISERROR('%s: AT_CreteInputFiles: Value of @phy_cid := %s before asisp_followphysicallink is called.', 0, 1,@now,@phy_cid ) WITH NOWAIT
          RAISERROR('%s: AT_CreteInputFiles: Value of @phy_id  := %d before asisp_followphysicallink is called.', 0, 1,@now,@phy_id ) WITH NOWAIT
        END

        -- Get object physical cid and id.
        -- CONVERT LOB to the physical object
        IF RTRIM(@obj_cid) = 'LOB'
        BEGIN

          -- WHEN calling asisp_followphysicallink @parentcid must be non-NULL IF the RETURN parameters are to be filled IN
          EXEC asisp_followphysicallink @phy_id, @phy_cid OUTPUT, @phy_id OUTPUT
	
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
            RAISERROR('%s: AT_CreteInputFiles: Resource phycical CID := %s.', 0, 1,@now, @phy_cid ) WITH NOWAIT
            RAISERROR('%s: AT_CreteInputFiles: Resource phycical  ID := %d.', 0, 1,@now, @phy_id ) WITH NOWAIT
          END

        END

        -- Attempt to lookup resource name is it wasn't populated in the event. Make sure the resource
        -- class exist before calling asisp_getobjname. It can't tolerate non-existent classes.
        IF EXISTS(SELECT * FROM obj_class WHERE LTRIM(RTRIM(cid)) = LTRIM(RTRIM(@obj_cid)))
        BEGIN
          IF @obj_name = ''
          BEGIN
            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
              RAISERROR('%s: AT_CreteInputFiles: Attempting to resolve resource name. It was empty in the event.', 0, 1, @now ) WITH NOWAIT
            END

            EXEC asisp_getobjname @obj_cid, @obj_id, @obj_name OUTPUT
            SET @obj_name = ISNULL(@obj_name,'N/A')

          END
        END

        -- If resource name is still empty at this point, then set it to default value.
        IF @obj_name = ''
          SET @obj_name = 'N/A'
         
        -- ****  had to convert event alert state to char to match parameter of called procedures
        SET @event_alertstate = CONVERT(NVARCHAR(16),@event_alertstate)

        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
          RAISERROR('%s: AT_CreateInputFiles: Call _OpenIntegrationRequest to add request and obtain a GUID.', 0, 1, @now ) WITH NOWAIT
        END

        EXEC _OpenIntegrationRequest @obj_cid, @obj_id, 'REQUEST_AUTOTICKET', 'REQUEST_ADD', @request_guid OUT
        IF @request_guid = ''
        BEGIN
          RAISERROR('AT_CreateInputFiles: _OpenIntegrationRequest did not return a GUID - Note/Ticket not created, obj_cid=%s, obj_id=%d',
                    0,1, @obj_cid, @obj_id) WITH NOWAIT
          SET @request_error_no = -30000
          BREAK
        END

        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
          RAISERROR('%s: AT_CreateInputFiles: Request GUID := %s', 0, 1, @now,@request_guid ) WITH NOWAIT
          RAISERROR('%s: AT_CreateInputFiles: Adding event number := %d to request for future reference.', 0, 1, @now, @eventno ) WITH NOWAIT
        END

        INSERT INTO #int_fields VALUES('REQUEST_EVENT_NO',@eventno)
        INSERT INTO #int_fields VALUES('REQUEST_AUTO_CREATE','YES') 
        INSERT INTO #int_fields VALUES('REQUEST_OBJECT_NAME',@obj_name) 
        INSERT INTO #int_fields VALUES('REQUEST_USER_ID','') 
        INSERT INTO #int_fields VALUES('REQUEST_PASSWORD','') 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_CID',@event_cid) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_ID',@event_id) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_DETAIL',@event_detail) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_NAME',@event_name) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_STATE',@event_state) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_TIME',@event_ctime) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_PRIORITY',@event_priority) 
        INSERT INTO #int_fields VALUES('REQUEST_EVENT_ALERT_STATE',@event_alertstate) 

        -- **************************************************************************************************
        -- For enhancement request MR1119023325 a call to AT_addExceptionInfo was added to insert the 
        -- exception propagation matrix and exception occurrance count into the <guid>.in file.
        -- **************************************************************************************************
        -- If the event that was passed was a Exception, add exception matrix information to the input files.
        -- **************************************************************************************************
        IF @event_cid = 'EXCP'
        BEGIN

          SELECT @event_parent = _ParentID FROM ExceptionHistory_V WHERE id = @event_id

          IF @event_parent IS NULL
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
            RAISERROR('%s: AT_CreateInputFiles: AT_addExceptionInfo failed - Could not get parent ID for ExceptionHistory id=%d.', 0,1, @now, @event_id) WITH NOWAIT
            SET @request_error_no = -30000
            BREAK
          END

          -- Add extended Exception information to temp stating table.
          EXEC AT_addExceptionInfo @event_parent,@obj_cid,@obj_id,@request_error_no OUT

          IF @request_error_no <> 0
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
            RAISERROR('%s: AT_CreateInputFiles: AT_addExceptionInfo failed - Exception information not added, event_id=%d, obj_cid=%s, obj_id=%d',
                      0,1, @now, @event_id, @obj_cid, @obj_id)
            SET @request_error_no = -30000
            BREAK
          END
        END

        -- Add event status information to integration request
        -- Initialize output variable values.
        SET @event_owned = ''
        SET @event_problem_id = ''
        SET @event_owner = ''
        SET @event_active = ''

        EXEC @return_code = AT_GetEventStatus @obj_cid, @obj_id, @event_cid, @event_parent,  
          @event_owned OUTPUT,@event_problem_id OUTPUT,@event_owner OUTPUT, @event_active OUTPUT

        IF @return_code <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121) 
          RAISERROR('%s: AT_CreateInputFiles: Call to AT_GetEventStatus failed with return code = %d.',0,1,@now, @return_code) WITH NOWAIT
          BREAK
        END

        -- Add event status infor to the request.
        INSERT INTO #int_fields VALUES('EVENT_OWNED',@event_owned) 
        INSERT INTO #int_fields VALUES('EVENT_PROBLEM_ID',@event_problem_id) 
        INSERT INTO #int_fields VALUES('EVENT_OWNER',@event_owner) 
        INSERT INTO #int_fields VALUES('EVENT_ACTIVE',@event_active) 

        -- Add all of the name/value pairs in the temp statging table to the IntegrationFields table
        INSERT INTO IntegrationFields  (request_guid, field_name, field_value)
          SELECT @request_guid, field_name, field_value from #int_fields
  
        IF @@ERROR <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles insert of name/value paris from temp staging table #int_fields into IntegrationFields failed.',
             0,1, @now)
          SET @request_error_no = -30000
          BREAK
        END

        -- Initilaize Input file with common data.
        DECLARE @common_input NVARCHAR(255)

        -- Can't do inline conversion WHEN calling a method, so CONVERT to temp variable first.
        DECLARE @temp_id NVARCHAR(30)
        DECLARE @temp_id2 NVARCHAR(30)

        SET @temp_id = CONVERT(NVARCHAR(30),@phy_id)
        SET @temp_id2 = CONVERT(NVARCHAR(30),@id)

        SET @request_type = ''
        SET @request_command_processor = ''
        SET @request_processor_language = ''

        -- Get the request type FROM the original request.
        SELECT @request_type = request_type, @request_method = method FROM IntegrationRequest WHERE request_guid = @request_guid

        -- Debug PRINT statement.
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreteInputFiles: Request Type := %s', 0, 1, @now, @request_type ) WITH NOWAIT
          RAISERROR('%s: AT_CreteInputFiles: Request Method := %s', 0, 1, @now, @request_method ) WITH NOWAIT
          RAISERROR('%s: AT_CreteInputFiles: Reading request processor name and language from SystemConfiguration table.', 0, 1,@now) WITH NOWAIT
        END

        SELECT @request_command_processor = propertyvalue FROM SystemConfiguration WHERE groupname = RTRIM(@request_type) AND propertyname = 'REQUEST_PROCESSOR_NAME'
        SELECT @request_processor_language = propertyvalue FROM SystemConfiguration WHERE groupname = RTRIM(@request_type) AND propertyname = 'REQUEST_PROCESSOR_LANGUAGE'

        SET @common_input = 'REQUEST_TYPE' + ',' + @request_type + '|' + 'REQUEST_METHOD' + ',' + @request_method + '|' + 'REQUEST_CONSOLE_VERSION' + ',' + @Verno + '|'
       
        SET @common_input = @common_input + 'REQUEST_OBJECT_CID' + ',' + @phy_cid + '|' + 'REQUEST_OBJECT_ID' + ',' + @temp_id + '|' + 'REQUEST_OBJECT_SOURCE_CID' + ',' + @obj_cid + '|'
        SET @common_input = @common_input + 'REQUEST_OBJECT_SOURCE_ID' + ',' + @temp_id2 

        -- Make sure we have all of the information to invoke the request processor.
        IF (@request_type = '') OR (@request_method = '') OR (@request_command_processor = '')
        BEGIN
          SET @err_description = 'Request Type := ' + @request_type + ' OR Request Method := ' + @request_method + ' OR Request Processor Name := ' + @request_command_processor + ' could NOT be determined.'
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles: Request Type = %s or Request Method = %s or Request Processor Name = %s could not be determined.',0,1, @now, @request_type, @request_method, @request_command_processor) WITH NOWAIT
          SET @request_error_no = -30000
          BREAK
        END

        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles: Request Processor Name := %s', 0, 1, @now, @request_command_processor ) WITH NOWAIT
          RAISERROR('%s: AT_CreateInputFiles: Request Processor Language := %s', 0, 1, @now, @request_processor_language ) WITH NOWAIT
        END

        -- Default request procesror language IS a WIN32 exe.
        IF  @request_processor_language IS NULL
          SET @request_processor_language = NULL


        -- If requests are not going to be directly read then create the input files.
        IF UPPER(LTRIM(RTRIM(@direct_request_read))) = 'OFF'
        BEGIN

          -- Initialize variables.
          SET @OAreturn_code = 0
          SET @request_error_no = 0

          WHILE @request_error_no = 0
          BEGIN

            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Calling RequestInitialize method.', 0, 1, @now ) WITH NOWAIT
            END
              
            -- Set timeout to default value of 45 seconds.
            SET @timeout_value = 45

            --Public Function RequestInitialize(lNewTimeOutSeconds As Long, sNewGUID As String, sNewRequestProcessor As String, sNewProcessorLanguage As String) As Long
            EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'RequestInitialize', @return_code out, 
                 @lNewTimeOutSeconds = @timeout_value, @sNewGUID = @request_guid, 
                 @sNewRequestProcessor = @request_command_processor, @sNewProcessorLanguage = @request_processor_language
    
            IF @OAreturn_code <> 0
            BEGIN
              -- Try to get the error information FOR the failed CREATE.
              EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method RequestInitialize() failed. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1, @now, @OAerr_description, @OAerr_source, @OAreturn_code ) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            -- Create request inut file.
            EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'CreateInputFile', @return_code out
            IF @OAreturn_code <> 0
            BEGIN
              -- Try to get the error information FOR the failed CREATE.
              EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)        
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method CreateInputFile() failed. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1, @now, @OAerr_description, @OAerr_source, @OAreturn_code ) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            IF @return_code <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method CreateInputFile() returned a error. Return code = %d.', 0, 1,@now, @return_code )  WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Calling InitializeInputFile() with input := %s', 0, 1, @now, @common_input ) WITH NOWAIT
            END

            EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'InitializeInputFile', @return_code out, @sInputData = @common_input
            IF @OAreturn_code <> 0
            BEGIN
              -- Try to get the error information FOR the failed CREATE.
              EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method InitializeInputFile() failed. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1,@now, @OAerr_description, @OAerr_source, @OAreturn_code ) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            -- ADD all of the integration fields to the CIntegrationManager object.
            -- Populate temp TABLE with the field record counts.
            INSERT INTO #field_stats SELECT field_name,COUNT(*) FROM IntegrationFields WHERE request_guid = @request_guid GROUP BY field_name

            -- Curor to traverse all of the request fields FOR the request_guid passed IN.
            DECLARE integration_fields_cursor CURSOR FOR SELECT field_name, field_value FROM IntegrationFields WHERE request_guid = @request_guid ORDER BY row_id FOR READ ONLY

            OPEN integration_fields_cursor
            FETCH NEXT FROM integration_fields_cursor INTO @request_field_name, @request_field_value

            SET @field_count = 1
            SET @current_field_name = @request_field_name

            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Processing requets input fields.', 0, 1, @now ) WITH NOWAIT
            END

            -- Loop AND INSERT all of the request name value pairs INTO the input file.
            WHILE @@FETCH_STATUS = 0
            BEGIN
 
              -- Get record COUNT FOR current name value pair.
              SELECT @request_field_row_count = field_count FROM #field_stats WHERE RTRIM(field_name) = RTRIM(@request_field_name)

              -- Debug PRINT statement.
              IF @verbose >= 1
              BEGIN
                SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
                RAISERROR('%s: AT_CreateInputFiles: @request_field_row_count := %d', 0, 1, @now, @request_field_row_count ) WITH NOWAIT
                RAISERROR('%s: AT_CreateInputFiles: Attempting to append field := %s into input file.', 0, 1, @now, @request_field_name ) WITH NOWAIT
              END

              -- ADD each name value pair to the input file.
              EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'AppendInputFile', @return_code out, @sFieldName = @request_field_name, @sFieldValue = @request_field_value, @lRowId = @field_count, @lRowCount = @request_field_row_count
              IF @OAreturn_code <> 0
              BEGIN
                -- Try to get the error information FOR the failed CREATE.
                EXEC _GetLastOAError @OIntegrationServer ,@OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
                SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
                RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method AppendInputFile() failed when appending field name = %s. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1,
                           @now,@request_field_name,@OAerr_description,@OAerr_source,@OAreturn_code ) WITH NOWAIT
                SET @request_error_no = -30000
                BREAK
              END

              IF @return_code <> 0
              BEGIN
                SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
                RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method AppendInputFile() returned a error when appending %s. Return code = %d.', 0, 1, @now, @return_code ) WITH NOWAIT
                SET @request_error_no = -30000
                BREAK
              END

              FETCH NEXT FROM integration_fields_cursor INTO @request_field_name, @request_field_value
              IF @current_field_name <> @request_field_name
              BEGIN
                SET @field_count = 1
                SET @current_field_name = @request_field_name
              END
              ELSE
                SET @field_count = @field_count + 1

            END -- END of FETCH loop.
              
            CLOSE integration_fields_cursor
            DEALLOCATE integration_fields_cursor

            -- Clean out #field_stats and #int_fields for next input file.
            DELETE #field_stats
            DELETE #int_fields

            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Input field processing complete.', 0, 1, @now)WITH NOWAIT
              RAISERROR('%s: AT_CreateInputFiles: Attempting to close input file.', 0, 1, @now)WITH NOWAIT
            END

            EXEC @OAreturn_code = sp_OAMethod @OIntegrationServer, 'CloseInputFile', @return_code out
            IF @OAreturn_code <> 0
            BEGIN
              -- Try to get the error information FOR the failed CREATE.
              EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method CloseInputFile() failed. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1, @now, @OAerr_description, @OAerr_source, @OAreturn_code ) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            IF @return_code <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Call to sp_OAMethod() for method CloseInputFile() returned a error. Return code = %d.', 0, 1, @now, @return_code ) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END

            -- Debug PRINT statement.
            IF @verbose >= 1
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Input file closed.', 0, 1, @now ) WITH NOWAIT
            END

             -- Something happend while creating the input file. Do not continue.
            IF @request_error_no <> 0 
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_CreateInputFiles: Could NOT create input file.',0,1, @now) WITH NOWAIT
              SET @request_error_no = -30000
              BREAK
            END
            
            BREAK -- If we got this far then input file was created without error.
              
          END -- End create input file loop
            
          IF @request_error_no <> 0 
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: AT_CreateInputFiles: Could NOT create input file.',0,1, @now) WITH NOWAIT
            BREAK -- Break out of pendingfilteredevent_cursor because something went wrong.
          END
          
        END -- END for direct read

      END -- End of duplicate age check

      -- Update the eventno of LastProcessedAutoTicketEvent for each event processed.
      -- UPDATE LastProcessedAutoTicketFilteredEvent SET eventno = @eventno
      EXEC asisp_setLastProcessedAutoTicketFilteredEvent @eventno
      IF @@ERROR <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Could NOT update eventno column of LastProcessedAutoTicketFilteredEvent. Error code=%d.',0,1,@now, @@ERROR) WITH NOWAIT
        SET @request_error_no = -30000
        BREAK
      END
      ELSE
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles: Update of eventno column of LastProcessedAutoTicketFilteredEvent completed without error for @eventno = %d.',0,1,@now, @eventno) WITH NOWAIT
        END

      -- Update the deleted attribute of #PendingAutoTicketFilteredEvent for each event processed.
      UPDATE PendingAutoTicketFilteredEvent SET deleted = 1 where eventno = @eventno
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRAN AT_CreateInputFiles
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Could NOT update deleted column of PendingAutoTicketFilteredEvent. Error code=%d.',0,1, @now, @@ERROR) WITH NOWAIT
        SET @request_error_no = -30000
        BREAK
      END
      ELSE
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles: Update of deleted column of PendingAutoTicketFilteredEvent completed without error for @eventno = %d.',0,1,@now, @eventno) WITH NOWAIT
        END

      -- Update request in IntegrationRequest table to mark it as a pending Auto Ticket request.
      -- A value of 100 in the status column will indicate that the request is pending processing
      -- by a request processor.
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Updating status of the request to pending for guid := %s',0,1, @now,@request_guid) WITH NOWAIT
      END

      UPDATE IntegrationRequest SET status = 100 WHERE request_guid = @request_guid
      IF @@ERROR <> 0
      BEGIN
        ROLLBACK TRAN AT_CreateInputFiles
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Could NOT update the status of the request to pending for guid := %s',0,1, @now, @request_guid) WITH NOWAIT
        SET @request_error_no = -30000
        BREAK
      END
      ELSE
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_CreateInputFiles: Update of status column of IntegrationRequest completed without error for @request_guid = %s.',0,1,@now, @request_guid) WITH NOWAIT
        END

    FETCH NEXT FROM pendingfilteredevent_cursor 
     INTO @eventno, @obj_cid, @obj_id, @event_name, @event_ctime, @event_alertstate, @event_priority, @event_state

  END  -- End of cursor for pendingfilteredevent_cursor

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Closing pendingfilteredevent_cursor.', 0, 1, @now) WITH NOWAIT
  END

  CLOSE pendingfilteredevent_cursor
  DEALLOCATE pendingfilteredevent_cursor

  -- Release ActiveX object by calling sp_OADestroy.
  IF @OIntegrationServer <> 0 and (@OIntegrationServer IS NOT NULL)
  BEGIN

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_CreateInputFiles: Attempting to destroy object instance.', 0, 1, @now) WITH NOWAIT
    END

    EXEC @OAreturn_code = sp_OADestroy @OIntegrationServer
    IF @OAreturn_code <> 0
    BEGIN

      -- Try to get the error information FOR the failed CREATE.
      EXEC _GetLastOAError @OIntegrationServer, @OAerr_source OUT, @OAerr_description OUT, @OAreturn_code OUT

      -- Debug PRINT statement.
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Call to sp_OADestroy failed. Object handle = %d. Description = %s. Source = %s. @OAreturn_code = %d.', 0, 1,@now, @OIntegrationServer,@OAerr_description,@OAerr_source,@OAreturn_code ) WITH NOWAIT
        SET @request_error_no = -30000
      END
  
    END
    ELSE
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_CreateInputFiles: Call to sp_OADestroy called and completed without error. Object handle = %d. Return code = %d.', 0, 1,@now, @OIntegrationServer,@OAreturn_code )  WITH NOWAIT
      END

  END
  ELSE
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_CreateInputFiles: sp_OADestroyed not called because no object handle available.', 0, 1, @now) WITH NOWAIT
    END

  -- Debug PRINT statement.
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Attempting to finish TRANSACTION AT_CreateInputFiles.', 0, 1, @now) WITH NOWAIT
    RAISERROR('%s: AT_CreateInputFiles: Last value of @@ERROR = %d. Request error = %d.', 0, 1,@now, @@error, @request_error_no ) WITH NOWAIT
  END
    
  IF (@@ERROR <> 0) or (@request_error_no <> 0)
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_CreateInputFiles: Rolling back transaction AT_CreateInputFiles.', 0, 1, @now ) WITH NOWAIT
    ROLLBACK TRANSACTION  AT_CreateInputFiles
  END
  ELSE
    IF @tcount = 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: AT_CreateInputFiles: Commiting transaction AT_CreateInputFiles.', 0, 1, @now ) WITH NOWAIT
      COMMIT TRANSACTION  AT_CreateInputFiles
    END
