-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/at_dequeuefilteredevents.sql
-- SID: 1.7.1.9
-- Date Modified: 04/11/12 14:26:30
--
-- Module Description:
--   Defines the AT_DeQueueFilteredEvents stored proc which dequeues rows from the 
--   PendingAutoTicketFilteredEvent table
--   and calls AT_Create or AT_CreatePending if INPUT_BATCH_MODE is enabled in the SystemConfiguration table.

USE Object
GO

-------------------------------------------------------------------------------------------
PRINT 'Creating procedure AT_DeQueueFilteredEvents.'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'AT_DeQueueFilteredEvents' AND type = 'P')
  DROP PROCEDURE AT_DeQueueFilteredEvents
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/at_dequeuefilteredevents.sql
-- SID: 1.7.1.9
-- Date Modified: 04/11/12 14:26:30
--
-- Module Description:
--   Defines the AT_DeQueueFilteredEvents stored proc which dequeues rows from the 
--   PendingAutoTicketFilteredEvent table
--   and calls AT_Create or AT_CreatePending if INPUT_BATCH_MODE is enabled in the SystemConfiguration table.
CREATE PROCEDURE AT_DeQueueFilteredEvents
  @cid ClassID,
  @id  ObjID,
  @rowlimit INT = 100,
  @verbose  TINYINT = 0
AS

  DECLARE @last_event INT
  DECLARE @current_event INT

  DECLARE @obj_cid            ClassID
  DECLARE @obj_id             int
  DECLARE @obj_name           ObjName
  DECLARE @event_cid          ClassID
  DECLARE @event_id           int
  DECLARE @event_name         ObjName
  DECLARE @event_ctime        datetime
  DECLARE @event_alertstate   int
  DECLARE @event_priority     int
  DECLARE @event_state        int

  DECLARE @duplicate_age INT
  DECLARE @duplicate_age_char NVARCHAR(255)
  DECLARE @input_batch_mode NVARCHAR(255)

  DECLARE @now NVARCHAR(40)
  DECLARE @eventno INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_DeQueueFilteredEvents: Called with parameters @cid=%s @id=%d @rowlimit=%d @verbose=%d',0,1,@now,@cid,@id,@rowlimit,@verbose) WITH NOWAIT
  END

  -- Get last ids of the last events processed.
  SELECT @last_event = eventno FROM LastProcessedAutoTicketFilteredEvent

  -- Get current evento in the queue.
  SELECT @current_event = eventno FROM PendingAutoTicketFilteredEvent
 
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: AT_DeQueueFilteredEvents: Last Filtered Event Processed := %d',0,1,@now,@last_event) WITH NOWAIT
    RAISERROR('%s: AT_DeQueueFilteredEvents: Current Filtered Event := %d',0,1,@now,@current_event) WITH NOWAIT
  END

  -- Set the maximum number of rows to process.
  SET ROWCOUNT @rowlimit

  -- If the last process event is not the last event then there are
  -- new events to process.
  IF @last_event <> @current_event
    BEGIN

      IF @verbose >= 1
      BEGIN
       SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
       RAISERROR('%s: AT_DeQueueFilteredEvents: Processing new filtered events.',0,1,@now) WITH NOWAIT
      END
      
      -- See IF there is a input batch mode value.
      SELECT @input_batch_mode = UPPER(LTRIM(RTRIM(propertyvalue))) FROM SystemConfiguration WHERE groupname = 'AUTO_PROBLEM' and propertyname = 'INPUT_BATCH_MODE'
      SET @input_batch_mode = ISNULL(@input_batch_mode,'OFF')

      IF @verbose >= 1
      BEGIN
       SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
       RAISERROR('%s: AT_DeQueueFilteredEvents: Input batch mode = %s.',0,1,@now,@input_batch_mode) WITH NOWAIT
      END
   
      -- See IF there is a duplicate age value.
      SELECT @duplicate_age_char = Rtrim(propertyvalue) FROM SystemConfiguration WHERE groupname = 'AUTO_PROBLEM' and propertyname = 'DUPLICATE_AGE'
      SET @duplicate_age_char = ISNULL(@duplicate_age_char,'5')

      -- Make sure duplicate age value is numeric. If not set it to the default value to 5 minutes.
      IF ISNUMERIC(@duplicate_age_char) <> 1
        SET @duplicate_age = 5
      ELSE
        BEGIN
          -- Make sure duplicate age value is a valid integer.
          -- If not set it to the default value to 5 minutes.
          IF CONVERT(INT,@duplicate_age_char) = 0
            SET @duplicate_age = 5
          ELSE
            SET @duplicate_age = CONVERT(INT,@duplicate_age_char)
        END

      -- Display the duplicate ticket age value.
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_DeQueueFilteredEvents: Duplicate ticket age = %d',0,1,@now,@duplicate_age) WITH NOWAIT
      END

      -- Get a set of filtered events to process.
      SELECT * INTO #PendingAutoTicketFilteredEvent
      FROM PendingAutoTicketFilteredEvent p
      WHERE p.eventno > @last_event and (p.deleted IS NULL or p.deleted = 0)

      IF @@ERROR <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: AT_DeQueueFilteredEvents: Could NOT insert rows into #PendingAutoTicketFilteredEvent table. Error code=%d.',0,1,@now,@@ERROR) WITH NOWAIT
        RETURN -30000
      END

      IF @verbose >= 2
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s : AT_DeQueueFilteredEvents:Initial available rows from #PendingAutoTicketFilteredEvent',0,1, @now) WITH NOWAIT
        SELECT * FROM #PendingAutoTicketFilteredEvent ORDER BY ctime
      END

      -- If the batch mode for creating input files is ON then call AT_CreateInputFiles
      IF @input_batch_mode = 'ON'
      BEGIN

        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: AT_DeQueueFilteredEvents: Calling AT_CreateInputFiles. Input batch mode = %s',0,1,@now,@input_batch_mode) WITH NOWAIT
         END

         EXEC AT_CreateInputFiles @cid, @id, @verbose
         IF @@ERROR = 0 
         BEGIN
           IF @verbose >= 1
           BEGIN
             SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
             RAISERROR('%s: AT_DeQueueFilteredEvents: Called to AT_CreateInputFiles completed without error.',0,1,@now) WITH NOWAIT
           END
           RETURN 0            
         END
         ELSE
           BEGIN
             SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
             RAISERROR('%s: AT_DeQueueFilteredEvents: Call to AT_CreateInputFiles failed with @@error=%d.', 11, 1, @now, @@ERROR) WITH NOWAIT
             RETURN -30000
           END
      END

      SET @eventno = 0

      -- Cursor over the rows in #PendingAutoTicketFilteredEvent and call AT_Create for each event.
      DECLARE pendingfilteredevent_cursor CURSOR FOR 
      SELECT eventno, obj_cid, obj_id, event_name, event_ctime, event_alertstate, event_priority, event_state
        FROM #PendingAutoTicketFilteredEvent
        ORDER BY eventno

      OPEN pendingfilteredevent_cursor
  
      FETCH NEXT FROM pendingfilteredevent_cursor 
        INTO @eventno, @obj_cid, @obj_id, @event_name, @event_ctime, @event_alertstate, @event_priority, @event_state

      WHILE @@FETCH_STATUS = 0
        BEGIN

	  IF @verbose >= 1 
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: AT_DeQueueFilteredEvents: Processing eventno = %d',0,1,@now,@eventno) WITH NOWAIT
          END

          -- Call AT_Create for each event to process.
          -- Do a duplicate age check for each event that was filtered.
          IF EXISTS( SELECT * FROM AT_Created 
            WHERE (LTRIM(RTRIM(obj_cid)) = LTRIM(RTRIM(@obj_cid))) AND 
                  (obj_id = @obj_id) AND 
                  (LTRIM(RTRIM(event_name)) = LTRIM(RTRIM(@event_name))) AND 
                  (event_alertstate = @event_alertstate) AND 
                  (event_priority = @event_priority) AND 
                  ((event_state = @event_state) or ((event_state IS NULL) AND (@event_state IS NULL))) AND
                  (DATEDIFF(minute,event_ctime,@event_ctime) < @duplicate_age))
          BEGIN
	    IF @verbose >= 1 
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_DeQueueFilteredEvents: Filtered event failed duplicate age test, eventno = %d.',0,1,@now,@eventno) WITH NOWAIT
            END
          END

          ELSE
            BEGIN

              IF @verbose >= 1 
              BEGIN
                SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
                RAISERROR('%s: AT_DeQueueFilteredEvents: About to call AT_Create fro eventno = %d.',0,1,@now,@eventno) WITH NOWAIT
              END

              EXEC AT_Create @cid, @id, @eventno, @verbose
              IF @@ERROR <> 0
              BEGIN
                SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
                RAISERROR('%s: AT_DeQueueFilteredEvents: Call to AT_Create failed. Error code=%d.',0,1,@now,@@ERROR)
              END
            END
   
          -- Update the eventno of LastProcessedAutoTicketEvent for each event processed.
          -- UPDATE LastProcessedAutoTicketFilteredEvent SET eventno = @eventno
          EXEC asisp_setLastProcessedAutoTicketFilteredEvent @eventno
          IF @@ERROR <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_DeQueueFilteredEvents: Could NOT update eventno column of LastProcessedAutoTicketFilteredEvent. Error code=%d.',0,1,@now,@@ERROR) WITH NOWAIT
              RETURN -30000
            END

          -- Update the deleted attribute of #PendingAutoTicketFilteredEvent for each event processed.
          UPDATE PendingAutoTicketFilteredEvent SET deleted = 1 where eventno = @eventno
          IF @@ERROR <> 0
            BEGIN
              SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
              RAISERROR('%s: AT_DeQueueFilteredEvents: Could NOT deleted column of PendingAutoTicketFilteredEvent. Error code=%d.',0,1,@now,@@ERROR) WITH NOWAIT
              RETURN -30000
            END

          FETCH NEXT FROM pendingfilteredevent_cursor 
          INTO @eventno, @obj_cid, @obj_id, @event_name, @event_ctime, @event_alertstate, @event_priority, @event_state
  
        END

        CLOSE pendingfilteredevent_cursor
        DEALLOCATE pendingfilteredevent_cursor

  END

  RETURN 0