-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\create_delete_procs.sql
-- SID: 1.1
-- Date Modified: 04/11/14 12:42:20
-- Creates delete_<cid> procs for directly contained classes
USE Object
GO

---------------------------------------------------------------------------------------------------
-- create_delete_procs
PRINT 'Creating procedure create_delete_procs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'create_delete_procs' AND type = 'P')
  DROP PROCEDURE create_delete_procs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\create_delete_procs.sql
-- SID: 1.1
-- Date Modified: 04/11/14 12:42:20
-- Creates delete_<cid> procs for directly contained classes
CREATE PROCEDURE create_delete_procs
AS

  DECLARE @sql NVARCHAR(4000), @cname NVARCHAR(48), @cid ClassID, @proc_name NVARCHAR(128), 
          @phy_delete INT, @mname NVARCHAR(24), @mid INT, @desc NVARCHAR(255)

  SET NOCOUNT ON

  DECLARE create_del_procs INSENSITIVE CURSOR FOR
    -- skip if it is a lightweight-class, and create only for directly contained class
    SELECT DISTINCT cname, cid FROM obj_class
    WHERE 
          flag_lightweight        = 0 
      AND flag_direct_containment = 1
      AND NOT EXISTS (SELECT * FROM custom_obj WHERE name = 'delete_' + cid)
      AND EXISTS (SELECT * FROM sysobjects WHERE name = 'delete_' + cid)
  OPEN create_del_procs
  FETCH NEXT FROM create_del_procs INTO @cname, @cid
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- create the delete_<cid> procedures
    SET @proc_name = 'delete_' + @cid

    SET @sql = '
IF EXISTS (SELECT * FROM sysobjects WHERE name = ''' + @proc_name + ''' AND type = ''P'')' + '
  DROP PROCEDURE ' + @proc_name + '
'

    EXEC(@sql)

    SET @sql = '
-- File: archive\db\sql\procs\create_delete_procs.sql' + '
-- SID: 1.1' + '
-- Date Modified: 04/11/14 12:42:20' + '
CREATE PROCEDURE ' + @proc_name + '
  @id           ObjID,' + '
  @delete_level SMALLINT = 0,' + '
  @verbose      TINYINT = 0' + '
AS' + '
  SET NOCOUNT ON' + '
  ' + '
  DECLARE @tcount INT' + '
  SET @tcount = @@TRANCOUNT' + '
  IF @tcount = 0' + '
    BEGIN TRANSACTION ' + @proc_name + '
  ELSE' + '
    SAVE TRANSACTION ' + @proc_name + '
  ' + '
  IF @verbose >= 1' + '
  BEGIN' + '
    DECLARE @spaces NVARCHAR(80)' + '
    SET @spaces=SPACE(@@nestlevel+1)' + '
    RAISERROR(''%s:' + @proc_name + ' %d, %d'', 0, 1, @spaces, @id, @delete_level)' + '
  END' + '
  ' + '
  DECLARE @result INT' + '
  EXEC @result = delete_obj ''' + @cid + ''', @id' + '
  IF @result < 0' + '
  BEGIN' + '
    ROLLBACK TRANSACTION ' + @proc_name + '
    RETURN @result' + '
  END' + '
  ' + '
  -- Call method(s) that are marked AS flag_called_on_obj_delete IN obj_method'

    SELECT @mname = m.mname, @mid = m.mid, @desc = m.description
    FROM obj_method AS o
      JOIN method_table AS m ON m.mid = o.mid
    WHERE
          o.cid = @cid
      AND o.flag_called_on_obj_delete = 1

    IF @mname IS NOT NULL
    BEGIN
      SET @sql = @sql + '
  -- ' + @mname + '(' + CAST(@mid AS NVARCHAR) + '): ''' + @desc + '''' + '
  RAISERROR(''%s: calling method _' + @mname + ' cid=%s, id=%d'', 0, 1, ''' + @cid + ''', @id)' + '
  EXEC @result=_' + @mname + ' ''' + @cid + ''', @id' + '
  IF @result<0' + '
  BEGIN' + '
    ROLLBACK TRANSACTION ' + @proc_name + '
    RETURN @result' + '
  END'
    END

    SELECT @phy_delete = flag_physically_delete FROM obj_class
    WHERE
          cid   = @cid
      AND cname = @cname

    IF @phy_delete = 1
    BEGIN
      SET @sql = @sql + '
  -- Delete the attribute row as flag_physically_delete is set in obj_class' + '
  DELETE FROM ' + @cname + '_A WHERE id = @id'
    END

    SET @sql = @sql + '        

  IF @tcount = 0' + '
   COMMIT TRANSACTION ' + @proc_name + '
  RETURN @result' + '
  '

    EXEC(@sql)

    FETCH NEXT FROM create_del_procs INTO @cname, @cid
  END
  DEALLOCATE create_del_procs

GO