-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/getrootcauseresources.sql
-- SID: 1.18
-- Date Modified: 04/11/15 15:20:44
--
-- Module Description
--   Defines the GetRootCauseResources stored procedure

---------------------------------------------------------------------------------------------------
-- GetRootCauseResources
USE Object
GO
PRINT 'Creating procedure GetRootCauseResources'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'GetRootCauseResources' AND type = 'P')
  DROP PROCEDURE GetRootCauseResources
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/getrootcauseresources.sql
-- SID: 1.18
-- Date Modified: 04/11/15 15:20:44
-- Returns the impacted Resources and root cause events for a specified BusinessSystem whose AlertState is not Green
CREATE PROCEDURE GetRootCauseResources
  @cid            ClassID,       -- Class id BusinessSystem  
  @id             ObjID,         -- Instance id of BusinessSystem    
  @cnt            INT  OUTPUT,   -- Number of events calculated, meaningful only when input table  
                                 -- not specified                                     
  @rscTableName   sysname = NULL,-- Output table for impacted resources, including path information 
  @eventTableName sysname = NULL,-- Output table for root cause exceptions and messages
                                 -- If Output tables are not specified then informationis output to the screen.
  @getEvents      TINYINT = 0,   -- 0 =  no event information returned, 1 = event information will be returned
  @verbose        TINYINT = 0    -- 1 = output debug messages 
AS  
-- --------------------------------------------------------------------------------------  
-- Optional Input takes precedence over parameters cid and id.  
-- The optional input table specified multiple cid,id pairs   
-- CREATE TABLE #RootCauseServices  
--(  
--  ServiceCID   ClassID NULL,  
--  ServiceID    ObjID NULL,  
--  cid    ClassID NOT NULL,  
--  id     ObjID NOT NULL  
--)   
-- --------------------------------------------------------------------------------------
  
  SET NOCOUNT ON  

  -- Input tables for _GetPBTContributors
  CREATE TABLE #Instances 
  (
    cid         ClassID       NULL,
    id          ObjID         NULL,
    rCorr       INT           NOT NULL,             
    className   NVARCHAR(24)  NULL,
    prodVer     NVARCHAR(85)  NULL,     
    instid      NVARCHAR(412) NULL,
    uinstid     NVARCHAR(412) NULL,
    objPath     NVARCHAR(255) NULL,
    maxRes      INT           NULL,
    errorId     INT           NULL
  ) 

  -- Output table for _GetPBTContributors
  CREATE TABLE #PBTChildren
  (
    parent_cid  ClassID   NOT NULL,
    parent_id   ObjID     NOT NULL,
    child_cid   ClassID   NOT NULL,
    child_id    ObjID     NOT NULL,
    alertstate  INT       NOT NULL
  )

  DECLARE @desc TABLE
  ( 
    ServiceCID  NVARCHAR(4)   NULL,
    ServiceID   INT           NULL,
    cid         NVARCHAR(4)   NULL,
    id          INT           NULL,
    gen         INT           NULL,    
    phy_cid     NVARCHAR(4)   NULL,
    phy_id      INT           NULL,
    done        TINYINT       NULL DEFAULT 0,  
    respath     NVARCHAR(700) NULL       
  )
  
  CREATE TABLE #event  
  (  
    ServiceCID  ClassID       NULL,  
    ServiceID   ObjID         NULL,  
    ObjectCID   ClassID       NULL,  
    ObjectID    ObjID         NULL,  
    cid         ClassID       NULL,   
    id          ObjID         NULL,  
    AlertState  INT           NULL,  
    Priority    INT           NULL,  
    ParentCID   ClassID       NULL,  
    ParentID    ObjID         NULL,  
    EventTime   DATETIME      NULL,
    respath     NVARCHAR(500) NULL    
  )
  
  CREATE INDEX event_index  
    ON #event(cid,id)   

  DECLARE @chev TABLE 
  (    
    ServiceCID    NVARCHAR(4)   NULL,
    ServiceID     INT           NULL,
    cid           NVARCHAR(4)   NULL,
    id            INT           NULL,
    EventCID      NVARCHAR(4)   NULL,
    AlertState    INT           NULL,
    Priority      INT           NULL,
    EventPriority INT           NULL,
    ChildCID      NVARCHAR(4)   NULL,
    ChildID       INT           NULL,
    ParentCID     NVARCHAR(4)   NULL,
    ParentID      INT           NULL,
    gen           INT           NULL,  
    respath       NVARCHAR(500) NULL,      
    RootCause     INT           DEFAULT 0
  )

  CREATE TABLE #chev    
  (    
    ServiceCID  ClassID       NULL,
    ServiceID   ObjID         NULL,
    ChildCID    ClassID       NULL,
    ChildID     ObjID         NULL,
    respath     NVARCHAR(500) NULL,      
    RootCause   INT           DEFAULT 0  
  )

  DECLARE @RootCauseLobs TABLE
  (    
    ServiceCID  NVARCHAR(4) NULL,
    ServiceID   INT         NULL,
    cid         NVARCHAR(4) NULL,
    id          INT         NULL
  )
 
  DECLARE
    @cmd        NVARCHAR(3000),
    @crlid      INT,
    @crlidpath  NVARCHAR(2000),
    @err        INT,
    @gen        INT,
    @now        NVARCHAR(40),
    @max_gen    INT,
    @rc         INT
  
  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
    RAISERROR('%s: GetRootCauseResources: Starting', 0, 1, @now) WITH NOWAIT  
  END 
  
  -- If optional input table exists it takes precedence over cid,id parameters  
  IF OBJECT_ID('tempdb..#RootCauseServices') IS NOT NULL   
  BEGIN  
    INSERT INTO @RootCauseLobs (ServiceCID,ServiceID,cid,id)   
    SELECT DISTINCT ServiceCID, ServiceID, cid, id FROM #RootCauseServices
  END  
  ELSE  
  BEGIN  
    INSERT INTO @RootCauseLobs (ServiceCID,ServiceID,cid,id) VALUES (NULL, NULL, @cid, @id)  
  END  

  -- First get Critical Resource List info so it can be excluded from the @desc table
  SELECT @crlid=crl.id, @crlidpath=o.idpath+'/%' 
  FROM   CRL_Container AS crl
    JOIN link          AS l WITH (NOLOCK) ON l.dst_cid = crl.cid AND l.dst_id = crl.id  
    JOIN ObjPathCache  AS o WITH (NOLOCK) ON o.cid = 'LOB'       AND o.id = crl.id 
  WHERE 
        l.deleted   = 0 
    AND l.link_type = 'LOBC'  
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
    RAISERROR('%s: GetRootCauseResources: CRL_Container id=%d, path=''%s''', 0, 1, @now, @crlid, @crlidpath) WITH NOWAIT  
  END

  -- Insert the parent resource  
  INSERT INTO @desc (ServiceCID, ServiceID, cid, id, gen, phy_cid, phy_id,respath)  
    SELECT DISTINCT r.ServiceCID, r.ServiceID, 'LOB', l.dst_id, 0, l.phy_cid, l.phy_id, o.idpath
    FROM   lob_link         AS l WITH (NOLOCK)
      JOIN @RootCauseLobs   AS r               ON r.id  = l.dst_id  
      JOIN ObjPathCache     AS o WITH (NOLOCK) ON o.cid = 'LOB' AND o.id = l.dst_id AND o.idpath NOT LIKE @crlidpath
      JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON r.cid = 'LOB' AND r.id = v.id     AND v._AlertStateID <> 1           
    WHERE l.src_id = 0   
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
    RAISERROR('%s: GetRootCauseResources: Found %d parent resource(s)', 0, 1, @now, @rc) WITH NOWAIT  
    IF @verbose > 1 BEGIN SELECT 'parent resources', * FROM @desc  END 
  END
    
  -- Get the root cause Exceptions and Messages on the specified LOB 
  INSERT INTO @chev (ServiceCID, ServiceID, cid, id, EventCID, AlertState, Priority, EventPriority, ChildCID, ChildID, ParentCID, ParentID, gen, respath)  
  SELECT DISTINCT d.ServiceCID, d.ServiceID,a.event_cid, d.id, a.event_cid, a.alertstate, a.priority, a.priority, d.cid, d.id, NULL,0, -1 ,d.respath   --- the parent is null and 0 was d.cid and d.id
    FROM   @desc         AS d  
      JOIN ObjAlertCount AS a WITH (NOLOCK) ON a.cid = d.cid AND a.id = d.id  
  WHERE
        d.gen = 0  
    AND a.event_cid IN ('EXCP','MESG')
    AND a.nviolations > 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
    RAISERROR('%s: GetRootCauseResources: Found %d child event(s) from exceptions and messages', 0, 1, @now, @rc) WITH NOWAIT  
  END

  -- Get the root cause Child events on the specified LOB
  INSERT INTO @chev (ServiceCID, ServiceID, cid, id, EventCID, AlertState, Priority, EventPriority, ChildCID, ChildID, ParentCID, ParentID, gen, respath)  
  SELECT DISTINCT d.ServiceCID, d.ServiceID,a.event_cid, c.id, c._EventCID, c._AlertState, c._Priority, c._EventPriority, c._ChildCID, c._ChildID, c._ParentCID, c._ParentID, 0,d.respath +'/LOB:' + CAST (c._ChildID AS NVARCHAR(9)) 
  FROM   ChildEvent_C  AS c WITH (NOLOCK) 
    JOIN @desc         AS d               ON d.cid = c._ParentCID AND d.id = c._ParentID  
    JOIN ObjAlertCount AS a WITH (NOLOCK) ON a.cid = c._ParentCID AND a.id = c._ParentID AND a.alertstate = c._AlertState AND a.priority = c._Priority 
  WHERE 
        c.deleted = 0  
    AND d.gen     = 0  
    AND a.event_cid IN ('CHEV')
    AND a.nviolations > 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
    RAISERROR('%s: GetRootCauseResources: Found %d root cause child event(s)', 0, 1, @now, @rc) WITH NOWAIT  
    IF @verbose >= 2 
    BEGIN  
      SELECT 'chev on LOB', * FROM @chev 
    END 
  END

  -- Go down the hierarchy find all root cause events
  SELECT @rc = COUNT(*) FROM @chev 
  SET @gen = 0
  WHILE @rc > 0  
  BEGIN  
    -- Get CHEVs on the parent LOB  
    INSERT INTO @chev (ServiceCID, ServiceID, cid, id, EventCID, AlertState, Priority, EventPriority, ChildCID, ChildID, ParentCID, ParentID, gen, respath)  
    SELECT DISTINCT c_prev.ServiceCID, c_prev.ServiceID,'CHEV', c_prev.ChildID, c._EventCID, c._AlertState, c._Priority, c._EventPriority, c._ChildCID, c._ChildID, c._ParentCID, c._ParentID, @gen  + 1, c_prev.respath + '/'+ c._ChildCID + ':' +CAST (c._ChildID AS NVARCHAR(9)) 
    FROM   ChildEvent_C  AS c WITH (NOLOCK) 
      JOIN @chev         AS c_prev          ON c_prev.ChildCID = c._ParentCID    AND c_prev.ChildID = c._ParentID  
                                           AND c_prev.AlertState = c._AlertState AND c_prev.Priority = c._Priority  
      JOIN ObjAlertCount AS a WITH (NOLOCK) ON a.cid = c._ParentCID              AND a.id = c._ParentID 
                                           AND a.alertstate = c._AlertState      AND a.priority = c._Priority 
    WHERE 
          c.deleted       = 0   
      AND c_prev.EventCID = 'CHEV'  
      AND c_prev.gen      = @gen 
      AND a.event_cid     = 'CHEV'
      AND a.nviolations   > 0 
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose > 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Found %d root cause event(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT  
      IF @verbose >= 2
      BEGIN 
        SELECT 'chev on LOB', * FROM @chev 
      END 
    END  

    SET @gen = @gen + 1  

  END  

  IF @verbose >= 1
  BEGIN  
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SELECT @rc = COUNT(*) FROM @chev
    RAISERROR('%s: GetRootCauseResources: Final number of root cause events = %d', 0, 1, @now, @rc) WITH NOWAIT  
    IF @verbose >= 2 
    BEGIN  
      SELECT 'final chev on LOB', * FROM @chev 
    END 
  END

  -- If caller asks for events - look for events on impacted resources added to the @chev tables
  IF @getEvents = 1
  BEGIN
    -- Get exceptions on the LOB  
    INSERT INTO #event (ServiceCID, ServiceID, ObjectCID, ObjectID, cid, id, AlertState, Priority, ParentCID, ParentID, EventTime,respath)  
    SELECT DISTINCT c.ServiceCID, c.ServiceID,c.ChildCID, c.ChildID, 'EXCP', e.id, e._AlertState, e._Priority, c.ChildCID, c.ChildID, e._DATE, NULL ---c.respath --+ '/' + c.ChildCID + ':' +CAST (c.ChildID  AS VARCHAR(9))
    FROM   @chev         AS c 
      JOIN ObjAlertCount AS oac WITH (NOLOCK) ON oac.alertstate = c.AlertState AND oac.priority = c.EventPriority 
                                             AND oac.cid = c.ChildCID          AND oac.id = c.ChildID
      JOIN Exception_V   AS e   WITH (NOLOCK) ON e._ParentCID = c.ChildCID     AND e._ParentID = c.ChildID  
                                             AND c.AlertState = e._AlertState  AND c.EventPriority = e._Priority  
    WHERE  
          e.deleted       = 0  
      AND c.EventCID      = 'EXCP'  
      AND oac.nviolations > 0 
      AND oac.event_cid   = 'EXCP'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT       
    IF @verbose >= 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Looked up details for %d directly contained exception(s)', 0, 1, @now, @rc) WITH NOWAIT  
      IF @verbose >= 2 
      BEGIN  
        SELECT 'excp on LOB', * FROM #event 
      END 
    END  

    -- Get exceptions on the physical resources  
    INSERT INTO #event (ServiceCID, ServiceID,ObjectCID, ObjectID, cid, id, AlertState, Priority, ParentCID, ParentID, EventTime,respath)  
    SELECT DISTINCT c.ServiceCID, c.ServiceID, c.ChildCID, c.ChildID, 'EXCP', e.id, e._AlertState, e._Priority, l.phy_cid, l.phy_id, e._DATE , NULL ---c.respath 
    FROM (SELECT DISTINCT ServiceCID    AS ServiceCID,
                          ServiceID     AS ServiceID, 
                          ChildCID      AS ChildCID,
                          ChildID       AS ChildID,
                          ParentCID     AS ParentCID,
                          ParentID      AS ParentID,
                          EventCID      AS EventCID,
                          AlertState    AS AlertState,
                          Priority      AS Priority,
                          EventPriority AS EventPriority
      FROM @chev)        AS c  
      JOIN ObjAlertCount AS oac WITH (NOLOCK) ON oac.alertstate = c.AlertState AND oac.priority = c.EventPriority
                                             AND oac.cid = c.ChildCID          AND oac.id = c.ChildID 
      JOIN lob_link      AS l   WITH (NOLOCK) ON l.dst_id = c.ChildID          AND l.src_id = c.ParentID
      JOIN Exception_V   AS e   WITH (NOLOCK) ON e._ParentCID = l.phy_cid      AND e._ParentID = l.phy_id  
                                             AND c.AlertState = e._AlertState  AND c.EventPriority = e._Priority  
      WHERE 
            e.deleted  = 0  
        AND c.EventCID = 'EXCP'  
        AND l.phy_cid IS NOT NULL  
        AND oac.nviolations > 0      
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT       
    IF @verbose >= 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Looked up details for %d indirectly contained exception(s)', 0, 1, @now, @rc) WITH NOWAIT  
      IF @verbose >= 2  
      BEGIN  
        SELECT 'excp on phy resources', * FROM #event 
      END 
    END    

    -- Get messages  
    INSERT INTO #event (ServiceCID, ServiceID,ObjectCID, ObjectID, cid, id, AlertState, Priority, ParentCID, ParentID, EventTime,respath)  
    SELECT DISTINCT chev.ServiceCID, chev.ServiceID, c.cid, c.id, 'MESG', m.id, m._AlertState, m._Priority, m._ParentCID, m._ParentID, v._DATE, chev.respath  
    FROM   Message_V                   AS m    WITH (NOLOCK)  
      JOIN CurrentMessageAlertStatePri AS c    WITH (NOLOCK) ON c.mesg_id = m.id  
      JOIN @chev                       AS chev               ON chev.ChildCID = c.cid AND chev.ChildID = c.id  
                                                            AND chev.EventPriority = c.Priority
      JOIN Message_V                   AS v    WITH (NOLOCK) ON v.id = c.mesg_id  
      LEFT OUTER JOIN #event           AS et   WITH (NOLOCK) ON et.cid = 'MESG'       AND et.id = m.id  
    WHERE 
          chev.EventCID = 'MESG'  
      AND et.cid        IS NULL  
      AND m.deleted     = 0   
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Looked up details for %d message(s)', 0, 1, @now, @rc) WITH NOWAIT  
      IF @verbose >= 2 
      BEGIN  
        SELECT 'mesg on LOB', * FROM #event 
      END 
    END  

    -- Remove Exceptions From Source of a Shorcut if it has not tripped counters
    DELETE e      
      FROM   #event           AS e
        JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.id = e.ObjectID AND v._LinkedCID = 'LOB' 
        JOIN Exception_V      AS x WITH (NOLOCK) ON x.id = e.id
    WHERE 
          e.ObjectCID   = 'LOB'
      AND e.ObjectID    IS NOT NULL
      AND e.ParentCID   = 'LOB'
      AND e.ParentID    IS NOT NULL
      AND v.deleted     = 0
      AND e.ObjectID    <> e.ParentID
      AND x._flags & 16 = 16  
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Removed %d mismatching shortcut exception(s) from result set', 0, 1, @now, @rc) WITH NOWAIT  
    END  

    DELETE e      
      FROM   #event           AS e
        JOIN LineOfBusiness_V AS v WITH (NOLOCK) ON v.id = e.ObjectID AND v._LinkedCID = 'LOB' 
        JOIN Message_V        AS x WITH (NOLOCK) ON x.id = e.id
    WHERE 
          e.ObjectCID   = 'LOB'
      AND e.ObjectID    IS NOT NULL
      AND e.ParentCID   = 'LOB'
      AND e.ParentID    IS NOT NULL
      AND v.deleted     = 0
      AND e.ObjectID    <> e.ParentID
      AND x._flags & 16 = 16  
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN  
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)  
      RAISERROR('%s: GetRootCauseResources: Removed %d mismatching shortcut message(s) from result set', 0, 1, @now, @rc) WITH NOWAIT  
    END  

  END

  -- Update RootCause flag in @chev.  If an exception or message is found on a LOB, set the flag = 1
  UPDATE c SET RootCause = 1
  FROM @chev AS c
    JOIN (SELECT ChildCID AS ChildCID, ChildID AS ChildID FROM @chev AS c2 
           WHERE c2.EventCID = 'EXCP' OR c2.EventCID = 'MESG'
          ) AS derived ON c.ChildCID = derived.ChildCID AND c.ChildID = derived.ChildID
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: GetRootCauseResources: Set RootCause on %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Now it is time to determine what the PBT resources, if any, are impacting the resource 
  INSERT INTO #Instances  
  SELECT DISTINCT ChildCID, ChildID, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL
  FROM @chev as res 
  WHERE  
        res.ChildCID='LOB'  
    AND res.EventCID='EXCP' 
     OR res.EventCID='MESG'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: GetRootCauseResources: Computed set of %d instance(s) to lookup PBT information for', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  EXEC _GetPBTContributors NULL, NULL, @verbose = @verbose
  
  -- Stick the results in the @chev table to finish processing 
  INSERT INTO @chev 
  SELECT DISTINCT
    c.ServiceCID,
    c.ServiceID,
    LTRIM(RTRIM(c.ParentCID)),
    c.ParentID,
    'PBT',
    p.alertstate,
    NULL,
    NULL,
    LTRIM(RTRIM(p.child_cid)),
    p.child_id,
    NULL,
    NULL,
    NULL,
    c.respath + '/' + RTRIM(LTRIM(p.child_cid)) + ':' + CAST(p.child_id as NVARCHAR(9)) AS respath,
    1 AS RootCause
  FROM   #PBTChildren AS p WITH (NOLOCK) 
    JOIN @chev        AS c ON c.ChildCID = p.parent_cid AND c.ChildID = p.parent_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: GetRootCauseResources: Copied %d result row(s) for further processing', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Move the  @chev table to the temp table #chev
  INSERT INTO #chev 
  SELECT DISTINCT ServiceCID, ServiceID, ChildCID, ChildID, respath, RootCause
  FROM @chev AS c 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: GetRootCauseResources: Copied %d result row(s) from table variable to temp table', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- If input tables have been specified then insert impacted resource data
  IF @rscTableName IS NOT NULL 
  BEGIN 
    SET @cmd = 'INSERT INTO ' + @rscTableName + '( ServiceCID, ServiceID, ChildCID, ChildID, respath, RootCause) ' + 
    'SELECT DISTINCT ServiceCID, ServiceID, ChildCID, ChildID, respath, RootCause FROM #chev AS c '
    EXEC (@cmd)
  END  
  ELSE
  BEGIN 
    SELECT DISTINCT ServiceCID, ServiceID, ChildCID, ChildID, respath, RootCause FROM #chev 
  END 

  -- If input tables have been specified then insert root cause events data
  IF @getEvents = 1
  BEGIN 
    IF @eventTableName IS NOT NULL 
    BEGIN 
      SET @cmd = 'INSERT INTO ' + @eventTableName + '( cid, id) SELECT DISTINCT cid,id FROM #event '
      EXEC(@cmd)
    END 
    ELSE 
      SELECT DISTINCT cid, id FROM #event
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: GetRootCauseResources: Finished', 0, 1, @now) WITH NOWAIT
  END

  RETURN 0  
GO
