-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/rodm_object_discovery_backend.sql
-- SID: 1.12.1.42
-- Date Modified: 04/12/13 12:39:53
--
-- Module Description
--   Defines the RODM_object_discovery_backend stored procedure

USE Object
GO

--------------------------------------------------------------------------------------------
-- RODM_object_discovery_backend
PRINT 'Creating procedure RODM_object_discovery_backend'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODM_object_discovery_backend' AND type = 'P')
  DROP PROCEDURE RODM_object_discovery_backend
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/rodm_object_discovery_backend.sql
-- SID: 1.12.1.42
-- Date Modified: 04/12/13 12:39:53
-- Called at the tail-end of the RODM discovery process.
-- This procedure performs the aspects of the discovery that must occur locally
-- to the central Object database.
CREATE PROCEDURE RODM_object_discovery_backend
  @verbose TINYINT = 0
AS
  DECLARE
    @cid          ClassID,
    @dst_cid      ClassID,
    @err          INT,
    @id           ObjID,
    @iObjectGMFF  ObjID,
    @last_run_id  INT,
    @link_type    LinkType,
    @msg_txt      NVARCHAR(255),
    @now          NVARCHAR(40),
    @rc           INT,
    @sql          NVARCHAR(1024),
    @src_cid      ClassID,
    @table        NVARCHAR(30),
    @tcount       INT,
    @ts           TIMESTAMP,
    @whereClause  NVARCHAR(500)
  
  -- Standard transaction logic
  SET @tcount = @@TRANCOUNT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Starting. TRANCOUNT=%d', 0, 1, @now, @tcount) WITH NOWAIT
  END
  
  -- perform garbage collection of links for deleted physical RODM objects
  --   retain the logical links as they will be undeleted if the physical object is rediscovered.
  IF EXISTS
  (
    SELECT 1
    FROM link AS l
    JOIN isa_chain AS i ON i.cid = l.dst_cid
    WHERE i.base_cid = 'RDMO' AND l.deleted=1 AND l.link_type <> 'LOBL'
  )
  BEGIN

    DECLARE gc_c CURSOR FOR
      SELECT DISTINCT link_type,src_cid,dst_cid
      FROM link AS l
      JOIN isa_chain AS i ON i.cid = l.dst_cid
      WHERE i.base_cid = 'RDMO' AND l.deleted=1 AND l.link_type <> 'LOBL'
    OPEN gc_c
    FETCH NEXT FROM gc_c INTO @link_type, @src_cid, @dst_cid
    WHILE @@FETCH_STATUS = 0
    BEGIN
      SET @table = @src_cid + '_' + @dst_cid + '_' + @link_type 
      SET @sql =
        'DELETE FROM i ' +
        'FROM ' + @table + ' AS i ' +
        'JOIN link ON link.link_type = ''' + @link_type + ''' AND ' +
                      'link.src_cid = ''' + @src_cid + ''' AND ' +
                      'link.dst_cid = ''' + @dst_cid + ''' AND ' +
                      'link.src_id = i.src_id AND ' + 
                      'link.dst_id = i.dst_id ' +
        'WHERE link.deleted=1 '
      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: RODM_object_discovery_backend: Error %d performing garbage collection on table %s, continuing', 3, 1, @now, @err, @table) WITH NOWAIT
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from %s marked for deletion', 0, 1, @now, @rc, @table) WITH NOWAIT
      END
      FETCH NEXT FROM gc_c INTO @link_type, @src_cid, @dst_cid
    END
    DEALLOCATE gc_c
  END

  -- Data Integrity Fix -- defect 120405 -- remove LU resources, 
  --   note these deletes should be removed once LUs are handled properly.
  DELETE FROM RODMManagedLink WHERE src_cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMManagedLink where src_cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMManagedLink where src_cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM RODMManagedLink WHERE dst_cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMManagedLink where dst_cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMManagedLink where dst_cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM RODMManagedObject WHERE cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMManagedObject where cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMManagedObject where cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM RODMLoad..RODMManagedLink WHERE src_cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMLoad..RODMManagedLink where src_cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMLoad..RODMManagedLink where src_cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM RODMLoad..RODMManagedLink WHERE dst_cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMLoad..RODMManagedLink where dst_cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMLoad..RODMManagedLink where dst_cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM RODMLoad..RODMManagedObject WHERE cid = 'LU'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from RODMLoad..RODMManagedObject where cid=''LU'', continuing', 3, 1, @now, @err) WITH NOWAIT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from RODMLoad..RODMManagedObject where cid=''LU''', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF @tcount = 0
    BEGIN TRANSACTION RODM_odb
  ELSE
    SAVE TRANSACTION RODM_odb

  -- Update information for existing RODM objects
  -- TRUNCATE TABLE Object..RODMNetworksToRegister  -- start with a clean slate
  -- If the resource is undeleted (deleted = 1 -> 0) then it needs to be registered
  INSERT INTO Object..RODMNetworksToRegister
  SELECT o.cid, o.id 
  FROM Object..RODMManagedObject AS o WITH (TABLOCKX)
  JOIN RODMLoad..RODMManagedObject AS r
       ON o._RODMClass = r._RODMClass AND o._RODMMyName = r._RODMMyName
  LEFT OUTER JOIN Object..RODMNetworksToRegister rntr ON rntr.cid=o.cid AND rntr.id=o.id
  WHERE r.DiscoverAction = 'UPDATE' AND o.deleted = 1 AND r.cid IS NOT NULL AND r.cid <> '' AND rntr.cid IS NULL

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into Object..RODMNetworksToRegister', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into Object..RODMNetworksToRegister', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Prepare a list of deleted objects being rediscovered
  -- that need to be have their PropagationSlot bucket counts
  -- reset.
  CREATE TABLE #objectsToReset (cid ClassID NOT NULL, id ObjID NOT NULL)
  INSERT INTO #objectsToReset (cid, id)
    SELECT DISTINCT o.cid, o.id
    FROM  Object..PropagationSlot          AS ps
    JOIN  Object..RODMManagedObject        AS o ON o.cid = ps.OwnerCID AND o.id = ps.OwnerID
    INNER JOIN RODMLoad..RODMManagedObject AS r ON r._RODMClass  = o._RODMClass 
                                               AND r._RODMMyName = o._RODMMyName 
    WHERE ps.SettingIndex = 0
      AND NOT (ps.Critical=0 AND ps.High=0 AND ps.Medium=0 AND ps.Low=0 AND ps.Ignore=0)
      AND r.DiscoverAction = 'UPDATE'
      AND o.deleted = 1

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d computing objects to reset', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Computed %d row(s) of objects to reset', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF EXISTS (SELECT 1 FROM #objectsToReset) 
  BEGIN

    -- Add objects to PACacheReload table for PropagationAgent to process
    INSERT INTO PACacheReload (cid, id)
      SELECT cid, id FROM #objectsToReset

    -- Add event to cause propagation agent to process ObjectsToRefresh table
    INSERT INTO StagedCMD (cmd, cid, id)
    VALUES ('EXEC pa_processPACacheReloads', 'PA', 1)

  END

  -- Update RODMManagedObject with loaded values, if undeleting object then reset additional columns:
  --   _AlertStateID       to Green
  --   _RegistrationStatus to register
  --   _DisplayStatus      to satisfactory

  /*
     note: The _DisplayStatus field is changed to use 0 instead of 129 in order to process a new
           MESG in the RODMStateChangeMap table. This will force a new message on the object
           that will support text indicating that the object was previously deleted and now
           rediscovered.
  */

  UPDATE o
  SET
    mtime                = CASE WHEN o.mtime > r.mtime THEN o.mtime ELSE r.mtime               END,
    deleted              = 0,
    _AlertStateID        = CASE WHEN o.deleted = 1     THEN 1       ELSE o._AlertStateID       END,
    _RegistrationStatus  = CASE 
                             WHEN o.deleted = 1 AND o.cid <> 'GMFA' THEN 1
                             ELSE o._RegistrationStatus 
                           END,
    _DisplayResourceName = r._DisplayResourceName,
    _DisplayStatus       = CASE WHEN o.deleted = 1     THEN 0       ELSE o._DisplayStatus      END
  FROM       Object..RODMManagedObject   AS o
  INNER JOIN RODMLoad..RODMManagedObject AS r ON r._RODMClass  = o._RODMClass 
                                             AND r._RODMMyName = o._RODMMyName 
  WHERE 
        r.DiscoverAction = 'UPDATE'

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating Object..RODMManagedObject', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) in Object..RODMManagedObject', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Use the RODMAutoRegistrationMap table to update the RegistrationStatus column of the
  -- RODMManagedObject records that will be inserted into the Object database.
  -- NOTE: when the distributed database support is added the RODMAutoRegistrationMap
  -- table will probably have to be moved to the RODMLoad database.
  DECLARE regc CURSOR FOR
    SELECT
      whereClause
    FROM
      RODMAutoRegistrationMap
    WHERE
      enabled = 1
    ORDER BY
      priority DESC
  OPEN regc
  FETCH NEXT FROM regc INTO @whereClause
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC(
    'UPDATE rmo ' +
    'SET _RegistrationStatus = 1 ' +
    'FROM RODMLoad..RODMManagedObject AS rmo ' +
    'WHERE ' + @whereClause + ' ' +
      'AND rmo.DiscoverAction = ''ADD'' ' +
      'AND rmo.cid <> ''GMFA'' '
    )

    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Error %d updating RODMLoad..RODMManagedObject.RegistrationStatus', 11, 1, @now, @err) WITH NOWAIT
      DEALLOCATE regc
      ROLLBACK TRANSACTION RODM_odb
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) for RODMLoad..RODMManagedObject.RegistrationStatus', 0, 1, @now, @rc) WITH NOWAIT
    END

    FETCH NEXT FROM regc INTO @whereClause
  END
  DEALLOCATE regc

  -- Maintain table of networks to re-register based on rows that are being added that
  -- have _RegistrationStatus = 1.  The table will be processed by a SQL job that's kicked
  -- off at the end of this procedure.
  INSERT INTO Object..RODMNetworksToRegister
    SELECT DISTINCT 'SNNT', snnt.id
    FROM SNANetwork_V AS snnt
    JOIN containment AS c ON c.dst_cid = 'SNNT' AND c.dst_id = snnt.id
    JOIN RODMLoad..RODMManagedObject AS rmo ON rmo._NetworkID = snnt.name
    LEFT OUTER JOIN Object..RODMNetworksToRegister rntr ON rntr.cid='SNNT' AND rntr.id=snnt.id
  WHERE
    c.deleted = 0
    AND rmo.DiscoverAction = 'ADD'
    AND rmo._RegistrationStatus = 1
    AND rntr.cid IS NULL

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into RODMNetworksToRegister', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into RODMNetworksToRegister', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Add newly-discovered RODM objects
  INSERT INTO Object..RODMManagedObject (name, cid, _RODMClass, _RODMMyName, _NetworkID, _DisplayResourceName, _RegistrationStatus)
    SELECT name, cid, _RODMClass, _RODMMyName, _NetworkID, _DisplayResourceName, _RegistrationStatus
    FROM RODMLoad..RODMManagedObject
    WHERE DiscoverAction = 'ADD'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    EXEC Object..asisp_getTBSMMessage 'SNNT',0,108,'RO','English','RODM_object_discovery_backend',@msg_txt OUTPUT
    SELECT @msg_txt
    IF @msg_txt IS NOT NULL 
      RAISERROR(@msg_txt,0,1)
    ELSE
      RAISERROR('%s: RODM_object_discovery_backend: Error adding RODMManagedObject records', 0, 1)
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into Object..RODMManagedObject', 0, 1, @now, @rc) WITH NOWAIT
  END

/* jcp... */
  /*
     "re-discover" previously deleted objects - they have objects in RODMManagedObject but no links. their
     RODMManagedLink rows will say DiscoverAction = 'UPDATE' - let's changed that to 'ADD' so the links
     get added
  */

  /* first delete the rows from Object..RODMManagedLink so the inserts won't fail */

  DELETE FROM o
  FROM Object..RODMManagedLink AS o
    JOIN RODMLoad..RODMManagedLink AS rml
         ON  o.link_type  = rml.link_type
         AND o.SrcClass   = rml.SrcClass
         AND o.SrcMyName  = rml.SrcMyName
         AND o.SrcField   = rml.SrcField
         AND o.DstClass   = rml.DstClass
         AND o.DstMyName  = rml.DstMyName
         AND o.DstField   = rml.DstField
    JOIN Object..RODMManagedObject AS so ON so._RODMClass = rml.SrcClass AND so._RODMMyName = rml.SrcMyName
    JOIN Object..RODMManagedObject AS do ON do._RODMClass = rml.DstClass AND do._RODMMyName = rml.DstMyName
    LEFT OUTER JOIN link ON link.link_type = rml.link_type AND
                            link.src_cid = rml.src_cid AND link.src_id = so.id AND
                            link.dst_cid = rml.dst_cid AND link.dst_id = do.id
    WHERE rml.DiscoverAction = 'UPDATE' AND link.link_type IS NULL

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from Object..RODMManagedLink', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from Object..RODMManagedLink', 0, 1, @now, @rc) WITH NOWAIT
  END

  /* now change the discover action */


  UPDATE rml SET DiscoverAction = 'ADD'
    FROM RODMLoad..RODMManagedLink rml
    JOIN Object..RODMManagedObject AS so ON so._RODMClass = rml.SrcClass AND so._RODMMyName = rml.SrcMyName
    JOIN Object..RODMManagedObject AS do ON do._RODMClass = rml.DstClass AND do._RODMMyName = rml.DstMyName
    LEFT OUTER JOIN link ON link.link_type = rml.link_type AND
                            link.src_cid = rml.src_cid AND link.src_id = so.id AND
                            link.dst_cid = rml.dst_cid AND link.dst_id = do.id
    WHERE rml.DiscoverAction = 'UPDATE' AND link.link_type IS NULL

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating RODMLoad..RODMManagedLink for UPDATE->ADD', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) in RODMLoad..RODMManagedLink for UPDATE->ADD', 0, 1, @now, @rc) WITH NOWAIT
  END

/* ...jcp */

  -- INSERT links FROM networks to top-level objects
  UPDATE snnt SET deleted = 0
    FROM SNANetwork_C AS snnt
      JOIN RODMLoad..RODMManagedObject AS rmo ON rmo.cid = snnt.cid AND rmo.id = snnt.id
    WHERE
      snnt.deleted = 1
      AND rmo.DiscoverAction <> 'DELETE'

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d undeleting SNANetwork_C', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Undeleted %d row(s) in SNANetwork_C', 0, 1, @now, @rc) WITH NOWAIT
  END

  DECLARE linkc INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(rmo.cid) FROM RODMManagedObject AS rmo JOIN RODMTopLevelClass AS tlc ON tlc.cid = rmo.cid
  OPEN linkc
  FETCH NEXT FROM linkc INTO @cid
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @table = 'SNNT_' + @cid + '_PHYC'
    EXEC(
    'INSERT INTO ' + @table + '(src_id, dst_id) ' +
      'SELECT net.id, ormo.id ' +
      'FROM ' +
        'RODMLoad..RODMManagedObject AS rmo ' +
        'JOIN Object..RODMManagedObject AS ormo ON ormo._RODMClass = rmo._RODMClass AND ormo._RODMMyName = rmo._RODMMyName ' +
        'JOIN SNANetwork_V AS net ON net.name = rmo._NetworkID ' +
        'JOIN containment AS c ON c.dst_cid = ''SNNT'' AND c.dst_id = net.id ' +
      'WHERE ' +
        'rmo.cid = ''' + @cid + ''' ' +
        'AND c.deleted = 0 ' +
        'AND NOT EXISTS ' +
        '( ' +
          'SELECT 1 FROM ' + @table + ' AS prev ' +
          'WHERE prev.src_id = net.id AND prev.dst_id = rmo.id ' +
        ') ' +
        'AND rmo.DiscoverAction <> ''DELETE'' '
    )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      EXEC Object..asisp_getTBSMMessage 'ROOT',0,109,'RO','English','RODM_object_discovery_backend',@msg_txt OUTPUT
      SELECT @msg_txt
      IF @msg_txt IS NOT NULL 
        RAISERROR(@msg_txt,0,1)
      ELSE
        RAISERROR('%s: RODM_object_discovery_backend: Error adding top level links for %s', 0, 1, @cid)
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into %s', 0, 1, @now, @rc, @table) WITH NOWAIT
    END
    FETCH NEXT FROM linkc INTO @cid
  END
  DEALLOCATE linkc

  -- link GMFHS Aggregate instances to the GMFHS Aggregates folder 
  --   Use the first GMFHS Aggregates folder 
  SELECT  @iObjectGMFF = MIN( o.id )
  FROM       GMFHSAggregateFolder_V AS o
  INNER JOIN link                   AS l ON l.dst_id = o.id
  WHERE
        l.deleted = 0
    AND l.dst_cid = 'GMFF'

  IF @iObjectGMFF IS NOT NULL -- Only link GMFHS Aggregate instances if a GMFHS Aggregates folder exists
  BEGIN
    -- First undelete any deleted GMFA resources because they are being rediscovered.
    UPDATE o 
    SET
      deleted = 0 
    FROM       RODMManagedObject           AS o
    INNER JOIN RODMLoad..RODMManagedObject AS l ON l.cid = o.cid 
                                               AND l.id  = o.id
    WHERE
          o.cid             =  'GMFA'
      AND o.deleted         =  1
      AND l.DiscoverAction <> 'DELETE'

    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Error %d undeleting GMFA', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION RODM_odb
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Undeleted %d GMFA row(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Create new links from folder to aggregates if they do not exist.
    INSERT INTO GMFF_GMFA_GENC (       src_id, dst_id )
    SELECT                       @iObjectGMFF,   o.id
    FROM       RODMManagedObject           AS o
    INNER JOIN RODMLoad..RODMManagedObject AS rl ON rl._RODMClass  = o._RODMClass
                                                AND rl._RODMMyName = o._RODMMyName
    WHERE
          o.cid              =  'GMFA'
      AND rl.DiscoverAction <> 'DELETE'
      AND NOT EXISTS ( SELECT 1
                       FROM link AS l
                       WHERE
                             l.deleted = 0
                         AND l.src_cid = 'GMFF'
                         AND l.src_id  = @iObjectGMFF
                         AND l.dst_cid = o.cid
                         AND l.dst_id  = o.id
                     )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      EXEC Object..asisp_getTBSMMessage 'GMFA', 0, 110, 'RO', 'English', 'RODM_object_discovery_backend', @msg_txt OUTPUT
      SELECT @msg_txt
      IF @msg_txt IS NOT NULL 
        RAISERROR( @msg_txt, 0, 1)
      ELSE
        RAISERROR( 'RODM_object_discovery_backend: Error adding GMFF->GMFA links', 0, 1)
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into GMFF_GMFA_GENC', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

--jms
  -- Insert registration records for GMFA objects. This has to be done after the links have been
  -- created, so it can't be immediately after the insert for the SNNT objects above.
  INSERT INTO Object..RODMNetworksToRegister
    SELECT DISTINCT 'GMFF', gmff.id
    FROM GMFHSAggregateFolder_V AS gmff
    JOIN containment AS c ON c.dst_cid = 'GMFF' AND c.dst_id = gmff.id
    JOIN link AS l ON l.src_cid = 'GMFF' AND l.src_id = gmff.id
    JOIN RODM..RODMManagedObject AS rmo1 ON rmo1.cid = l.dst_cid AND rmo1.id = l.dst_id
      -- we need to join the RODMLoad..RODMManagedObject table because it has the correct DiscoverAction.
      -- The DiscoverAction value is not carried forward to the RODM version of the table.
    JOIN RODMLoad..RODMManagedObject AS rmo2 ON rmo1._RODMClass  = rmo2._RODMClass
                                                AND rmo1._RODMMyName = rmo2._RODMMyName
    LEFT OUTER JOIN Object..RODMNetworksToRegister rntr ON rntr.cid='GMFF' AND rntr.id=gmff.id
  WHERE
    c.deleted = 0
    AND l.deleted = 0
    AND rmo1.cid = 'GMFA'
    AND rmo2.DiscoverAction = 'ADD'
    AND rmo1._RegistrationStatus = 1
    AND rntr.cid IS NULL

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into Object..RODMNetworksToRegister for GMFA ADD', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into Object..RODMNetworksToRegister for GMFA ADD', 0, 1, @now, @rc) WITH NOWAIT
  END
--jms

  -- Insert registration records for GMFA objects that were associated with a different operating
  -- system than the last time they were registered.  They need to get re-registered after the
  -- switch to the new operating system.
  INSERT INTO Object..RODMNetworksToRegister
    SELECT DISTINCT 'GMFF', gmff.id
    FROM GMFHSAggregateFolder_V AS gmff
    JOIN containment AS c ON c.dst_cid = 'GMFF' AND c.dst_id = gmff.id
    JOIN link AS l ON l.src_cid = 'GMFF' AND l.src_id = gmff.id
    JOIN RODM..RODMManagedObject AS rmo1 ON rmo1.cid = l.dst_cid AND rmo1.id = l.dst_id
      -- we need to join the RODMLoad..RODMManagedObject table because it has the correct DiscoverAction.
      -- The DiscoverAction value is not carried forward to the RODM version of the table.
    JOIN RODMLoad..RODMManagedObject AS rmo2 ON rmo1._RODMClass  = rmo2._RODMClass
                                                AND rmo1._RODMMyName = rmo2._RODMMyName
    LEFT OUTER JOIN Object..RODMNetworksToRegister rntr ON rntr.cid='GMFF' AND rntr.id=gmff.id
  WHERE
    c.deleted = 0
    AND l.deleted = 0
    AND rmo1.cid = 'GMFA'
    AND rmo2.DiscoverAction = 'UPDATE'
    AND rmo1._RegistrationStatus = 1
    AND rntr.cid IS NULL
    AND rmo1._LastMonitoringOSID <> gmff._MonitoringOSID
    AND gmff._MonitoringOSID <> 0

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into Object..RODMNetworksToRegister for GMFA UPDATE', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into Object..RODMNetworksToRegister for GMFA UPDATE', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Update informaton on existing RODM links
  UPDATE o SET mtime = CURRENT_TIMESTAMP
    FROM Object..RODMManagedLink AS o
    JOIN RODMLoad..RODMManagedLink AS r
         ON  o.link_type  = r.link_type
         AND o.SrcClass   = r.SrcClass
         AND o.SrcMyName  = r.SrcMyName
         AND o.SrcField   = r.SrcField
         AND o.DstClass   = r.DstClass
         AND o.DstMyName  = r.DstMyName
       AND o.DstField   = r.DstField
    WHERE r.DiscoverAction = 'UPDATE'

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating Object..RODMManagedLink.mtime', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated Object..RODMManagedLink.mtime for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- ADD newly-discovered RODM links - we don't USE the object id's FROM the RODMLoad database because the id
  -- field IN the RODMManagedObject TABLE uses the identity property AND may have gotten a different object id
  -- WHEN the object was created IN this database
  INSERT INTO Object..RODMManagedLink (link_type, SrcClass, SrcMyName, DstClass, DstMyName, SrcField, DstField,
                                       src_cid, src_id, dst_cid, dst_id)
     SELECT r.link_type, r.SrcClass, r.SrcMyName, r.DstClass, r.DstMyName, r.SrcField, r.DstField,
            r.src_cid, so.id, r.dst_cid, do.id
     FROM RODMLoad..RODMManagedLink AS r
       JOIN Object..RODMManagedObject AS so ON so._RODMClass = r.SrcClass AND so._RODMMyName = r.SrcMyName
       JOIN Object..RODMManagedObject AS do ON do._RODMClass = r.DstClass AND do._RODMMyName = r.DstMyName
     WHERE r.DiscoverAction = 'ADD'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    EXEC Object..asisp_getTBSMMessage 'ROOT',0,111,'RO','English','RODM_object_discovery_backend',@msg_txt OUTPUT
    SELECT @msg_txt
    IF @msg_txt IS NOT NULL 
      RAISERROR(@msg_txt, 11, 1)
    ELSE
      RAISERROR('%s: RODM_object_discovery_backend: Error inserting RODMManagedLink', 0, 1)
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into Object..RODMManagedLink', 0, 1, @now, @rc) WITH NOWAIT
  END


  -- Negate outstanding child events
  --  DECLARE  @src_cid    ClassID
  DECLARE  @src_id     ObjID
  --  DECLARE  @dst_cid    ClassID
  DECLARE  @dst_id     ObjID
  DECLARE  @EvtCID     ClassID
  DECLARE  @EvtDetail  NVARCHAR(255)
  DECLARE  @EvtCount   INT
  DECLARE  @Priority   INT
  DECLARE  @AlertState INT

  /* defect 120910: have the propagation server call 
     asisp_fixObjectState through pa_updateObjectPriority */

  -- Delete RODM links that no longer exist
  DELETE o
    FROM Object..RODMManagedLink AS o
    JOIN RODMLoad..RODMManagedLink AS r
         ON  o.link_type  = r.link_type
         AND o.SrcClass   = r.SrcClass
         AND o.SrcMyName  = r.SrcMyName
         AND o.SrcField   = r.SrcField
         AND o.DstClass   = r.DstClass
         AND o.DstMyName  = r.DstMyName
         AND o.DstField   = r.DstField
    WHERE r.DiscoverAction = 'DELETE'

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d deleting from Object..RODMManagedLink', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Deleted %d row(s) from Object..RODMManagedLink', 0, 1, @now, @rc) WITH NOWAIT
  END

  /* 120910: prepare PendingDelete batch to handle deleting objects to do things such as:
       marking LOBs as deleted
       clearing events on LOBs
       creating notification records for console
  */
  CREATE TABLE #backend_ts ( cid ClassID NOT NULL, id ObjID NOT NULL, timestamp NOT NULL )
  INSERT INTO #backend_ts (cid, id) VALUES ( 'ROOT', 0 )
  SELECT @ts = timestamp FROM #backend_ts

  INSERT INTO PendingDelete(timestamp, cid, id, generation, cno, flag_propagation, in_lob)
    SELECT
      @ts, 
      rdmo.cid, 
      rdmo.id, 
      1, 
      o.cno, 
      o.flag_propagation, 
      CASE WHEN EXISTS (SELECT 1 FROM link AS l3 WHERE l3.link_type = 'LOBL' AND l3.dst_cid = rdmo.cid AND l3.dst_id = rdmo.id AND l3.deleted = 0) THEN 1 ELSE 0 END
    FROM RODMManagedObject AS rdmo
    JOIN obj_class AS o ON o.cid = rdmo.cid
    WHERE rdmo.deleted = 0
    AND NOT EXISTS
    ( 
      SELECT 1
      FROM RODMManagedLink AS l
      WHERE
          l.dst_cid = rdmo.cid
      AND l.dst_id  = rdmo.id
    ) 
    AND NOT EXISTS
    (
      SELECT 1
      FROM link AS l2
      WHERE
            l2.dst_cid = rdmo.cid
        AND l2.dst_id  = rdmo.id
        AND l2.deleted = 0
        AND l2.link_type <> 'LOBL'
    ) 

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into PendingDelete', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) from PendingDelete', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Delete "top-level" RODM objects that appear under the SNNT/GMFF objects but were not rediscovered
  INSERT INTO PendingDelete(timestamp, cid, id, generation, cno, flag_propagation, in_lob)
    SELECT DISTINCT
      @ts,
      rmo.cid,
      rmo.id,
      0,
      o.cno,
      o.flag_propagation,
      CASE WHEN EXISTS (SELECT 1 FROM link AS l3 WHERE l3.link_type = 'LOBL' AND l3.dst_cid = rmo.cid AND l3.dst_id = rmo.id AND l3.deleted = 0) THEN 1 ELSE 0 END
    FROM RODMManagedObject             AS rmo 
--    JOIN RODMTopLevelClass           AS tlc   ON tlc.cid = rmo.cid -- I don't think this is really necessary and it excludes GMFAs
      JOIN link                        AS l     ON l.dst_cid = rmo.cid AND l.dst_id = rmo.id
      JOIN RODMLoad..RODMManagedObject AS rlrmo ON rlrmo._RODMClass = rmo._RODMClass AND rlrmo._RODMMyName = rmo._RODMMyName
      JOIN obj_class                   AS o WITH (NOLOCK) ON o.cid = rmo.cid
    WHERE l.deleted = 0 AND l.src_cid IN ('SNNT','GMFF') AND rlrmo.DiscoverAction = 'DELETE'

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d inserting into PendingDelete', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Inserted %d row(s) into PendingDelete', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set the display status to 1, which causes an "object undiscovered" message to 
  -- be generated by the RODMManagedObject trigger, based on the RODMStateChangeMap
  -- table.  This message may be matched by a "object rediscovered" message later.
  UPDATE o SET
    _DisplayStatus = 1
  FROM RODMManagedObject           AS o 
  JOIN RODMLoad..RODMManagedObject AS r ON r._RODMClass = o._RODMClass AND r._RODMMyName = o._RODMMyName
  WHERE r.DiscoverAction = 'DELETE' AND r.cid IS NOT NULL AND r.cid <> ''

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating RODMManagedObject for unrediscovered objects', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) in RODMManagedObject for unrediscovered objects', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Any resource that does not have any links to a parent is considered deleted.
  --   deleted means RODMManagedObject.deleted = 1
  UPDATE o SET
    deleted = 1
  FROM RODMManagedObject AS o 
  WHERE 
        o.deleted = 0
    AND NOT EXISTS
    ( 
      SELECT 1
      FROM RODMManagedLink AS l
      WHERE
            l.dst_cid = o.cid
        AND l.dst_id  = o.id
    )
    AND NOT EXISTS
    (
      SELECT 1
      FROM link AS l2
      WHERE
            l2.dst_cid = o.cid
        AND l2.dst_id  = o.id
        AND l2.deleted = 0
        AND l2.link_type <> 'LOBL'
    ) 

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating RODMManagedObject for unlinked objects', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) in RODMManagedObject for unlinked objects', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Remember which LOBs are being resurrected so we can clear them.

  CREATE TABLE #lobs_renew (id INT)

  INSERT INTO #lobs_renew
  SELECT l.src_id
  FROM       link              AS l
  INNER JOIN RODMManagedObject AS o ON o.cid = l.dst_cid
                                   AND o.id = l.dst_id
  INNER JOIN LineOfBusiness_C  AS lob ON lob.id = l.src_id
  WHERE
        l.deleted   = 1
    AND l.link_type = 'LOBL'
    AND l.src_cid   = 'LOB'
    AND o.deleted   = 0
    AND lob.deleted = 0

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d searching for LOBs to relink', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Found %d row(s) of LOBs to relink', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- On rediscovery deleted LOBL links are undeleted if the RODM object exists.
  --   update relies on tU_link_LOB_any_LOBL to update additional tables and columns
  UPDATE l SET
    deleted = 0
  FROM       link              AS l
  INNER JOIN RODMManagedObject AS o ON o.cid = l.dst_cid
                                   AND o.id = l.dst_id
  INNER JOIN LineOfBusiness_C  AS lob ON lob.id = l.src_id
  WHERE
        l.deleted   = 1
    AND l.link_type = 'LOBL'
    AND l.src_cid   = 'LOB'
    AND o.deleted   = 0
    AND lob.deleted = 0  -- if the object is deleted from the LOB from the console,
                         -- lob.deleted will be 1.  Without this condition, we will
                         -- set link.deleted=0 - the object does not reappear in the
                         -- LOB but we're doing unnecessary work which may confuse
                         -- things.  It's best to check to see if the object is really
                         -- still appearing in the LOB.

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Error %d updating link for relinked LOBs', 11, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION RODM_odb
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Updated %d row(s) in link for relinked LOBs', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION RODM_odb

  -- run job to re-register networks if newly-added objects
  -- are to be automatically registered

  IF EXISTS (SELECT 1 FROM RODMNetworksToRegister)
  BEGIN
    -- Lookup the last executed update of the ObjPathCache table
    SELECT @last_run_id = ISNULL(MAX(id), 0) FROM ObjPathCacheRun
    EXEC _SetSystemConfiguration ROOT, 0, 'RODM', 'LastObjPathCacheRunId', @last_run_id, 'INT'
    
    EXEC msdb..sp_start_job 'Automatically Register Newly-discovered RODM Objects'
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: RODM_object_discovery_backend: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
