-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
--
-- Module Description
--   RODMManagedObject table definition

USE RODM
GO

-- Define the table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedObject' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMManagedObject'
  CREATE TABLE RODMManagedObject
  (
    cid                           ClassID  NOT NULL CONSTRAINT RODMMO_cid_DF DEFAULT (''),
    id                            ObjID  NOT NULL IDENTITY(1,1),
    name                          ObjName  NOT NULL CONSTRAINT RODMMO_name_DF DEFAULT (''),
    revision                      INT  NULL CONSTRAINT RODMMO_revision_DF DEFAULT (0),
    nchild                        INT  NULL CONSTRAINT RODMMO_nchild_DF DEFAULT (0),
    description                   NVARCHAR(255) NULL CONSTRAINT RODMMO_description_DF DEFAULT (NULL),
    ctime                         DATETIME  NULL CONSTRAINT RODMMO_ctime_DF DEFAULT (CURRENT_TIMESTAMP),
    mtime                         DATETIME  NULL CONSTRAINT RODMMO_mtime_DF DEFAULT (CURRENT_TIMESTAMP),
    userid                        UserID  NOT NULL CONSTRAINT RODMMO_userid_DF DEFAULT ('SYSADM'),
    deleted                       TINYINT  NULL CONSTRAINT RODMMO_deleted_DF DEFAULT (0),
    _AlertStateID                 INT  NULL CONSTRAINT RODMMO_AlertStateID_DF DEFAULT (1),
    _PriorityID                   INT  NULL CONSTRAINT RODMMO_PriorityID_DF DEFAULT (6),
    _CurrentStateID               INT  NULL CONSTRAINT RODMMO_CurrentStateID_DF DEFAULT (0),
    _DesiredStateID               INT  NULL CONSTRAINT RODMMO_DesiredStateID_DF DEFAULT (0),
    _HasOwnershipNote             TINYINT  NULL CONSTRAINT RODMMO_HasOwnershipNoteID_DF DEFAULT (0),
    DiscoverAction                NVARCHAR(8) NULL CONSTRAINT RODMMO_DiscoverAction_DF DEFAULT (NULL),
    _RODMClass                    VARCHAR(60) NOT NULL,  -- #TSQL
    _RODMMyName                   VARCHAR(255) NOT NULL, -- #TSQL
    _NetworkID                    NVARCHAR(40) NULL CONSTRAINT RODMMO_NetworkID_DF DEFAULT (''),
    _RegistrationStatus           TINYINT  NULL CONSTRAINT RODMMO_RegistrationStatus_DF DEFAULT (0),
    _DisplayResourceName          NVARCHAR(255) NULL CONSTRAINT RODMMO_DisplayResourceName_DF DEFAULT (NULL),
    _DisplayStatus                INT  NULL CONSTRAINT RODMMO_DisplayStatus_DF DEFAULT (132),
    _operationalState             TINYINT  NULL CONSTRAINT RODMMO_operationalState_DF DEFAULT (0xff),
    _usageState                   TINYINT  NULL CONSTRAINT RODMMO_usageState_DF DEFAULT (0xff),
    _administrativeState          TINYINT  NULL CONSTRAINT RODMMO_adminstrativeState_DF DEFAULT (0xff),
    _availabilityStatus           TINYINT  NULL CONSTRAINT RODMMO_availabilityState_DF DEFAULT (0x00),
    _proceduralStatus             TINYINT  NULL CONSTRAINT RODMMO_proceduralState_DF DEFAULT (0x00),
    _unknownStatus                TINYINT  NULL CONSTRAINT RODMMO_unknownState_DF DEFAULT (0x01),
    _nativeStatus                 TINYINT  NULL CONSTRAINT RODMMO_nativeState_DF DEFAULT (0xff),
    _flags                        INT  NOT NULL CONSTRAINT RODMMO_flags_DF DEFAULT (0),
    _HostEventTime                DATETIME NULL DEFAULT(NULL),
    _HostSeqNo                    NVARCHAR(50) NULL DEFAULT(NULL)
  )
  ALTER TABLE RODMManagedObject ADD CONSTRAINT RODMMO_PK PRIMARY KEY CLUSTERED(_RODMClass, _RODMMyName)
END
GO

-- Add unique constraint on cid, id columns
IF NOT EXISTS (SELECT 1 FROM sysobjects WHERE name = 'RODMMO_cid_id_UQ')
  ALTER TABLE RODMManagedObject ADD CONSTRAINT RODMMO_cid_id_UQ UNIQUE (cid, id)

-- Drop obsolete columns _ChildEventMatrixID and _ExceptionMatrixID

IF EXISTS (SELECT 1 FROM sysobjects WHERE name = 'RODMMO_ChildEventMatrixID_DF')
  ALTER TABLE RODMManagedObject DROP CONSTRAINT RODMMO_ChildEventMatrixID_DF

IF EXISTS (SELECT 1 FROM syscolumns WHERE OBJECT_NAME(id) = 'RODMManagedObject' AND name = '_ChildEventMatrixID')
  ALTER TABLE RODMManagedObject DROP COLUMN _ChildEventMatrixID

IF EXISTS (SELECT 1 FROM sysobjects WHERE name = 'RODMMO_ExceptionMatrixID_DF')
  ALTER TABLE RODMManagedObject DROP CONSTRAINT RODMMO_ExceptionMatrixID_DF

IF EXISTS (SELECT 1 FROM syscolumns WHERE OBJECT_NAME(id) = 'RODMManagedObject' AND name = '_ExceptionMatrixID')
  ALTER TABLE RODMManagedObject DROP COLUMN _ExceptionMatrixID

IF EXISTS (SELECT 1 FROM sysobjects WHERE name = 'RODMMO_PriorityID_DF' AND type = 'D')
  ALTER TABLE RODMManagedObject DROP CONSTRAINT RODMMO_PriorityID_DF -- see defect 121509

IF NOT EXISTS (SELECT 1 FROM syscolumns WHERE name = '_HostEventTime' AND id = OBJECT_ID('RODMManagedObject'))
  ALTER TABLE RODMManagedObject ADD _HostEventTime DATETIME NULL DEFAULT NULL

IF NOT EXISTS (SELECT 1 FROM syscolumns WHERE name = '_HostSeqNo' AND id = OBJECT_ID('RODMManagedObject'))
  ALTER TABLE RODMManagedObject ADD _HostSeqNo NVARCHAR(50) NULL DEFAULT NULL
GO

--
-- Triggers
--

--------------------------------------------------------------------------------
-- tI_RODMManagedObject

IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tI_RODMManagedObject' AND type = 'TR')
  DROP TRIGGER tI_RODMManagedObject
GO

PRINT 'Creating trigger tI_RODMManagedObject'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
-- Module description
--   Insert trigger used to apply default object priority based on the cid.
--   Ordinarily, this is done via a default constraint on the table but
--   a default constraint does not let us have different defaults for each
--   RODM class.

CREATE TRIGGER tI_RODMManagedObject ON RODMManagedObject FOR INSERT WITH APPEND
AS
  UPDATE rmo SET _PriorityID = ISNULL(ev.value, 6)
  FROM RODMManagedObject rmo
  JOIN inserted i ON i.id = rmo.id
  LEFT OUTER JOIN Object..AttributeDefault ad ON ad.cid = rmo.cid AND ad.aname = 'PriorityID'
  LEFT OUTER JOIN Object..enumeration enum ON enum.name = 'ObjectPriority'
  LEFT OUTER JOIN Object..enum_value ev ON CONVERT(NVARCHAR(510), ev.value) = ad.value AND enum.eid = ev.eid
GO

--------------------------------------------------------------------------------
-- tU_RODMManagedObject
IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tU_RODMManagedObject' AND type = 'TR')
  DROP TRIGGER tU_RODMManagedObject
GO

PRINT 'Creating trigger tU_RODMManagedObject'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
-- Module Description
--   update trigger used on the RODMManagedObject table.
--   After asisp_processStagedRODMStateMsgs updates an object's attributes, this
--   trigger will create exceptions and/or messages based on the new changes.
--   Additionally on logical deletion (deleted=0 -> 1) it will unregister the RODM resource, 
--     and prevent further registrations
CREATE TRIGGER tU_RODMManagedObject ON RODMManagedObject FOR UPDATE
AS
  --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': Start'
  DECLARE @id ObjID
  DECLARE @cid ClassID
  DECLARE @name NVARCHAR(255)
  DECLARE @whereClause NVARCHAR(255)
  DECLARE @exclusive TINYINT
  DECLARE @eventCID ClassID
  DECLARE @eventName NVARCHAR(54)
  DECLARE @eventText NVARCHAR(255)
  DECLARE @alertState INT
  DECLARE @priority INT
  DECLARE @oldAlertState INT
  DECLARE @oldPriority INT
  DECLARE @done TINYINT
  DECLARE @rows1 INT
  DECLARE @rows2 INT
  DECLARE @time DATETIME
  DECLARE @sql NVARCHAR(2048)
  DECLARE @_RODMMyName NVARCHAR(255)
  DECLARE @cname NVARCHAR(24)

  DECLARE @oldOperationalState  TINYINT
  DECLARE @oldUsageState  TINYINT
  DECLARE @oldAdministrativeState TINYINT
  DECLARE @oldAvailabilityStatus TINYINT
  DECLARE @oldProceduralStatus TINYINT
  DECLARE @oldUnknownStatus TINYINT
  DECLARE @oldNativeStatus TINYINT
  DECLARE @oldDisplayStatus INT

  DECLARE @newOperationalState  TINYINT
  DECLARE @newUsageState TINYINT
  DECLARE @newAdministrativeState TINYINT
  DECLARE @newAvailabilityStatus TINYINT
  DECLARE @newProceduralStatus TINYINT
  DECLARE @newUnknownStatus TINYINT
  DECLARE @newNativeStatus TINYINT
  DECLARE @newDisplayStatus INT

  DECLARE @HostEventTime DATETIME
  DECLARE @HostSeqNo NVARCHAR(50)

  DECLARE @stateName NVARCHAR(32)
  DECLARE @stateNum INT
  DECLARE @oldValue INT
  DECLARE @newValue INT

  --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': Checking FOR updated columns'
  IF UPDATE(_operationalState) 
     OR UPDATE(_usageState)
     OR UPDATE(_administrativeState)
     OR UPDATE(_availabilityStatus)
     OR UPDATE(_proceduralStatus)
     OR UPDATE(_unknownStatus)
     OR UPDATE(_nativeStatus) 
     OR UPDATE(_DisplayStatus) 
  BEGIN
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': CREATE a temp TABLE to hold the actions that will be performed. '
    -- CREATE a temp TABLE to hold the actions that will be performed. 
    SELECT CAST('' AS NVARCHAR(2048)) AS action INTO #events

    -- copy the inserted/deleted tables to temporary tables that can be used IN dynamic SQL statements
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': SELECT * INTO #new FROM inserted'
    SELECT * INTO #new FROM inserted
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': SELECT * INTO #old FROM deleted'
    SELECT * INTO #old FROM deleted

    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': declaring outerCursor'
    DECLARE outerCursor INSENSITIVE CURSOR FOR 
      SELECT 
        new.id,
        new.cid,
        new.name,
        new._RODMMyName,
        old._operationalState,
        old._usageState,
        old._administrativeState,
        old._availabilityStatus,
        old._proceduralStatus,
        old._unknownStatus,
        old._nativeStatus,
        old._DisplayStatus,
        new._operationalState,
        new._usageState,
        new._administrativeState,
        new._availabilityStatus,
        new._proceduralStatus,
        new._unknownStatus,
        new._nativeStatus,
        new._DisplayStatus,
        new._HostEventTime,
        new._HostSeqNo
      FROM inserted AS new JOIN deleted AS old ON old.id = new.id 
      WHERE 
            new.cid <> ''
        AND new.cid IS NOT NULL
        AND new.id  IS NOT NULL
        AND ( old._operationalState    <> new._operationalState
           OR old._usageState          <> new._usageState
           OR old._administrativeState <> new._administrativeState
           OR old._availabilityStatus  <> new._availabilityStatus
           OR old._proceduralStatus    <> new._proceduralStatus
           OR old._unknownStatus       <> new._unknownStatus
           OR old._nativeStatus        <> new._nativeStatus
           OR old._DisplayStatus       <> new._DisplayStatus )
    OPEN outerCursor
    FETCH NEXT FROM outerCursor INTO @id, @cid, @name, @_RODMMyName,
       @oldOperationalState, @oldUsageState, @oldAdministrativeState, @oldAvailabilityStatus, @oldProceduralStatus, @oldUnknownStatus, @oldNativeStatus, @oldDisplayStatus,
       @newOperationalState, @newUsageState, @newAdministrativeState, @newAvailabilityStatus, @newProceduralStatus, @newUnknownStatus, @newNativeStatus, @newDisplayStatus,
       @HostEventTime, @HostSeqNo
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': cid=' + @cid + ' id=' + CAST(@id AS NVARCHAR) + ' name=' + @name
      SELECT @cname = cname FROM obj_class WHERE cid = @cid

      SET @stateNum = 1
      WHILE @stateNum <= 8 BEGIN
        IF @stateNum = 1 BEGIN
          SET @stateName = 'operationalState'
          SET @oldValue = @oldOperationalState
          SET @newValue = @newOperationalState
        END
        ELSE IF @stateNum = 2 BEGIN
          SET @stateName = 'usageState'
          SET @oldValue = @oldUsageState
          SET @newValue = @newUsageState
        END
        ELSE IF @stateNum = 3 BEGIN
          SET @stateName = 'administrativeState'
          SET @oldValue = @oldAdministrativeState
          SET @newValue = @newAdministrativeState
        END
        ELSE IF @stateNum = 4 BEGIN
          SET @stateName = 'availabilityStatus'
          SET @oldValue = @oldAvailabilityStatus
          SET @newValue = @newAvailabilityStatus
        END
        ELSE IF @stateNum = 5 BEGIN
          SET @stateName = 'proceduralStatus'
          SET @oldValue = @oldProceduralStatus
          SET @newValue = @newProceduralStatus
        END
        ELSE IF @stateNum = 6 BEGIN
          SET @stateName = 'unknownStatus'
          SET @oldValue = @oldUnknownStatus
          SET @newValue = @newUnknownStatus
        END
        ELSE IF @stateNum = 7 BEGIN
          SET @stateName = 'nativeStatus'
          SET @oldValue = @oldNativeStatus
          SET @newValue = @newNativeStatus
        END
        ELSE IF @stateNum = 8 BEGIN
          SET @stateName = 'DisplayStatus'
          SET @oldValue = @oldDisplayStatus
          SET @newValue = @newDisplayStatus
        END

        IF @oldValue <> @newValue
        BEGIN
          SET @done = 0
          DECLARE innerCursor INSENSITIVE CURSOR FOR
            SELECT whereClause, exclusive, eventCID, eventName,  eventText, alertState, priority
            FROM RODMStateChangeMap
            WHERE enabled = 1 AND stateVar = @stateName
            ORDER BY salience DESC
          OPEN innerCursor
          FETCH NEXT FROM innerCursor INTO @whereClause, @exclusive, @eventCID, @eventName, 
                                           @eventText, @alertState, @priority
          WHILE @@FETCH_STATUS <> -1 AND @done = 0
          BEGIN
            -- Perform substitution of variables FROM instance data (e.g. - %name%)
            SELECT @eventName = replace(@eventName, '%name%', @name)
            SELECT @eventText = replace(@eventText, '%name%', @name)

            SELECT @eventName = replace(@eventName, '%cid%', @cid)
            SELECT @eventText = replace(@eventText, '%cid%', @cid)

            SELECT @eventName = replace(@eventName, '%cname%', @cname)
            SELECT @eventText = replace(@eventText, '%cname%', @cname)

            SELECT @eventName = replace(@eventName, '%RODMName%', @_RODMMyName)
            SELECT @eventText = replace(@eventText, '%RODMName%', @_RODMMyName)

            SELECT @eventName = replace(@eventName, '%oldValue%', CAST(@oldValue AS NVARCHAR))
            SELECT @eventText = replace(@eventText, '%oldValue%', CAST(@oldValue AS NVARCHAR))

            SELECT @eventName = replace(@eventName, '%newValue%', CAST(@newValue AS NVARCHAR))
            SELECT @eventText = replace(@eventText, '%newValue%', CAST(@newValue AS NVARCHAR))

            SELECT @eventName = replace(@eventName, '%stateName%', @stateName)
            SELECT @eventText = replace(@eventText, '%stateName%', @stateName)

            SELECT @rows1 = COUNT(*) FROM #events

            IF @HostEventTime IS NULL
              SELECT @time = CURRENT_TIMESTAMP
            ELSE
              SELECT @time = @HostEventTime

            DECLARE @timetemp NVARCHAR(64)
            SET @timetemp = CONVERT(NVARCHAR, @time, 121)

            IF @eventCID = 'EXCP'
            BEGIN
              SELECT @sql = 'INSERT #events SELECT ''EXEC Object..asisp_createEXCP ' + 
                            + '@name='''''        + @eventName + ''''''
                            + ',@parent_cid=''''' + @cid + ''''''
                            + ',@parent_id='      + CAST(@id AS NVARCHAR)
                            + ',@DATE='''''       + @timetemp + ''''''
                            + ',@TIME='''''       + @timetemp + ''''''
                            + ',@EXCP_NAME='''''  + @eventName + ''''''
                            + ',@EXCP_CD='''''    + @eventText + ''''''
            END
            ELSE
            BEGIN
              SELECT @sql = 'INSERT #events SELECT ''EXEC Object..asisp_createMESG ' + 
                            + '@name='''''        + @eventName + ''''''
                            + ',@parent_cid=''''' + @cid + ''''''
                            + ',@parent_id='      + CAST(@id AS NVARCHAR)
                            + ',@DATE='''''       + @timetemp + ''''''
                            + ',@TIME='''''       + @timetemp + ''''''
                            + ',@MESSAGE='''''    + @eventName + ''''''
                            + ',@REASON_CD='''''  + @eventText + ''''''
            END

            IF @HostSeqNo IS NOT NULL
              SELECT @sql = @sql + ',@SEQ_NO=''''' + @HostSeqNo + ''''''

            IF @alertState IS NOT NULL
              SELECT @sql = @sql + ',@AlertState=' + CAST(@alertState AS NVARCHAR)

            IF @priority IS NOT NULL
              SELECT @sql = @sql + ',@Priority=' + CAST(@priority AS NVARCHAR)

            SELECT @sql = @sql + ''' FROM #new new, #old old WHERE ' + @whereClause + 
                                 ' AND new._RODMMyName = ''' + @_RODMMyName + ''' AND old._RODMMyName = ''' + @_RODMMyName + ''''
            --PRINT @sql
            EXEC(@sql)

            SELECT @rows2 = COUNT(*) FROM #events

            IF @rows2 > @rows1 AND @exclusive = 1 SET @done = 1

            FETCH NEXT FROM innerCursor INTO @whereClause, @exclusive, @eventCID, @eventName, 
                                             @eventText, @alertState, @priority
          END
          DEALLOCATE innerCursor
        END /* the VALUES are different */

        SET @stateNum = @stateNum + 1
      END /* FOR each state variable */

      FETCH NEXT FROM outerCursor INTO @id, @cid, @name, @_RODMMyName,
         @oldOperationalState,@oldUsageState,@oldAdministrativeState,@oldAvailabilityStatus,@oldProceduralStatus,@oldUnknownStatus,@oldNativeStatus,@oldDisplayStatus,
         @newOperationalState,@newUsageState,@newAdministrativeState,@newAvailabilityStatus,@newProceduralStatus,@newUnknownStatus,@newNativeStatus,@newDisplayStatus,
         @HostEventTime, @HostSeqNo
    END

    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': deallocating outerCursor'
    DEALLOCATE outerCursor
  END

  IF @rows1 > 0
  BEGIN
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': declaring eventCursor'
    DECLARE eventCursor CURSOR FOR SELECT * FROM #events
    OPEN eventCursor
    FETCH NEXT FROM eventCursor INTO @sql

    WHILE @@FETCH_STATUS <> -1
    BEGIN

      IF len(@sql) > 0
      BEGIN
        --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': sql=' + @sql
        --PRINT @sql
        EXEC(@sql)
      END

      FETCH NEXT FROM eventCursor INTO @sql
    END
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': deallocating eventCursor'
    DEALLOCATE eventCursor
  END

  -- reset _HostEventTime and _HostSeqNo if they were changed
  IF UPDATE(_HostEventTime) OR UPDATE(_HostSeqNo)
    UPDATE rdmo SET _HostEventTime = NULL, _HostSeqNo = NULL
      FROM RODMManagedObject rdmo
      JOIN inserted i ON i.id = rdmo.id
      WHERE i._HostEventTime IS NOT NULL OR i._HostSeqNo IS NOT NULL 

  -- deleted maintenance
  --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': start deleted section'

 
  -- unregister any registered RODM resource that is being logically deleted.
  --   bulk and rediscovery registrations are handled in RODM_object_discovery_backend 
  IF UPDATE( deleted )
  BEGIN
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': declaring cursorRodmDelete '
    -- Commenting out below cursor as all the steps within the cursor were commented out previously anyway
/*    DECLARE cursorRodmDelete CURSOR FOR 
    SELECT i.cid, i.id
    FROM       inserted AS i
    INNER JOIN deleted  AS d ON d.id = i.id 
    WHERE
          d.deleted             =  0
      AND i.deleted             =  1
      AND d._RegistrationStatus <> 0
    OPEN cursorRodmDelete
    FETCH NEXT FROM cursorRodmDelete INTO @cid, @id
    WHILE @@FETCH_STATUS <> -1
    BEGIN 
      --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': cid=' + @cid + ' id=' + CAST(@id AS NVARCHAR) 
      --EXEC Object.._RODMRegister @cid, @id, 0, 1, 0 -- The registration scope may be 3 (root and descendents), 
      -- but cannot unregister descendents because the descendents may be linked to other parents.
      --  DO NOT MAKE THE Object.._RODMRegister CALL FROM HERE, IT CAUSES THE PROPAGATION AGENT TO ABEND.
      FETCH NEXT FROM cursorRodmDelete INTO @cid, @id
    END
    CLOSE cursorRodmDelete
    DEALLOCATE cursorRodmDelete
*/
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': update mtime, _RegistrationStatus '
    -- mark _RegistrationStatus for no registration, update mtime to allow row to be deleted based on last mtime update.
    UPDATE RODMManagedObject
    SET
      mtime               = CURRENT_TIMESTAMP,
      _RegistrationStatus = CASE
                              WHEN o.cid = 'GMFA' THEN o._RegistrationStatus
                              ELSE 0
                            END
    FROM       RODMManagedObject AS o
    INNER JOIN inserted          AS i ON o.id = i.id
    INNER JOIN deleted           AS d ON d.id = i.id 
    WHERE
          d.deleted = 0
      AND i.deleted = 1
      
    -- Delete from propagation-related tables in case the same objects get rediscovered
    DELETE x FROM Object..CurrentMessageAlertStatePri AS x
    JOIN inserted AS i ON i.cid = x.cid AND i.id = x.id
    JOIN deleted  AS d ON d.cid = i.cid AND d.id = i.id
    WHERE d.deleted = 0
      AND i.deleted = 1

    DELETE x FROM Object..ObjAlertCount AS x
    JOIN inserted AS i ON i.cid = x.cid AND i.id = x.id
    JOIN deleted  AS d ON d.cid = i.cid AND d.id = i.id
    WHERE d.deleted = 0
      AND i.deleted = 1

    DELETE x FROM Object..RootCauseState AS x
    JOIN inserted AS i ON i.cid = x.cid AND i.id = x.id
    JOIN deleted  AS d ON d.cid = i.cid AND d.id = i.id
    WHERE d.deleted = 0
      AND i.deleted = 1

    DELETE x FROM Object..OpenEventCount AS x
    JOIN inserted AS i ON i.cid = x.cid AND i.id = x.id
    JOIN deleted  AS d ON d.cid = i.cid AND d.id = i.id
    WHERE d.deleted = 0
      AND i.deleted = 1

    DELETE x FROM Object..PropagationSlot AS x
    JOIN inserted AS i ON i.cid = x.OwnerCID AND i.id = x.OwnerID
    JOIN deleted  AS d ON d.cid = i.cid AND d.id = i.id
    WHERE d.deleted = 0
      AND i.deleted = 1
  END

  --PRINT 'tU_RODMManagedObject: ' + CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 113) + ': finish'
GO

--------------------------------------------------------------------------------
-- tD_RODMManagedObject
IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tD_RODMManagedObject' AND type = 'TR')
  DROP TRIGGER tD_RODMManagedObject
GO

PRINT 'Creating trigger tD_RODMManagedObject'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
-- Module Description
--   Delete TRIGGER used ON the RODMManagedObject TABLE to support cleanup of
--   old exceptions and messages applied to rodm objects on previous discoveries.
CREATE TRIGGER tD_RODMManagedObject ON RODMManagedObject FOR DELETE WITH APPEND
AS
  -- Delete old exceptions that should no longer be visible
  DELETE c FROM Object..Exception_C AS c
  WHERE EXISTS ( SELECT * FROM deleted AS d
                   JOIN Object..Exception_A AS ea ON d.cid = ea._ParentCID 
                   AND d.id = ea._ParentID 
                 WHERE c.id = ea.id
               ) 
 
  -- Delete old messages that should no longer be visible
  DELETE c FROM Object..Message_C AS c
  WHERE EXISTS ( SELECT * FROM deleted AS d
                   JOIN Object..Message_A AS ma ON d.cid = ma._ParentCID 
                     AND d.id = ma._ParentID 
                 WHERE c.id = ma.id
               )

GO

--------------------------------------------------------------------------------
-- tI_RODMManagedObject_Prop

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_RODMManagedObject_Prop' AND type = 'TR')
  DROP TRIGGER tI_RODMManagedObject_Prop
GO

PRINT 'Creating trigger tI_RODMManagedObject_Prop'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
CREATE TRIGGER tI_RODMManagedObject_Prop ON RODMManagedObject FOR INSERT WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  -- Create the ChildEvent arrays
  INSERT INTO Object..PropagationSlot (OwnerCID, OwnerID, MatrixIndex, SettingIndex, ArrayIndex,
    ResourceCID, ResourceID, Critical, High, Medium, Low, Ignore, tid)
  SELECT i.cid, i.id, 0, 0, 2,
    i.cid, i.id, 0, 0, 0, 0, 0, ISNULL(ps.id, 0)
  FROM inserted AS i
    LEFT OUTER JOIN Object..PropagationSlot AS ps ON ps.OwnerCID = i.cid AND ps.OwnerID = 0
      AND ps.SettingIndex = 1 AND ps.MatrixIndex = 0 AND ps.ArrayIndex = 2

  INSERT INTO Object..PropagationSlot (OwnerCID, OwnerID, MatrixIndex, SettingIndex, ArrayIndex,
    ResourceCID, ResourceID, Critical, High, Medium, Low, Ignore, tid)
  SELECT i.cid, i.id, 0, 0, 3,
    i.cid, i.id, 0, 0, 0, 0, 0, ISNULL(ps.id, 0)
  FROM inserted AS i
    LEFT OUTER JOIN Object..PropagationSlot AS ps ON ps.OwnerCID = i.cid AND ps.OwnerID = 0
      AND ps.SettingIndex = 1 AND ps.MatrixIndex = 0 AND ps.ArrayIndex = 3

  -- Create the Exception arrays
  INSERT INTO Object..PropagationSlot (OwnerCID, OwnerID, MatrixIndex, SettingIndex, ArrayIndex,
    ResourceCID, ResourceID, Critical, High, Medium, Low, Ignore, tid)
  SELECT i.cid, i.id, 1, 0, 2,
    i.cid, i.id, 0, 0, 0, 0, 0, ISNULL(ps.id, 0)
  FROM inserted AS i
    LEFT OUTER JOIN Object..PropagationSlot AS ps ON ps.OwnerCID = i.cid AND ps.OwnerID = 0
      AND ps.SettingIndex = 1 AND ps.MatrixIndex = 1 AND ps.ArrayIndex = 2

  INSERT INTO Object..PropagationSlot (OwnerCID, OwnerID, MatrixIndex, SettingIndex, ArrayIndex,
    ResourceCID, ResourceID, Critical, High, Medium, Low, Ignore, tid)
  SELECT i.cid, i.id, 1, 0, 3,
    i.cid, i.id, 0, 0, 0, 0, 0, ISNULL(ps.id, 0)
  FROM inserted AS i
    LEFT OUTER JOIN Object..PropagationSlot AS ps ON ps.OwnerCID = i.cid AND ps.OwnerID = 0
      AND ps.SettingIndex = 1 AND ps.MatrixIndex = 1 AND ps.ArrayIndex = 3

GO

--------------------------------------------------------------------------------
-- tD_RODMManagedObject_Prop

IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tD_RODMManagedObject_Prop' AND type = 'TR')
  DROP TRIGGER tD_RODMManagedObject_Prop
GO

PRINT 'Creating trigger tD_RODMManagedObject_Prop'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
-- Module description
--   Insert trigger used to apply default object priority based on the cid.
--   Ordinarily, this is done via a default constraint on the table but
--   a default constraint does not let us have different defaults for each
--   RODM class.

CREATE TRIGGER tD_RODMManagedObject_Prop ON RODMManagedObject FOR DELETE WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  -- Delete from PropagationSlot
  DELETE p
  FROM Object..PropagationSlot AS p
    JOIN deleted AS d ON d.id = p.OwnerID AND d.cid = p.OwnerCID 
  WHERE p.SettingIndex = 0

GO

--------------------------------------------------------------------------------
-- tI_RODMManagedObject_MNO2

IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tI_RODMManagedObject_MNO2' AND type = 'TR')
  DROP TRIGGER tI_RODMManagedObject_MNO2
GO

PRINT 'Creating trigger tI_RODMManagedObject_MNO2'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
CREATE TRIGGER tI_RODMManagedObject_MNO2 ON RODMManagedObject FOR INSERT WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  -- Insert into the ManagedObject table

  -- since the default for the _PriorityID column comes from AttributeDefault rather than
  -- a default constraint, we'll have to refer to that table.
  INSERT INTO Object..ManagedObject (cid, id, name, Priority)
  SELECT i.cid, i.id, i.name, COALESCE(i._PriorityID, ev.value, 6)
  FROM inserted i
  LEFT OUTER JOIN Object..AttributeDefault ad ON ad.cid = i.cid AND ad.aname = 'PriorityID'
  LEFT OUTER JOIN Object..enumeration enum ON enum.name = 'ObjectPriority'
  LEFT OUTER JOIN Object..enum_value ev ON CONVERT(NVARCHAR(510), ev.value) = ad.value AND enum.eid = ev.eid
  WHERE i.deleted = 0
GO

--------------------------------------------------------------------------------
-- tU_RODMManagedObject_MNO2

IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tU_RODMManagedObject_MNO2' AND type = 'TR')
  DROP TRIGGER tU_RODMManagedObject_MNO2
GO

PRINT 'Creating trigger tU_RODMManagedObject_MNO2'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
CREATE TRIGGER tU_RODMManagedObject_MNO2 ON RODMManagedObject FOR UPDATE WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  -- Handle updates to the deleted column
  IF UPDATE(deleted)
  BEGIN
    -- delete rows from ManagedObject
    DELETE m
    FROM Object..ManagedObject AS m
      JOIN inserted AS i ON i.id = m.id AND m.cid = i.cid
      JOIN deleted AS d ON i.id = d.id
    WHERE i.deleted = 1 AND d.deleted = 0

    -- Insert rows into ManagedObject
    INSERT INTO Object..ManagedObject (cid, id, name, Priority)
    SELECT i.cid, i.id, i.name, i._PriorityID
    FROM inserted AS i
      JOIN deleted AS d ON i.id = d.id
    WHERE i.deleted = 0 AND d.deleted = 1
  END

  -- Handle changes to the priority column
  IF UPDATE(_PriorityID)
  BEGIN
    UPDATE m
    SET Priority = i._PriorityID
    FROM Object..ManagedObject AS m
      JOIN inserted AS i ON i.id = m.id AND m.cid = i.cid
      JOIN deleted AS d ON d.id = i.id
    WHERE d._PriorityID <> i._PriorityID
      AND i._PriorityID <> m.Priority
  END

  -- Handle changes to the name column
  IF UPDATE(name)
  BEGIN
    UPDATE m
    SET name = i.name
    FROM Object..ManagedObject AS m
      JOIN inserted AS i ON i.id = m.id AND m.cid = i.cid
      JOIN deleted AS d ON d.id = i.id
    WHERE d.name <> i.name
      AND i.name <> m.name
  END
GO


--------------------------------------------------------------------------------
-- tD_RODMManagedObject_MNO2

IF EXISTS (SELECT * FROM RODM..sysobjects WHERE name = 'tD_RODMManagedObject_MNO2' AND type = 'TR')
  DROP TRIGGER tD_RODMManagedObject_MNO2
GO

PRINT 'Creating trigger tD_RODMManagedObject_MNO2'
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/tables/rodmmanagedobject.sql
-- SID: 1.4.4.19
-- Date Modified: 04/11/12 16:59:36
CREATE TRIGGER tD_RODMManagedObject_MNO2 ON RODMManagedObject FOR DELETE WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  -- Delete from ManagedObject
  DELETE m
  FROM Object..ManagedObject AS m
    JOIN deleted AS d ON d.id = m.id AND m.cid = d.cid
GO
